# Router object, invocation returns a function meant to dispatch  http requests.
Router = (options = {}) ->

# Required modules, all of them from standard node library, no external dependencies.	

  urlparse = require('url').parse
  querystring = require('querystring')
  fs       = require('fs')
  path_tools = require('path')
  spawn  = require('child_process').spawn
  domain = require 'domain'
  net = require 'net'
  http = require 'http'
      
# End of required modules

#In-memory stores and other variables

  nsr_sessions = {}

#End of In-memory stores and other variables

# Auxiliary functions.

  thousandSep = (num, sep = ",") ->
    return num.toString() unless num.toString().length > 3
    resp = num.toString().split('').reverse().join('').replace(/(\d{3})/g, "$1#{sep}").split('').reverse().join('')
    if resp.charAt(0) is sep then resp.slice(1) else resp
    
  padString = (stri, quantity, direction = "r", padchar = " ") ->
    stri = stri.toString() if stri.constructor.name is "Number"
    len = stri.length
    dif = quantity - len
    return stri if dif <= 0
    padstri = (padchar for n in [1..dif]).join ''
    if direction is "r" then "#{stri}#{padstri}" else "#{padstri}#{stri}"

  uuid = ->
    d = new Date().getTime()
    guid = "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, (c) ->
      r = (d + Math.random() * 16) % 16 | 0
      d = Math.floor(d / 16)
      ((if c is "x" then r else (r & 0x7 | 0x8))).toString 16
    )
    guid

  _isEmpty = (obj) ->
    Object.keys(obj).length is 0

  _extend = (obj_destiny, obj_src) ->
    for key, val of obj_src
      obj_destiny[key] = val
    obj_destiny

  _parsePattern = (pat) ->
    re = /\/:([A-Za-z0-9_]+)+/g
    m = pat.match(re)
    if m
      pars = (x.slice(2) for x in m)
      retpat = pat.replace(re, "/([A-Za-z0-9_\-]+)")
    else
      retpat = pat
      pars = null
    {pattern: retpat, params: pars}

  _make_request_wrapper = (cb) ->
    wrapper = (req, res) ->
      body = []
      contentType = 'application/x-www-form-urlencoded'
      if req.headers['content-type']
        contentType = req.headers['content-type']
      mp_index = contentType.indexOf('multipart/form-data')
      if req.method.toLowerCase() isnt 'get'
        req.setEncoding('binary') if (mp_index isnt -1)
        req.on 'data', (chunk) ->
          body.push chunk
        req.on 'end', () ->
          body = body.join ''
          if contentType is 'text/plain'
            body = body.replace('\r\n', '')
          if mp_index is -1
            req.post = _bodyparser(body)
          else
            req.post = _multipartparser(body, contentType)
            for obj in req.post['multipart-data']
              req.fileName = obj.fileName if obj.fileName
              req.fileLen = obj.fileLen if obj.fileLen
              req.fileData = obj.fileData if obj.fileData
              req.fileType = obj.fileType if obj.fileType
          req.body = _extend req.body, req.post
          try
            cb(req, res)
          catch e
            dispatch._500 req, res, req.url, e.toString()
      else
        try
          cb(req, res)
        catch e
          dispatch._500 req, res, req.url, e.toString()

    wrapper

# End of Auxiliary functions.


# Constants.

  mime_types =
    '':      'application/octet-stream'
    '.bin':   'application/octet-stream'
    '.com':   'application/x-msdownload'
    '.exe':   'application/x-msdownload'
    '.htm':  'text/html'
    '.html': 'text/html'
    '.txt':  'text/plain'
    '.css':  'text/css'
    '.mid':  'audio/midi'
    '.midi': 'audio/midi'
    '.wav':  'audio/x-wav'
    '.mp3':  'audio/mpeg'
    '.ogg':  'audio/ogg'
    '.mp4':  'video/mp4'
    '.mpeg': 'video/mpeg'
    '.avi':  'video/x-msvideo'
    '.pct':  'image/pict'
    '.pic':  'image/pict'
    '.pict': 'image/pict'
    '.ico' : 'image/x-icon'
    '.jpg':  'image/jpg'
    '.jpeg': 'image/jpg'
    '.png':  'image/png'
    '.gif' : 'image/gif'
    '.pcx':  'image/x-pcx'
    '.tiff': 'image/tiff'
    '.svg':  'image/svg+xml'
    '.xul':  'text/xul'
    '.rtf':  'application/rtf'
    '.xls':  'application/vnd.ms-excel'
    '.xml':  'application/xml'
    '.doc':  'application/msword'
    '.pdf':  'application/pdf'
    '.mobi': 'application/x-mobipocket-ebook'
    '.epub': 'application/epub+zip'
    '.js':   'application/x-javascript'
    '.json': 'application/json'
    '.sh':   'text/x-sh'
    '.py':   'text/x-python'
    '.rb':   'text/x-ruby'
    '.c':    'text/x-csrc'
    '.cpp':  'text/x-c++src'

  default_options =
    version: require('../package.json').version
    logging: true
    log: console.log
    static_route: "#{process.cwd()}/public"
    serve_static: true
    list_dir: true
    default_home: ['index.html', 'index.htm', 'default.htm']
    cgi_dir: "cgi-bin"
    serve_cgi: true
    serve_php: true
    php_cgi: "php-cgi"
    served_by: 'Node Simple Router'
    software_name: 'node-simple-router'
    admin_user: 'admin'
    admin_pwd: 'admin'
    use_nsr_session: true
    avail_nsr_session_handlers: ['dispatch.memory_store', 'dispatch.text_store']
    nsr_session_handler: 'dispatch.memory_store'


  escaped_favicon = '%89PNG%0D%0A%1A%0A%00%00%00%0DIHDR%00%00%002%00%00%00%25%08%06%00%00%00%00%EC%BA%92%00%00%00%04gAMA%00%00%B1%8F%0B%FCa%05%00%00%00%20cHRM%00%00z%26%00%00%80%84%00%00%FA%00%00%00%80%E8%00%00u0%00%00%EA%60%00%00%3A%98%00%00%17p%9C%BAQ%3C%00%00%00%06bKGD%00%FF%00%FF%00%FF%A0%BD%A7%93%00%00%00%09pHYs%00%00%27%3A%00%00%27%3A%01%5D5%BBa%00%00%0C%95IDATX%C3%C5%99%7B%8C%5D%C5y%C0%7F3s%CE%B9%CF%BD%FB%F6%EEzw%CD%FA%01%18pLpDpx%87%A0%26A4%16%A1%01%A9I%AB%145%A5%15%7FT%22%EAKi%DA%A24BQ%1B%9AT%A1%89%13Z%29%0A%A1%10%15%D5n%0A%14%88%C0%01c%BB%3C%16%E4%C4%06%8C%D7%F6z%BD%DE%87%EF%BD%7Bw%F7%EE%7D%9C3%8F%FEq%EF%EE%DE%F5%EE%925Q%DBO%1A%CD%E8%DE93%DFo%E6%FB%BE%F9%E6%1C%C1%05%CB%A5%C0%A8%07%BBZ%E0%E2%7E%E8%DB%0C%9D%97@r%0B%24%07%20%D1%0D%B1V%08%D2%A0%02%90%5E%ED9%AB%C1%84%10%16%A1%3A%05%E5q%28%9D%82%D2q8w%0C%CE%0C%C1%7B%23%B0%B7%00%BD%1A%DE%BD%20%AD%C4%DA%BA9%E0c1%B8%A5%1F.%DF%01%BD%3B%A1c%3B4o%86t+%C4%92%E0y%A0%04%C85%0C%EB%00%0B%18%07ZC%B5%04%C5%29%98%1E%82%ECa%18%3D%04G%07%E1%85%118X%5D%8B%9A%EF%D3%E3f%60%1Fp%FF%3A%B8%EAz%D8t%1B%F4%5C%07m%1B%20%99%04o%ED%EB%B0fq%80%06J%25%C8%9F%86%B1W%E0%C4%D3%F0%E6%7ExhrQ%A7%E5%E2-%FF%A9%1D%C8%01%1B%7B%AE%FC%F0%8E%5DJ%5E%7F%F7%99%D1%CB%3F2W%ECh%AAV%93h%ADX%DB%AA%7F%10%11%F5%B1%13I%A5%BA%B7%C6b%99%AD%E9t%FB%5D%BD%BD%A97%8CqO%1C%3E%3C%B5%17%18%5B%D4q%95%1D%F1%93_%C2%9A%E3%09%3F%FE%3B%B7%19%B7%E3%BE%AB%B6G%D7%7E%E3o%5Bb%89x%9A%B1%B1%90%B3c%9A3g%1C%A3%A306%A6%98%98%94%0C%0FKfg%BBp%CE%07%07%CE%D5%17v%CD%12%D1%D44%C1%C0%80%A5%AB%CB%D2%DDm%E8%EBs%F4%F5I%D6%AF%F7%E8%E9%09%A8T%8A%7C%F9%CBS%D5%C1%C1%E0%00%0C%3E%0C%3Fz%1A%B6%94%E1%07%CB%87%EB%DC%E6H%B4%DF%DF%97%E9%7B%FE%A1%E6%81%89%5C%F3%80q%9D%17%CF%B8%EF%FD%F3q%E7%9Cv5%B1%CE%DA%C8U*%15%97%CFO%BB%DD%3F8%E86%5D%F6%BA%0B%9A%AB.%D6%EC%5C%90v%CEK8%A7%02%E7%A4%E7%9C%90%CE%21%5C%1Do%B5Ru%BD%BD%AF%BB%EF%7E%F7%A0%CB%E5%A6%5D%A5Rq%D6F%CE9%5B%9F%D3%B8%EF%7C%E7%B8%8B%C5f%5C%CD%A7%26r%F0%FCCp%7F_%E3%8A%29%80%0D78%E6%C6%FFzk%A2%ED%B3%FF%104%7D%E4%0B%7E%B2%25%E5%27%04N%05d%F33%DCx%8D%A3%B5%25%09%08%84%90x%9EG%3C%EEc%9DcvN%91-@Y%07%C8@%E1%05%8B%B1Jz%20d%DDZV%D9%8D%AE%AE%02%9F%FA%94%E3%F6%DB%5B%D9%B2%A5%03%CF%F3%11b%D1tO%9C%C8%F3%95%AFD%8C%8C%AC%AB%0F%94J@%F7%D5%D0u%09%7C%FF-%D8%97%85%07P%1D%97%3DL%25%FF%E4E%A9%AE%CF%7E%3B%DE%BC%FDv%3F%19W%5E%1C%BC8%F8qA%A1%E4%D3%92%9A%60c%BF%E0%B57%CFaMDK%26%86%94%8A%DE%F5%CD%7C%E2%A6%14W%EF%98E%A9%2C%E7%0A%86%8A%0E%90%BEB%05%A0%FCz%F1@%A8%1A%94%03%B0%11%5D%5D9%3E%F7%B9%09%BE%F65%C9%BD%F7%F600%D0%8E%10%02k%0D%C7%8F%E7y%E3%8D%2C%A9%94%E1%D1Gs%3C%FEx7%D6%06%8DN%20k%21%BFu%23%FC%E3+%F0%DB%D3%22%D1%7Eu%AA%F3%B2o%3C%18o%D9y%9FP%89ekg-%ACk%3ACO%7B%89%F7F%DA%E8i/%F1%9B7%86%7C%FA%E6V.%EAkAJ%05@%B9%5C%E1%D5%C1%2C%3F%D9%5Bf%DF%C1%16%CE%E5%5B%B1%DA%C3%EA%FA%09%12%81%D3%9A%B6%E6%29n%BD%A9%C0%17%3E%9F%60%E75%1D%24%93%F1%FA%3C%86%93%27%0B%3C%F9%E4%14%8F%3D%160%3C%9Cd%DB%B6%3C%A7O%279%7D%BAo%95%1D-%5B8%F40%FC%D9_%88%BEkw%DF%91%E9%DD%F5%3D/%D1%B5n%A58%E4%1CX%EB%B0%C6%21%90X%E3%90%AE%C4%A6%AE%2C%B7%5D%1B%F1%E9%1BZ%E9%EF%5D%04%CA%E7%0B%3C%F2%E8%7B%EC%7E%BC%8BRu%03%AE%0E%A2C%88%AB%D3%DCw%CF%04%7F%F4%FB%17%D3%D6%D6%B2%00p%EA%D4%3C%80%CF%91%23%1DDQ%B2nZ%B6%5E%BF_%84%9C%98%84%BD%7F%28%B6%DE%F1%E2%9E%A6%9E%EBv%09%E5/%C411%1Fx%EA%11%C89p%B6%5E%9BZ%5Bk%874%256vf%B9mg%C4%27%AEi%26%3B%15%B2%E7g%25%5E%7C-%C3d%AE%0D%13%F9%0B%BBa%23%B0QDgK%9E%5B%AF%9F%E1%EE%3B%92tv%04%FC%E7%D3%D3%FC%F8_%7D%0E%BF%D5AXNr%E1a%3D%02%5E%D9+v%FC%C1/%B3%A9%CE+%DA%11%20%CE%1F%A3%21%B6%60k%00%D6%D6a%0CX%03%3A%AA%01u%A7s%14K%01%93%856%9C%09j%FFG%8B%10%26%AC%B5u%15L%18%D2%D9%92%27%15%0B%19%1Aj%A7%3C%97%C4i%B1%D0%DF%99%0Bd%E1H%CEK%B4%26%DA%E3%CD%8B%3B%28%EA%BB%82%9Bw%CC%86%1D%B1%8B%00%D6%80%D3%E0%1B%81%89R%8C%85%29%90%10%CF%2C%05%90%F5%B6%F4jm%E9%81%F1%03%B2%C5n%C6s%80%0F%B1T%1D%B0%1E%ACLx%A10%89vOz%B3%C4%EA%20%E2%7Cst%CB%21%16@4%18%0D%BA%AA%C9%C4%A6H%8A2%13%B3%ADTu%1A%15%08dX%07%09%EB%CA%AB%C6%C8%E5HzEd%3Cbf6%03%C2%C3%AB%CF%3Fo%15%17%063%8B%A7b%E3%C4%9A%3E%84Pr%19%C8J%26e%CD%22@*%9Cb%FB%FA%02%9F%FCp%9C%DE%8E%24%AF%1C%9E%E4%B9%B7%F2%9C%C8%B6%13%AA%14%C6%135%E5%EB%00B%3A%7CU%E2%8A%8D%E7%D8%F5qMw%BBb%CF%7F%E5%D9w%A0%85l%B6%15%E9%AB%C5y%D7%0Cc%81q%BC%B6-%1E%5E%A2%88%97%C8%2C%804%3A%BB%B3%60%B4%C59%89%AB%03%24%CBS%5C%D6R%E0%D6K%E3%5C%B9%A5%8FT2%01@%7Fw3%5Bz%86%F9%F6%BF%BF%C3%89%E2v%3C%15%D4%00%EA%E3%FA%EE%2Cw%7E%7C%82%DF%FB%AD%0D%F4%F5%B6%01%92%9Bo%28q%E0P%96%C7%9E%2C%B0o%7F%1D%C8S%B5%C5%D3%B5%F9%DF%3F%E5%29%82%F0P7%FE%E5%9F%FC%8D%09C%82T%86%20%25%F1%12%B5%C3%D0%8B%81%8A%81%A4Lw%F9%04J%84%08Wf%5Bb%9C%BB%AF%80%3B%3E%DA%C5%E6%BE%0E%02%DF%C7Z%C3%D9%C9%02O%BD%3A%C1%93%83%3E%A3%D5%0D%A0%E2%9C%1F%81%9C%93D%91%21PU%3A%5B%05%E9T%8CX%2C%60%CB%A6%0C%B7%DE%94%60%FB%E5%D3Da%8E%F1IG%A5*X%D7y%0E%E1%22%AA%D5%C4*%10%1A%21%CE%22D%0C%F1%C7%27%87%9D%D5%1E3%A3%11%A9u%FDH%25%17%FC%C3XGKv%98%7B%06bDV0W%8D%D8%DA%DBN%3A%95%ACm%AA%B5L%E4%A6y%F9%ED%3C%FB%86%14%C33%1DD%26%85%8D%04%A6%0A%A6Z%3B%3F%1A%DBQ%D9%21%A2%22%97tg%F9%CCM%96%DF%B8%B1%9D%9E%AEL%3D-q%CC%CC%94%F8%F9%CB9%7Ey%A4%C2%0D%D7%C5y%F4%D1%12%BBwo%02%17%9C%07a%11r%04%A9%7C%AC%D55%90tw%3F%F9%E3%A7%D1%15A%D3%FA%F5H%CF%C39%D0%D3%B3%DCR%9D%E0%F6%CB7%22%95Z2%CC%D4%F4%0C/%1C%19%E7%E5%D1%18%23%E5v%22%9B%C2%85%02%13%D6%23Pu%B1nl%EB%06%20%A5g%D9%DA%9Bc%D7%AD%8EO%DE%D4%C6%FA%9E%F9%F0%E9p%CE%22%84%E2%B5%D7%C6%B8%EB.%C7%A9S%EB%97%EE%84%3C%8B%0A%1C%CEm%C0F%23%F5tN%082%FD%FD%F8%09%C5%DC%E40%D2+%E1%A5%1C%DDQ%96%9D%BD%ED%0B%10%CEZt%14R%AET8%3B%99%E5%BF%DF%1De%B8%14C%26%D2x%F3%8E-%97%17%CE%AB%85%00%A5%04%9A%0C%83%C7%FA%F9%D6%23%25%9E%7B%E1%1DtT%AD%3B%84@%88%DA%9CW%5D%D5%C1%5Dw%CD%21D%A5%0EQB%F9%C3%F8%29%85%F4%FA%17%CCw%E1b%25%A4%24%DD%BD%9ER%BE%C0%D4%89Q%3Ci%B92*R%AC%269s%F2%0CS%15M%BEl%C8W%1C%F9%B2d%BA%AA%C8%257%A2Ll%21%03XV%7E%85%08@J%83%03%5Ez%AD%99%91%B1Qz%3A%25%5D%9D%8A%AEN%9F%8E%F6%80%D6%D6%18w%DE%19c%CF%9E3%1C%3B%9E%22H%CD%A2b%9D8%DB%82%0D%17%7Dp%E9%0DQ%08b%99V%A4%97%A68%7C%8A%17%C7%A7%D8%3F%A5%A0%B9%0B%E3%B5%A0%8D%8F%B1%12%A3%14%D6%93%D84%A8%BA%C9%D8%C60%BDJ%A1%B1%AE%9FQB%C4%C8%CFm%E3%A9%7D%06%13j%D0%06%CFE%C4TH%3AQ%A6%BD9%C7%BAu%13%C8D%89%D4%BA+%90j%00%AB%03%8C%5D%BA%28KA%EA+%A9%7C%9F%F4E%17%13vl%A0%98%CB%13%E5%E6%80%08%15d%102%85%B3rI%DA%B1%98K%B1%90%ED6%16%A7%1B%B2%01%B3%3C%CD%C1%82@%21%9C%AA%BD%8A%A8D%CC%84%8E%09%5B%E6%A4/%F1Nn%C2%0B%DA%F0%13%09l%B4%F2n%7BV%1BK%E3%D5G%2C%1EH%5E%10%23%D5%D1%83NET%A6%8BT%0A%D3DsY%ACV@%12H%E2l%805%1E6R5%A8%D5%E0V%804%1Ald0%91%C6%EA%10gKHU%22%D6d%90%81%8F%F22@%BA%96%7CF%60L%5D%B7ey%A5%B1%9E%10b%19%DF%7C%BF%F9%FB%B7P%3E%B1L+%7E%BC%95%A8%1C%11%CEU%08g%E6%A8%16s%E8%B2AW%C1%86%0A%13yX%AD%B0%DA%C3D%0A%1BIL%3D%92%99%D0aB%8B%09%EB%8AG%06g4B%19%FC%24%04%29%85Pq%84%E8%C0%B98V%FB%D8%FA%A2%CC%1B%0B%AC%96%1B%0B%E7E%E5r%1E%E8%5C%89d%C9C%F5%24Rz%3E%7E%D2G%F9M%04%190U%83%AEDD%15%8D.G%B5R%890%D5%0A%26%04%A9%1C%D2%07/%26%EA%A6%A5p%D6%C3%B98%CE%FA8%E3a%8D_%5B%80hq7%17%92%D6%F7%05%98%FF%B3%9C%F7%E6%26%27%0E%B4%0Cl%D9%25%F0%7Fu%98i%F0%A3y%F3%13R%21%7D%85%0FHU%CB%08%1AS%F7F%E5%E6M%CD4%FE%07%08%BB%D4%02%D6%A4%C3%82D%C0%C4%01%99%7B%F7%D8%0F%CB%F9%FC%E4%05%DDgV%BA%B45%026%80%BA%86I%9D%5B%AA%AC%5Ba%D85%E9%21%1A%1F%CEO%C2%B1%1F%CA7%FF%E5%91%E7%CE%1D%3D%FADT.%DB%25%F7%90%0F%28%E2%BCzE%8DW%EA%F3%81%A4l%E1%E8%13%CE%3C%F2%9C%FC%D0%E7%BF8w%E2g%CF%7E3%FB%F6%D1gL%18%3A%04%88_%87%E4%FFLB%E7%DC%D1g%9C%7D%F6%9B*%F8%E2%9C%1Az%F6i%EE%3D%B0%7F%FA%D5%7F%DA%3D%1845o%88eZ7%0B%E5Kg%EBq%FF%FC%83%CD%ACr%E0%AD%E5%F7%D5%FA%AC%A1%EF%92%83%D6%94%B4%8D%7E%F1%94%8D%7E%FA%A7%7E%EA%EBC%D1%DCGk/%E8%5E%7C%E0%01%7E%F7%F9%7D%D9C%DFzh%BF%8A%C5%95%9FH_%AA%82D%02%27%96O%FE%FF%04R%CB%0A%2C%26%3A%977%E1%EB%DF7%E1%9E%AF%C62%0F%9E%28%E7j%06%BA%90%D2%BE%FC%F5%07h%BFd%DB%CC%D0%B3%7B_R%B1%C4%3BBx%9D%CAO%F4H/%F0%96%00%FDo%83%AC8%8F%C3%E8%D9%8A%AD%1E%7BYW%7E%FEW%D1%DC%C3%BB%A5J%E7+%85%CF%2C%18%DA%92%DC%7C%F2%17%83D%B3%B3%FA%DD%FFx%FCm%1C%CF9%E7NZm%D3B%FA%EDB%FA1%E1%E4%D2%15Zu%E2%0F%08%B2%0CJc%A2%C2%AC%AE%0C%1D%8CJ%AF%FE%7DX%FC%C9%83%95%C2W_%85Ym%C2%FDK%3CF%9D%EFB%BAR%06%A0%3AS%28%1Eyb%F7%1BV%EBgt%A5%3C%A8%CB%E5%A2%D5%26%E9%ACL%82%F4q%12%9CXUi%FB%01@%ACqX%1Da%A3bIW%C6%86t%E9%9D%BDa%F1%D0%DFU%A7%FF%ED%A1%D2%E4%9F%BF%84+%14%9D%C9%83+/s%FD5E%C0%7B%0E%3A%7E%FA%A5%8F%C5z%AE%BA%A5%BFy%C3%E5%3B%E2-%BD%3B%FDd%C7v%15k%DE%2CU%ED%8B%95%B3%9Eg%8D%12VKl%24%96%BE%D3Z%F6%7E%CBak%E9%8A%D3U%ADM%B5Z%D2%95%E2TT%9E%1E%D2%E5%EC%E1pn%F4P8%7Bt%B0Rxad%DD%F6%83%D53%07%7E%AD/V+K%A6%EFRJ%B9Q%AF%EF%9A%5D-%E9%EE%8B%FB%E3-%7D%9B%FDT%E7%25R%25%B7%08%99%1C@%D4%BF%21%BA%20m%8D%0A%AC%91%9E%D3%60B%ABMhB%5D%0D%8B%A6Z%9D%D2%D5%F2%B8%AE%94N%99j%E9xX%3Aw%2C*%9E%19%AA%CE%BE7R%1C%DF%5B%F0%E2%BD%3A%9A%BB%B0o%88%FF%03%C9%B2%26v%1E%C8%18%09%00%00%00%25tEXtdate%3Acreate%002013-09-14T18%3A41%3A58-03%3A00%E9%21%CE%C1%00%00%00%25tEXtdate%3Amodify%002013-08-31T18%3A29%3A00-03%3A00a%96q@%00%00%00%19tEXtSoftware%00www.inkscape.org%9B%EE%3C%1A%00%00%00%00IEND%AEB%60%82'

  escaped_icons_png = '%89PNG%0D%0A%1A%0A%00%00%00%0DIHDR%00%00%02%82%00%00%01%3A%08%03%00%00%00%B5_%C3%A2%00%00%00%07tIME%07%D8%08%13%0A%00%01%3A%F52%12%00%00%00%09pHYs%00%00%0B%12%00%00%0B%12%01%D2%DD%7E%FC%00%00%00%04gAMA%00%00%B1%8F%0B%FCa%05%00%00%02%EBPLTE%FF%FF%FF%F7%F7%F7%EF%EF%EF%E7%E7%E7%E4%E1%E4%BE%92%8C%97%0E%09%B3%9F%A2%D2%D7%DE6Z%8F%27S%8Fk%80%8E%CF%CE%C4%91%8C%89%C6%C6%C6%A5%9D%A7%8C%8C%84%D1%A5a%F7%B3G%DE%DC%D2%D2%D6%D6%C0%C0%C6%BB%B8%B5%DE%DE%DE%D3%D3%CE%B4%B2%BB%A9%B5%B1%B5/*%DFra%E0NH%E2.*%BF%11%09i%82%AA3h%ACGn%AC%93%A7%AFs%9D%CB%DE%EB%F7%A5%B1%AC%F7%B4V%B9%AD%A5%3DELUUXdlkvs%80%ABw%0D%ADfKxmg%82%7Ct%90%90%90Tq%8D%F1%D6%CB%F0%CC%C5%CB%D3%E9%B3%C2%D8%83%AB%D3%96%B3%DCe%99%C9MX%7F%80%89%91%F7%C4n%F3%C7d%B9%B6%A5%82y%83%DA%9D%29%91%9D%90%89%95%AEd%88%BC%B7%CA%E9%C1%C5%B0%AB%0E%08%82%9F%C9%D4%E8%F8%C8%DE%F5%E4%E9%F7%F8%C4y%F7%AD%3D%F7%B6a%F8%E5%B7%9C%A9%C4%EF%F7%EF%F5%C7%90%FF%FF%F7%F7%FF%FF%F8%CAE%FB%E2%DE%F7%DE%DE%DB%DE%EC%F7%E8%A4%F7%DBg%FB%E2%90%FF%F7%F7%E7%EF%F7%D6%DA%E7H%7E%B8%F7%F7%EF%EF%EF%E7%A1%A9i%B2%C2%C6%FF%F7%CE%FB%EF%CA%EF%F7%F7%F7%D9%A6s%86%8C%E7%EF%E7%F8%EBK%F9%EDf%F6%DEW%F4%DBHw%24%1A%F6%E0u%F9%EC%91%FB%F1%B3%FF%FF%EFqxK%FD%F0V%9E%AA%8B%F9%F3%A3%F5%DB%85%D6%12%0A%A9xo%B1%C1%CE%E7%E7%DE%FF%F7%AD%9C%92%8A%29%29%26%FF%F7%DE%D3%C8g%F7%F7%E7%DE%E4%BE%FF%EF%BD%FB%F4%91%FB%F5%7F%F3%DE%94%EF%EF%CE%FF%F7%E7%B2%B3%CE%80%8Fz%DE%E7%DE%F6%C7%87%F5%F2%C2%F7%EF%EF%E7%EF%EFq%5CWy%97L%AE%9D%29c%86%19%DA%C2%26%A3%D5h%C7%C3%8B%F8%CAR%F5%F8%D0pK%1EAC%25%F7%DD%99%E1%D3%D3%F4%96ijZu%FF%FF%E7%F4%DF6%FBw%5E%FA%5E%5C%9E%C7%3B%F5%EA%7E%F9%F0%3C%EF%D6%00%D8%BF%AE%FF%FB%B9ib%22c%60Kgs1%D6%E1%A9%F7%E4%1B%EF%DE%18%F1%D8%0C%F0%C5%0E%EF%BD%007Ck%CD%C8%DA%BC%D5%A5%29Z%A5%EF%E7%EF%E7%EF%CE%EF%D6%29%1F%1C%0B%F7%E7%E7Ss%04%1B%3Ak%FF%FC%CE%EF%DE%DE%A8n-%D9w%0C%D4k%2C%E7%EF%FF%F7%A09%F4%9D%9A%F7%F7%FF%F7%FF%F7%D6%D6%B5%F5%CB%9D%C6%A5%B5%EF%EB%E2%EF%EF%F7%FF%FF%DE%EF%EF%B5%EF%D6%1C%F1%D6%B7%F7%DE%21%F6%B3%7E%DD%3E%0A%F8P%2C%F5%B8%B8%E7%D6%D6%F3%E2%CE%F3%E2%C6%EF%DE%BD%F3%E9%D8%F7%E7%CE%FB%ED%E2%E7%E2%D6%E0%CA%D4%F7%DA%C9%F9%A2%29j%BA%1B%F9%A2%0B%EF%B5%00%B4%3A%0B%FF%DE%B1%F4%7F%0E%F7%E2%C1%F5%8E%14%FB%E7%C6%F7%F7%DE%F7%94.%F7%B9%18%F3%B5%08%F6%C20%EF%CA%00%F3%D6-%F6%E4*%FF%EF%D2%F7%DE%B5%EF%EF%BD%F7%DA%10%FF%F7%EF%F4%EA%0D%F7%F3%18%EF%F7%FF%E7%F7%FF5oSz%00%00%A3%9FIDATx%DA%EC%BD%7B%7C%14%D7%99%F7Y%97F%B6%A4VU%C9%D4%AD%5B%20%B5..P%B7%A4%EE%D2D-%01%81VK%A2%95%D8%21%1A%3C%F9%AC%60%C7%06%C9%C9%D8%C1%60%F00%18%09%0F%1AM2%C1%23b%0FVK%C8%91%F0%C7K%02%1E%16%F0%C7%17%E1%BC%83%86%BC%23%298%1F%B3%E3%8F%AF%93%AC%9D%D9%F1%DAc%5E%C7%B1%938%8C%F3%EE%CC%EC%9F%FB%9C%AAn%F59%A7%AAK%128%D9%EC%BE%FD%C8@%BB%1F%9D%BA%9C%FA%D6%EF9%E7%D49O1L%C1%0AV%B0%82%15%AC%60%05+X%C1%0AV%B0%82%15%AC%60%05+X%C1%0AV%B0%82%15%AC%60%05+X%C1%AE%C7%D2%C4%0Fa%AC%7C%20%19%22l%03K%17_%5BQ%D4%D4t%B8%A2%A8h%90%F6%98eB%E6%93%21%99%D6%BF%02%F5%1B%07%D2%B6%F5%E7%3B6V7De%D13%60%A5%C3.%DF%EA%86o%B1%82%C1p%7D8%22%E6%FE%BF6%14%A2%7E%E30%EBV%8E%E7%3D6%BA%BE%BA%BA%BAf%D1cv3%A3%E1I%B0%06c%F9%25%CD%B0m%09%F3%BAv%7C%E3%26%86i%C3%AA%95%E1D%892%91%C3%DCi%0F%06W%0Eo%FB%EA6%21%9E%B3%B6%7B%FF%96%C2E/Z%B5%02%F8K%ADj%BA_%A3*%D4T%92%D9%1A%D2MT7%3CE8%93%FE%E8%A7%1F%FD%E5_%EE%7C%8B%FE%1E%19%AB%07%CD2%29%A9%CA%A6%BC%C8%E9s%E7%26%CF%D2%DF%956%17%97%94%94%147%E7%E77%11A%F5%A4Ef%EAs%DF%85%0E%C5C%04%B7%D2%A4%5BQ%7Dn%CE%9DA%5D%87%BF%AA%3B%3AB%D5%F0%B9Fw%FB%15%7Ez%3A%0F%BF%C6%93%A2%F2%0E%98%22%3E%A9%FA7%FBi%F6%053%16%8DD%A21Sp%165%25%81%03%F3%F9t%E2%CA%FF%0E-%1C%0C%AA%B8%05%83a%CC+%1A%1AO%98f%E0%C7%99%FEY%FA%04%F6Clx%F3%91%3F%BD%E3%3BO%1Fj%5B%B0%F8%BD%BB%FE%96%DCwEjEQQ%05@%D8%D4%D4D%D5%8B%EC%13%ED%1D%25%0D%9F%8EhLS%A8%1D%18%FE%CB%07%C1%B6%ED%1DNS%97%25%90%04%FA%14%5D4%C1%CA%EC%FB%A6%CC%A5%EA-%E3%26g%27%29%15%D4%8BKj%3A%D7%D5%D5%AD%AB%29Y%EF%22cR%03%E0%17%D1%E5%A1%19A%9F%21%10%0Ct%C6w%60w%D2%E1%89A%97%1D%F2s%5B%DFve0X%BC%C3@%08%86B%D5%BA%5E%1Crc%90%9F%AE%EDpg0a%5E%18%11%83%81%80%28%06%D4%FE%E1%E1%E1M%D8%81%27%A3%D1%06ID%179%104%A4h4%9A%24%CB%86%05%1F%98%A6%05%0D%7F%98q71%E2%05%E7%8Dx%ED%23%08%1A%EAQ%CB%A2%B1X%EC%A8j%10%08J%9A%A6%13%C6iR%CEk6%A4c%D6O%C4%FAi%90%1A%1BsN%FF%91%BF%BE%E3%15%84%60%8F%08%D6%E3%82%A0Za%11h1X%B4%96p%95%F1%3E%CD%AA*%9F%E4%D3dP%C1%03%E3%04%82lz%7C%DB_%3F%F0%C0%03%7F%FE%E0%C1G%DFn%27%C4%D5%0Cj%1C%CB%B0A%5CW9%89q5%DF%E4l%FB%24yQ%C5%9A%9Aumm%5D_%F8%C2%9F%B4%B4t%87Hq%0E%00%7D%0D%12%9F%9C%89h%E6L%C4%90%EA%EB%139gG%A8%A6%AEm%C7%C2%B5%F7O%3C%ED%82%20%3Fw%A5%BD%7D%AB%0B%83zs%BC%07%82p%5DOs%07%FC%13%AAEZH%97%9D%AE%BDp%C1%95%C1%84%F8N@c%EFy%FAi%B6O%E9%DF%B3k%EF%EEa%7F%CE%19Ue%BF%01*%18m%88%25%92%82%22%28Q%0A%00%AD%BE%9E%13%EB%91%E5A%90%95%E4%88%F5Aa%BD%BC%DEe%3D%0C%21%18%B5%EC%DC%B9%A9%A9%FD4%82%BC%FE%D0J%CC8%9D%C7%AE%A6%A4%19%AA%A1%8A%AA%A1%88%F0%B7%D8%D0%F8Vy%8EA%F6%C0%B6%07%0E%EAq%20%B0%07%0C%18t%20X%B4%CA%D2%3F%C4%60SE%11%E1%92%92%BC/%88%184%05%CD%974%18%7F%FA%81v%DC%9Fn%FF%CE_%D8%08%EE%3D%FD%C0%D7%867%E3E%19%96%85%C0%22Z%7Fg%8CuG%90%9F%9C%BD%3C%7B%0E%FF%86%D3%3B%DA%BAz%EF%FC%11%3A%FE%AE%CEu5%C5d%5D%E9%0D%E1%99%24%3F4%A3%18%D0%10LPW%24%14%EA%ECn%DB%91%FD%BF%C1%F2%B4%B3%91%09%1A%08%08%BA1X%DC1%12%8F_%E8%A9mn%0E%85J%3BB%1D%3D%0E%19D%04%F6%F4%14%87%9C%0C%8A%E6%3B%01%EE%E1G%1E%99%18%E6%B8%BEm%D7%FE%EA%A3%D3%277%E5%BCQ%5Dl%8864%00%80%C8DE%A7%1049%DD%C7%F3%3E%BF%9Al%C8%83%A0%28q%B6%27%D4%E1%E5%F5.%EBa%08%C1d%CE%9C%08j+u9k+5%12A%0E%8F%D1j%E3%C3%E5%83%98%0C%86V%7EcsK%BC%0D%08D*%28%3A%10lJ5e%08%04k*%22j%D6%07%F2%8BdP0%7D%9A%DF%84%86%E0%BE%FF%B5%1Ds%FB%D3_%7B%F4%81%EF%3C%FA%C0%03%0F%3E%B8w%DF%83%DF9%92%C6%60%90%00%BDD%82_%02%82%EC%E4%D9%8Fg%27o%C2%BF%D2%3BZ%5Ef%EE%FC%A1%85%E0%9D%BD%DD%EBjJ%89%BA%12Tq%88%17%C3%F5%11%D3%25%BA%84%D8%D0%BA%EE%B6PfC%E5%ED%E79%FA7%80%C0%F6G%1Ei%9Fsa0X%1C%8F%8F%8Cd%3Bn%1D%1Dy%08%EC%19%29.u0%F8%E4%85%C0%CE%99%D7%0EB%00%06%04%1F%BC%EF%5B%0F%9C%DE%93%BB%27%83%92%08%FC%252%26%25L%7F4H%01%B0%88%0A%26%23e%19%8Cx%97%E6A%CE%EB%5D%D6%C3%E0%08%12XSP%AA%17%1D*%08%E8%D9%26%AF%E4I%04%99%DCU%E6d%8A@%26%D4%7B%D7%88%85%20%82%10%218%9A%1E%C3%DC%83%10%7D+%B2%04%A6*%1Ek%22N%CF4yM%10%A1M%A8A%28f%D2%C3%0F%B6%B7c%DE%F4%91%07%B6%3D%F0%E8%A3%0F%3C%B8m%1F%A8%20%A8-%16%8AM%8EU%A0%91%B68%82%ECc%93%ED%8D%B3%93%84%94%05%3AF%BAz%FF%EF%FF%FC%D1%7F%FE%E7mp%E8-w%B5%96%E0%A18%AC%8AB%C3%D0P%3DfX/2T%CBv%D6%F5%F4%84%92Ih%84%C6%CA%1Bc%F4%0E%11%81%1F%3Er%CB%87%1F%7E%E8%C2%A0%5Es%E1PO%A8%F6%D0%A1%91%9EP%A8%A7%86%D6%9B%0C%81%3D%A1%E6%E6%0D%14%83%A6%18%D0%FA%D8%99%0C%82%AF%7C%EB%BE%DD%7B%FAs%F4%C7%04%1B@%BBM%5CV%26%05%82%E4%81%81%0A%22%01%F1%AB%D0%C7%C2%1B%16%D6%09%83I%91%956F%1D%0A%13hf%F3y%9D%60Qe%BD%10%1C%22%8DF%90%07%F64d%FA%90%E0%82%A0%3Fc*D%E1%26%9C%C0%C5%10D%C1%D7%C2%AF%B2%A2%AA%AA%AA%A2%A9%82%EC%91H%8A%A6%81zB39%09%0D%C1%3F%FF%B3%07%B0%B6%E0%81%E1G%F7m%DB%B6%ED%81m%DB%F6%1D%3C%BDw%DB%F6%DD%DB%D2%B9%DB%5E%94%D9%E8%CC%CC%0C%86%20%EF%8E%60tr%7E%8E%EE%8C%D4%86%3A%DBn%FD%CF%1F%DE%FA%C3%5B%BB%3AC%9D%9D%A1%DE%0E%7C%94%A3%3E%2C%06%C5H%24%22%06%CDhB%0DF%22%AA%82U%96%12%EA%08u%D6%B5uw%A2%FF%E9o%3FK%EF%D3%22%B0%7D%EB%ED%ED%AE%0C%06B%17@%06%11%7E%3D%23%1D%1D%1D%CD4%81%3D%96%06%8E%1Cjn%EE%E8%21%19%7CR%09p%3B%FB%F4%19%1BA%7E%DB%E9%83%9B%1Ar%DE%88%D1%60%03hfL%F4%93%8D%B3%9C%0A%86%8DX%3D%B9%DB%B0%2C%87%A3%BE%3E%1B%23%BD%99c%88Q%02%C2%EB%04%8B.%EB%81%60%A4%1E%03%10j%99B%90%D32%26%055%8D%CB%A7%822%10Hh%20%81%A0%E8%86%60%A6/RY90%00%0C65%91%ADAU%E25%D9%00%0EM%08%BB%0F%DE%B7%EB%CF1%04%FB%0Fn%3Bx%E4%C8%3E%B0%D3%7B%F7%EE%3D%BDg%CF%B6%E1%03%0BNAee%A9%DEg%21%F8%0FOpO%FC%04nq7%04%A5%C9%D9%B9%D9%FD%93d%D5%94v%7F%F1%CE%3B%FF%F3%0B_%F8BWMMMqqMw%17.G%F5*4%AA%12a%E8%07s33*%13%99%093%F85S%20%84v%AEk%EB%EC%14%84M%E5%8D%FB%A9%7DZ%04%02%7Bk%A01%B8%A6%7D%0D%C1%60suu%27%04bh%0Ar%20%C4%885%E8%93%60%7BF%04%8EX%D6%0Cx%C6I%1D%7C%F2%1D%85%E3v%EE%5C93%5E%8E%10%E4%FFE%BF%F0%24%86%A0%08%8D@%A9%2C%09%B7%B3iBK%CB4%A8%C6%A0%C9%CA%A8-%08%ED%9Dp%2CL%20%C8%85%C3%3E_%98gY%84Q-%CF%C0%ED%B8%A16%AF%974%97%B2%1E%08%26%A0%9B%17%0E%A3%A1%064%DA%15%A1%11d5%ED%DB%C8%94%84%29%7F%9B%CD%A3%82%AAE%20Y%E9%A1%DE%96%91%85%B6%20%EA%8E%90%08V%0C%DA*%08%04%22%06+%07I%04%19%C9%00%05T%B4%A4%C0%A7%0F%9E%3E%B9%07%1F%94I%3F%BC%1D%C8%3B%08%0A%B8w%FB%F6%DD%7B.%8Eo%BF%27%E7%E5L8*%13%DA%82%1C%F7%93%D5%ABW%AD%5E%7D%C6%15%C1%03%93%B3%1F6NNR%8E%E2%F8%9D%3F%FA%CF%3B%BF%F8%C5/%B6%D5%D4T%03%82%3D%DD%04%82%82X%1FV%F8H%BD%C1%A9%F5P%C5%91z%85%B8fJhCg%5Dg%5Bghm%7F%FB%AC%8F%DC4%3F%F7%06h%60%FB%96G%E0%CF%E7%B7n%DD%FA%F9%7B%E6r-%8F%EA%9E%91%0B%F1%9E%9E%0B%C5%E8%86%D80rs%1C%FE%A76%D7%27%E6%A7%8D%0C%81%F1P%07%84%EB%10%DE%1Ed%9F%7CG%D3%FB8%96%3D33c@%E3%5C%93%E5w0%04%A3%0A%D2%C0%A4h%88%224%10%E0B%28%01%12A%A4%82%BC%A5%82%91%FA%FA%06%DC%03%18%81%FB%0E%96/%E3%C3L%A0%16n%0E%C6%08%E5%F5%3A%10%A4%CBz%20%18N%242%FF%85%A1%B9%EAPA%E6%DB%DF%7E%E8%21P%92%FAD%7D%F8%DB%8C%BB%0A%DA%1A%28R%08%7E%C1B%10%C5a%7B%5C%B0%A9r%20%E7e%07+V%15%A5%00%C1%01%DB%E8@l%F5HdU5%A1%21%B8oO%FB%9E%07q%04%BF%B3%7B%F7%F6%DD%BB%F7%80%FDz%DC2%7CLR%B2%22%B0%08%7F%5E%5C%0DV%B9%CA%0DA1%16%9Bo%9C%9C%A5D%10%10%FC%11%F3rKWWW%1B%12%C1%E2%E2%BA%BA%12%CC%1B%21%9A%81%96%0D%11%C5%15%A9%23T%D7%D9S%17*OK%3C%B1O%7E%EE5%A4%81%97%B7%DE%BE%E6%91%AD%B6%BD%95%D3%C1j%14%83%3B%3A.X%1Dp%A1%A7w%C4%12%C2%85%C2%D3%1D6%81%87%0EAW%D9%EA%B3%AC%9F%CE%5D%EC%27%DF%F1%A9%3E%D0A%7EF%9E%01%11%F4%0B%3E%02A%C1%90l%11%B4%11Tu%83B%90%E7x%A4%83%BC%1A%A6%FA%23%B6%92%C1U@%18i%1B%98%9B%9B%19%B9%26%AF%97Y%AC%EC%E2%08B%84%09C%93%21l%D2%08Z%00%82%29e3%0F1%AE*hk%A0%1CpUA%7B%5C%D0%89%A0%AF%C9b%B0%CA%02%B0%B2%A9%28J%B7%16%C4%C3%3EY9%0C%0D%C1m%7B%C6%D3%27%B7%E5%20c%D3_C%D8%1D%DC%BB%7D%CF%1E%9B%C0%F1G%B1%3E1%F4G2%08%7E%0E%00%AC%AC%7C%D1%0DA%29h@%24%9E%A4%5B%C9%C5m%3Fb%D8.t%C3%D4%D5%01%83%DD%DD%EBp%04%99%21%88%13b%98EO%94D%EB%BF%C8%10%B5Y%C6g%A2%EE%C8%E12%96D%90%D1%21%0E%B7_%7E%E4%F6%CBk2%04%BE%3D%97%EB%E9Twt%00%81%1D%B6%0A%8A%17%EE%02%02k%3Bp%15%EC%88%C7%0F%1D%3A%14%EF%09%C5%0F%21%02%9B%A7%B1%BE%DB%93%EF%C8%7D%7Dp%B6%FA%CCL%10%22%AAO%E3I%04%15%25%13%82%81@C%F7%AB%02%85%A0%CE%A3%07%12%86Q%06%91%90%C6H%D3%A0I%A7%97iaF%EE%60%84%10%A34%E7%F52%8B%95%F5@0bG%E1l%20v%22%C8%F3_%FFz%9F%3C%14%8E%CD%F0%AE*%C8%DB%04%CA%A2%03%C18Bp%E1%E9%08%81%20%27%07P%87%24%05%CD@@pUQL%A7%11%1C%1CT%19up0%BDo%F7x%3A%FD%EBv%1C%C1%8F%9E%7F%FE%F9%E7l%FC%9EK%C3%E7%E7%7F%83%3D%2214@%90%07%04y%7E%15%A2%5B%F3%F9%1C%08*%26%CB%1B%D2YG%A7u%7D%DB%CB%BD%BD%B7%F5%ACkkm%AD%EB%AC%A9%E9%EE%EC%24%EE%5E@0%C2%88%AC%DA%90%1F%C1%EC%13A%95w%B6%05%B7l%BD%7D%EB%BB%8F%5C%B9r%05%08%FC%19F%20%13%82%A6_OGO%BC%A3%94%03%C9%E8%8D%F7%8C%AC%A7%DA%82%A5%23%1D%10%9CC%1D%23%A8%AFR%3A%8D%F7%D3%9FT%04%10%88%3E%DF%CC%0C%12%0A%20PSp%04%FD2@%A8%1A@%A1%A8%0A%7EC%10d%0A%C1%88%8E%B8%D5%04%08%82%0E%04u9%AC%EB%7E%84Q%C7%DD%8C%D1%C1tl%C8%EBe%16+%EB%81%20%16%88%5DU%10%CE%EB%EB%7D%9C%19%EE%0B%BA%21%08%22%D8pnb%B0QUT%27%82%5D%5E%08%AA%B1%8A%15%15%A9%D4U%14-%9B%26%0D%12A%AD%29%95%AA%ACHUT%A6%AA%E6%C7%D3%9B%D3%BF%3E%88%B5%F6%D2%DB%F6%02v%E3%CF%5B%86%10%DC%F3%9Bt%EE%F1%AC%60%B0p%C8%22z%EC%E9%5B5%B0%1A%08t%22X%26%B0%82%28%C6%1C%A3%FD%1D%DD-%3D%DD5%BD-%ADuuu%DD%DD%87%E2%9D%E4%E8%C8PCX%84%92%9A%28@i%C1%15AA%F8%114%3E%26%0C%07%82%C0%E0%A7%B7%BF%7By%CB%15%CB%08%02-dk@%FAn%AE%85%0Eoo/p%E6%E8%117%F7%A0%E1%C2x%DCA%20%D3%20*%88%3D%8B@%C4%12%1F%10%B1%B1%95%24z%EA%10K%06%14%BF%A0%8Af%24%E2xB%176%17f%07%98%0E%04%05%7FX%10%842%19%94%8CaC%0A%13%0A%E6%F3%3A%11%A4%CB.%86%A0%84%FE%0BK%A6D%DE%0AY%04%A1%B97S%1FvWA%BEq%EAl%A3a%18%AAI%23x%D7%5D-%A8%5D%95%B5%8A%01%EC%91%15%EBS%C4%FB+V%ADZe%0D%CC%14%3D%A6j8%0F7U%0C@%08M%01%86%D0a%AEjb%D2%171%04%F9%F2%23%DBvc%08%8E%EF%DE%FD6%86%20W%C6%2C%20%E8%7B%11%11%E8@P%96X-%28J%14%05%60ZOkg%0Dt%84%7B%BB%BA%BB%E3%7F%C2%B6t%13%E3%82%08%C10%13%86*%0B%8BI%D5%87%1E%D79%11T%B8%CD%87zj%03%8A%CF%11%FC%F9%B9%7B%DE%7D%F7K%EE%042%CD%21%D4%00%BC%D0%DB%7B%F3%05h%F8uTS%83%C0%A0%83%28T%F7%5Cp%10%C8hO%5E%00%1D%E3U%04%20%10%C8%C9%17%9E%C4%EFf%3E%AA%AA%C9DC%04%2C%DA%60%06%13Q%8D%0C7%00%5EC%2C%9C%F9D%EE%94%0D+B%B8AP%24%1D9j70%5C%8D%96%CFK%9B%A3%AC%27%82%D9@%0C%08%26%9D%08%A2%80%06*%C8%F1%FA%D7%FD%7F%C8%B9%A8%A00%D3%18N%28%8A.%D0%08%B6@%9Du%84%3A%B2%D6%1C%120%CAX%24%83%F7%17%81%DD%7F%FF%F4%FAzh%91%60N%A5%02%14p%60%60%F0p%D3%C0@%AA%AA%B2BI%8Fc%3Db_%FF%F0%EE%ED_%19%CF%208%BEg%FB%F6%8F%EE9%8C%8D%0AH9%15%CC%18%8D%60RaU7%02%A1%3BZ%B3%0E%10%EClk%E9%ED%85%E3%B9%AD%F6%5E%C2%3B%04%BD5%DB@%06%D1%24%A3%84%0B%82%FA%E6M%87FjU%CD%D9%0B%87X%7C%F9%F3%27N%9Cp%23%B0f%24kw%DD5%92%97%C1%91%0B%3D4%81h%92%C2%3B%3E%1B%3F0%CE%F7%8E%89%0F0%B3Z4%20%06%AC9%0A%81@Plh%88%FA%C9yh%00%5E%C4H%D8%D3%B5%D0%A5%E70B%AD/%A3%82dM%60%08%1C%A3%9E%D0%11%5E%27XTYO%04%ED%BE%08%EA%8E%24%C5%24%8D%A0%CCc%87%04%7D%27%0CA%D6R%C1%DC%13%3AC%22%02%5B%A8%23%80O%C1%81V41%89%88%E54A5%D0%E5%9C%0E%C1%0FN%20%D3TY588%90Z%EBoL%0D%CC_%AD%AAlJ%8F%EF%C5TP%1F%7F%05%3A%C3V%97x%E3%5E%B0W%1EN%0B%D8Q%9A%D0%C7%C3%11%D4%28%04%B9H%3D%A8%98+%81%0C%BF%BE%A6%B3%B8f%C7%8E%9A%CE%96%3Fa%FE%E8%AE%0E%F29%F0%90%94%9B%D3%D6%80%FE%8A%B9%20%E8%3F%B3vSO%87%1B%82%8Con%EB%97%1E9q%E2gs2%D5%04%D0%AB%7BFF%D0%23%3A%B0%0B%BD%23%17.%F4%14%D3%93%7B4h%0F%F6%8C%5C%E8%98%D69%AA%B0%EFI%F1%1D%8D%E3%7D%8A%DF%C7s7%BF%23%3E%29%E3%97L%8E%A2%CEp%D2L%9AR%22%91%00%04%05%F2%C4%CD%88%19%8B%04%25%D0H%00%80%D1%05%0Ds%DBO%96%22%92%603qw%0D%B1g%CAK%99%B7%1772%10S*%C8J%FCC%0F%E1%13e%BE%CD%F39%CE8%E2%01%1D%C7%07%25B%E2C%1D%8A%86YG%28DV%3C%CB%F2%9A%AC%0B%821%DD%D6%D65M%B8**%AB%14h%0Ar%E9%8DU%95%A9%C6%AA%CA%AB%04%82%AC%96%1E%FE%CD%C1%3D%BB%B7o%DF%BB%F7%20%D8w%3E%1A%3E%8C%13%0C%CDJ%1CA%8DB%90%ABoH%D4%0FE%5D%09dxu%7D%CD%BA%CE%1D%9D%DDh%98N%0E%8A%A4%16%E1%08%DA%1C%D2%08%B2%FE%B5%9B%CE%9C%11%E4%DA%1E%D9%81%20%0B%91dn%EB%ED%F7%FClN%00VH%8EJk%E2%23%D5%CD%A1%F8%05D_q%C7H%E9%06%81%BC%DC%D0%C9%98n%18%19%11%A7%83%BA%EC%23%0AC%9B%E2I%F3%C2%3B%3EktL%B8%60%3Ei%F8y%DC%9B%88Z%08%9Ae%16%82%D1%A8%9F%1AF%8EY%A3%C1h%D8%C60%22I%7C%86%91%F5%80%89%E7L%29%F3%10%28D%DC%16%B4%97%3AYO/%F1%9B%D9@%9C%B5H%22%9C%3B%3FN%D2%F8%958%82+%1F%D2r%9C%F1%12%17%20L%22NnChC%0F%EE-%0D%E9t%F3%1F%8E%93%D7%A6%A1%DD%D55M%7C%0F%08B%F8%3D%0C%E0%CDWUV%0DTV%8D%8Fc%3Db%86C%0C%3Ez%04%10%3Cxp%DBw%3Ez%7B8%ED%C3%B7%EC7X%961%F0@%CC%608%B0%F53%A6X%3F%D4%C0%B8%9B%264%97tvw%B7%D5u67o%10I%0E%00%C1%9CY%8F%FD%29%04YS%E17o%3A%E3%17%14%8D%D7%25%EAl%D1%B9%CA%CA%DC%A7_%9ESuY%23%19d%B9%D2%9A%9Af%B5%B4%1A%10%0CmPK%8BK5%D2%CF%CB%D0%9B%98n%E8%986%0DU%D1%C9%E6%9C%CF%1F%7C27e%F5IQ%25zv%BC%9C%88%26Lt%BC%B1%06%D45Q%A8%7B%8F%E5%C3R%C4j%29Fba%5D%C5/%21%BA%3C%D0%CA%F4I%C1%3C%90%5D%B7%97DP%B4%E6v%9A%89%05%C3%114Ez%CA%AAh.%9C%1FG%CF%A8%26U%90%DD%10%A2Lw%99o%A6%D5w%AC%9F%1E%D9@%1E%E7%60%D5%C0@%D5%E0%81v%08%B2%F3%E8%E3%FC%C6%8D%C3%F8%7CAN%EBO%3F%F2%DA%C3%3F%FD%E8%A3%9F%3E%7Cp8%BD%96%27%E5U%12%D4%A4%84w%D2%F0%06ldff%A8%7Eh%C6%94A%1D%81%03%C7%21q%B2Qj%CF%9A%16%05%FA%B1Sd%886%F2y+%27%24t%D665%A1R%A3L%16%82%C2%DA%B9%B9%C3%AA%A0k%A4%92%01%08%1D%CD%8A%AAV%F7%5E%E8%ADV%11d%BC%03A%E8%E8NO7%98%A2A%23%C8jB@z2c%12E%20b0%99%88f%ACA%15%E8%15%09%2C%DF%10.++%83%10%DD%10N%C8%C4%BD%8C%94%0C%AE9g%86%A9i%AE7%EC%25%7E%D3%0C%D3f%E6%8E%82%95%1D%94%95%E5%A2%29%CB%F9dY%CE%E8%23%7C%A2j%15%FC%10f1%F3%CBN%04%D9%E9pXe%A6%E9%26%F6%8A%A6%A6%9B%FC%F6%DA%90%C6%ABW%AFZ%1F%C8M%FB%0E%8CZ%DF%CE%8E%1E%D0%E8%C6%91f%18%8AB%1C%B5%BA%E0K%CE%D8%16T%15%C1%12%23%D7%DA%E35G%A8%CC%ECW%D3%D5@%C00D%23%A0%EA%3E%87%AAkJfNJ%22f8%CE%D6bPWU%C1%21%82v%5DBmU%F7%F6%F6V%0B%F0%0B%F4%CDa%C9%20j9%83%08BES%5B%E6%90FJ%0D%0De%E0%16x%C7%7E%A1%AC%02G%1D%00%B4%7D.%97%80O%D8Q0bj%D4%3D%C7Z%E1%94u%9B%A9z%83%5E%B2N%FD%24%27%C4%05Ej%CA%E7%02%1A%C2%9A%BCM%08s%3B%0C%EF_X%A8%C3%7CG%C7%FB4%80%5Bs%CA%95%C5%09x%7CyN%13%DD%84vQ%E2%90M%84%DFP%7D%D4%D6@n%F1%0Ar%3B%24%AB%EF%E9%CA%28%90%A4%E9%BA_%D0%1D%F7%E3%C2QC%8Dr%AE%25y%D8hiuuui%1EyF%DB%B6%5Cy%0E%CB%AE%0D%F73%B2%C2b%5E%1E%7C%82%A0%80%09%EE%CD%E3%82%15%AC%60%05+X%C1%0AV%B0%82%15%AC%60%05+X%C1%0AV%B0%82%15%AC%60%05%FB%7D%B4%C3%8D%60%87%97%EF5%A7m%5B%8F%1E%C3l%DE%EC%9E%B1%07M%BCZ%DE%E1%A8%09%CA%A4%E5%95_%C4X1%11%89D%12%E2%F2G%B5%C9%CD%B8%1C%B8%88%DBgz%D0%F9%CDlFkd%9AK%85%EB%F0z%9A%11%A1%EC%B7vF%EA%60%E3l%F9kW%CAgg%0F/%CF+JRd%DAJ%B95%1D%91%0E%0Fo%7Bt%C2mp%5E%B5%9E%C4%BA%E6%99%CAg%09%1F%99%06GNx%FC%F2%CAM%13%9BW.g%EBBCmKKoK%BC%B6%21%DFe%11%EE%BF%DF%EDL%C8%1Ce%87%FB%FB%0F%D0%14%E2%EB%1C9%FA%82%89%C1%05s%B9%96%1E%5Egn4%DC%AB7w%B6%D6%B5%B5%AE+m%0E%3A%B7%BA%88%D7%D3%F8%08%CB%11%C6.%9A%5Bf%09%19%90xE5%A4%A3%E4wj%E3K%FF%DC%F8%B5/%FF%FC%9E%C6%B7%5E%9AU%E9%12%5E%5E%D0%A6%C0tG%28%14%EA%9EV%99%AF%3E%F8%A7w%3C%BA%C9%E5%D1%D3%10%BA%D0%C1%21f%19%96%F0g%E5%15Y%E9J%C1%03A%7Db%F8%E4%F0%04%F5%E4%9E%3D%5CQ%94*%AApM%0F%284tu%B5%B6%B5%B5%B6%C6%5B%EEug%90+%AA%A8%B8%DF%E5%7B%5C%8B%EF%F8%EA%BE%7D%FB%F6n%5E%D8%81n%C9%84%C1%DC%91%3D%02%96%A3%D7%DD%06%5D%3F%E6%BE%12%23Aw%AFs%E5%16%7E%FE%A5%EBj%9A%8Bk%D6%D5%D5%EDp%C9e%E7%ED%F5%D69%E5%28GNp%E1%8F.%22%24%DE%19%90tE%3D%20%C5%CE%5BF867%FE%DB%3D%27%DA%DB%BFt%B9q%F6%9E%7B%FE%AD%91%A2%CC%D3+1%9A8%1Djnn%0EM%8B%FC%DB%0F%FE%F5%1D%DB%5C%11%94%0C%C3%88x%20%C8F%D1%C9%97a%DF%24%F4ilv%CE%B4%A2%E0%08%26IE8%B0%E7%BE%7D%F7%ED%D9Ll%F1%A6%A2%8A%D5%AB%BE%FB%BD%D5%15E79%F6%C6GZ%DAZ%D1%22%84C%DD%3D%BD%0D%EE%89%D6%EE_%BD%BAHu%7E%0F%E7%EB%D32%CF%DA%DF%3A%BD%EF%A3%9D%AFL%2C%C8o%04%3D%A25%14%C6%1F%B4%E7%B5%05%15%17%04%7D%7E%BF%24%E9%3A%EF%86%A0%18%F5kBDu%F3%E2%FBE%0F%FC%09%04%EF%DDP%AA2Js%CD%BAu%EBv4%3Bn9o/%E8%1Ce8AF%92F%D0t%A9%14%E2%1C%BC2%20%1D%DE%0F%E8%8D%9E%EA%9F%98%98%E8%27%11%1C%3Cs%CF%89%13%27n%9F%9D%9D%DDz%E2%C4%3D%9B%A9tf%9E%5E%0C%C1%84%AA%BE%F2%E0%03%C3nw%C9P%3D%1C%FC%94%03%C1%24v%EF%99b%20%80%9F%3C%81%60%F3%B4%CF%9F%20%F6J%D8_%3D%F5%20%FC%99%20%0E%BA%EA%7Bh%E9ge%E5%EA%D5%29g%F2%D7%91%AE%BA%96%96%0B%17.%C4k%BB%E3%23niI%B9%A2%D5%03%95n2%88P%C8%D8Co%9F%DE%B6%93%FDM%EE%84%23Ah%B1%1A%D6%CC%92%9DV%D4%DA%A9%D11%09%10%F4%A1Iu%BC%1B%82zT%EBY%D7%A3Ed%DE%1DA%DF%C2%DCu%F8%05%BC%0A%9A7%A0%DBH%2C%DE%D1%B9n%5D%A9C/%BD%BD%A0s%2C%8E%98L%E8%5C%C2%EF%A3%10%D4%BDgmyg@%3A%7Fj%02Y%FF%A9%D1%F3%13%24%82%8D%3F%FFo%27%16%EC%BF%BDFf%F4%F0%F6%12*%C8%F3%7F%F7%D5%7E%EA%3E%83cFK%C7%23%91hxfh%08S%05%E0O%60x%9E%E3%AD%AB%A5J%9125%0F%82%CD%EB%D7S*%08%15%F9%D3%D9%C9%D9%9F2%F64%9E%07w%7D%EB%C1o%FD%D9%01%EC%17n%AAZU%99%B5%D5%29Z%07%1BZZ%DB%80%C0x%FCPmmO%8B%DB%B4%D7%D8%FD%C0%EE%EA%22g%AB%06C%D0%F7w%FBN%BF%F2%D1A%5C%05M%D3%D08%3D%D3%131%04N%A0I%02%04%ED%C4D%00%99C%90%22Zw%5DK%5D%8F%16%D5%9D%5E%7B%BFH%FFl%12H%15%B4%EE%E5hq%27Xi3%1DL%BD%BDH%E70%06e%1F%A1s%11%9EF%90%CB%1Bf%BD%F3%23%D9%08N%9C%B2b%F0%FE%A3%0E%04%B7%DE%F3%B5%9F%23%A9%3Bq%CF%CF%DE%9E%28%A7%11%F4%F2J%8C%0FSA%9E%BF%03%AF%1A%5E7%87%86%C2eb0Q%8F%E6tK%093%9C%3B%BD%08%C7%09%E1%FA%A8%22%DB+%02d9%22%12%08%CA%D39%02%D7O%FBt%0A%C1%D95%1B%D7%CC2%B2%1C%909%F1%A5%BD%BB%9E%DA%F5W%F8%9C%EA%A2%EFU%E6lu%11u%3D%23-u%DD%C0%9F%A2%04%8C%DA%DA%DB%5C%EA%D4%07%22%E8%7F%A7%B2%A2%C8%81%09%9C%EFB4%7C%E8%EF%B6%9D%DE7%9C%9BZ%1A%09%26%25%91%B7fW%FA%05%BF_%D0%84%A4K%20%86k%0C%22%C89UP%14%7B%EAzo%8B%D7%1D%12U%CEE%05%7D%F9U%D0%88Z%11%D4%82l%5D%B1%23%98zz%3DuN%8F%D1q%98%E7%12%F9%1A%83%DE%F9%912%08%9E%DF/%25%0DU%E7%9D%08%BE%7B%CF%CF%C1%7E%F6%E9%95%F2%F9F%27%82%1E%5E%0BA%D4%1D%E9%986%E8%BBS%1D%9A%89%88%AA%A2%26%1A%1A%ACD%00%B1%98i%CE%60%08j%11C%0D%97%E9f%2CcV%5B%DE%81%603%B2iU%A0%10%9C%DCX%BEf%92U%92b@6%03%07%8E%FC%D5+%F8%CA%92%C3%15%AB%17%00%1C%80%88Ju%E4%1B%BA%BA%BBG%0E%DD%7Dw%20%18%EC%E9%B9%CBE%05%EF/%AA%5C%A5i%ABV%17%95%D1%1E8_k%EA%A5%7D%3D%1Ez%F8%AB%D8%C4%7E%29%29%9A%3E%D9%C8%8E%C7%18%9A%83@%1D%10%D4%F5%A4%08%CA%CF%04%E9K%19%D1%3A%BBzom%ED%EE%96%23N%AF%B5_%CC%08%043%C1%B4%B8s%07%82%8C%0E%A6%DE%5EO%9DS%CD%EC%BA%09%3E%7B%CA%5C%9E%C6%20%91%01%C9%03A%C7%A7%0Cd%8F%BC%7B%CF%3Dk%D6lY%B3%A6%BD%DD%05A/%AF%C4%18%C1%0C%82%92%AAB%CDbU3%14T%83%E1p4%2C%28V%82%87H%2C%5C%9F%20%11%9C1USH%B0%0B%8B%A3%84%DC%D9%27%B4%E9%1C%81%D0%16%94%1D*%B8%E6%96Y%96%CD%26%7F%23%D3%7D%10%04%22%06%C93%DA0%D2S%7B%F3%05%E84%08r%ED%C8%21%E7%E2X%0DD%F0%89%99%A1%27@%3F%E9%BE%15%9C/%BFp9%90%98a%7BM%26DCm%AE%26%0D%CF%AE%19%88%05%D1%E2h%01e%26g%82%B1%00%B5i%A5%BB5%DE%D6%D5%DD*%88N%AF%B5_%24%80%28%0B%09U%CFv0-.%CE@V%5CL%06SO/%E8%1CK%12%C8b%3A%97T%B1%99%CE%99edA%F7%94%FED%06%A4%E5%23%18%DEt%19%20%BB%E5%96%5B.%B7%CF%A7f%0FS%9B%C0%BC%ED%F3%B4%17%EEN%21%83%A0%8AP%C0%ABf%26%26J1%B3%7E%28%12%09%DF%BBa%03%60X%3F%D4@%20%A8%84%EB%C3%AA%91Pj%B2%E6@%B0%B99%8B%E0J%12%C1%FE%B3%93%93%93g%FB%D9%EC%A8%21%89%60%11%B4%04%17%FEg%60%605%95%EEK%BE7%DEs3%87%F0%95%83%BD%1B%9C%A32%F7W%80%08%CE%0Ci+*%8B%A8%7C%BA%E8%7C%19L0%88%F3%0DDEyC%17eXB%85@%8C%09.%046.%28%92%94%05%12%E6%CDumm%F1%BA%9B%A3%86%D3%BB%B0_4%F2C%D5%B3%1DL%8BQ%8F%17u9%90%D2%E1%9D%06O/%E8%1CoM%C6%CE%02%C8%E2%3A%97%D09%1AANw%7FF@d@%BA%0E%04%D7%9A%AF%5D%D9%B2eMyyy*%95%A2W%E3c%DE%C1h%C4%A4%11%E4Uh%0BN7wL%8B%8AB%A9%60%D4%8C%95E%86%86%A6%C2%0D%D6r%DD%C8%D4P%04GP4d%08%D3RT%FD%7E%C6Jp%04%7D%D3%CD9%9B%F6i%24%82g%C7%C0.%5D%CA%AE%3C%A5%B2%0F%A6%BEK%22%98%A2%AA%C2%EC%88%DF%ED%0B%A8%8A%A6%5E0%9C%22%A8%17%AD%AE%D4%B4%A1z%9F%062H%8D6J%18%80%7C%1Fy%BEJD%E4%9B%BBZ%09%8B%E7%10%04%02%B9%E0%C2%08%80d%F8%FC8e%E0%8D%19%F1%BAP%DD%05Srz%ED%FD*J%D8P%8Cp%98%AAg%3B%98%EA%C5%C0%18%D8%BAb%3F%D9i%F0%F4%82%CEY%E0e%0D%A5%B3%C9%E9%9Cc%C0%86nI%12%08%E62%20%5D%17%82%BA%AC%06%0D%23%D9%04%086%85%C9%D5%099%AF%04%CD5%93%F4B%D5%08%D3%CD%D3%3A%FC%11%1C*%08%BF%0C%CA74%05%0284%03%FD%E20%81%A0%02%17%5B%D0%D5%A8%A2%96%94@%7C%28%01%C3%11%E4K%F1%A1i%8DB%B0%FF%12%B2QN%B3%B2g%A3%9E%22%A1%82%DF%F3TA%86m%B8%FB%EE%0B%01%05-%B6hp%0E%5D%83%08%AE%B0%10%F4%BDXY%F4X%7E%04%E9%F3%E5%C3%A2%BF9N%22%D8%93C0F%3D%CF%83V%19%96%90%09%3E%FA%A2bo%DB%5D%11Uj%88%FA%28ov%BF%91%08RA%99%DCo%26%98%FA-%CA%2C%C6%F0%60%EA%EDM%E8T%87%83%D0%B9%08%F5%88%0A%25%60%F3%E5E0%97%01i%B9%08%B2%1C@%A6%0A%16d%A9%14%93J%85%F1%152%B8%D7F%90X%3F%03U%A3OO%EB%1A%FC%25%E7E0%5C%3Fc%AF7%22%10%84%C0%ED%89%A0%0E%B2%FFmh%FDX%B99u%0AA%8E%7B%E8%A1%87r%12%C8%93%E9%3E%EC%B6%E0@%D6%E8%B6%20%B4%F6%1A.%04%83%8A%AA%F668%9F%C2%09%96%08j%F5ah%83%83%0C%92%FD%82%05%04%FB%EC%9D%93jd%28%1D%24%82m%DD%B9%E7%111.%C8%B3%CA%C2%11%F3%BC%A4%E2%08%82%97%8F%96%99%0D*0%28D%28of%BF%5CFBeb%BF%D9%60%8A%28C%8C%91%C1%D4%DB%EB%ADs%11%9F@%3F%3BQ%F2%23%98%CB%80tS%91m%F4h%98%05%5E%12%8DO%9F%3FN%20%E8%0B%1B%26DD%80%AC%29%F5c%E5Lj%12_%7E%87y3%08%12%8B%F3%24%26%28C8U%E0%17%82t%20%9E%89H%16%82%F5%F5C3L%04tp%A8%3E2%B3P6%A2%99%A6%5C%3F%15%8C%A8j%89e%3F%F8%C1%0Fp%04%E5%85%DB%D22%AAG%2C%B3%3A%B4%039%F4%C3%A9%BC%EAS5G%8F8G%60%95%F3%C17%C8%A0%F9%D2%01Av%EB%0EWT%9E%D14%1FZ%B5%C7%BFH%8FO%C3%F9ZG%F5u%CE%05%C1%B0%D1%D0%D3F%22%18%CA%3D%7B%89%C9%26%A7h%B9%0B%CD%C5%08%04-%AFn%EAl0%E2%97%23%94%D7%DE%AF%82%A2%B0%95%E5E%23%BAA%D9%60%0A%D1%B6X%A7%83%A9%B7%D7%5B%E7%10%82*%5Cx%23j%9A%D1%A0%04%BD%FC%BC%08%12%19%90P%B2%3FdX%03%88%3F%10%05%F0%26%CEs%B1Q%EB%E1%C8%A9%F3%18c%BC%0C%3A%27%06%11d%20%82P%3E%85%25%27%C2%BD%19%04%89%D4Epw%B2%99%98%C4%5B%0B1%09%04c6%82@_%24%99%80%7F%EB%C33%B9Q4%0E%04%AE%A1%C1%88%24U%0B%3FdJ%EE%F4%94%85%192%B6%1D%C5%C7S%01AN%E6%00y%F4%E3%82%20%07B6%90C%B0%C8%F9%CCPm%10%B4c%FC%CD%1B%9C%23%0Cj%D1%EA%D5@%20tG%D0%A5%03%19%24%9E%AB%C2%F92%99%F3u%22%18%93%3A%E2%04%82mmXo%04%20k%88%E6%E4%24%1Ai%08%C4%96%EA%C5%F7+7H%A6%88%EF7%A1/%94*%CEn%22%17Ls%A1V@Q%D8%11j%05%D6L%8A%C1%00Z%18%AD%C8%D6%0F%81%60%02z%18%28VE%03%82%A8*B%2C/%82x%06%A4%D4%C0%8F%FF%A0%F2%EF_%1C%C0%10%3C%BA%BF%7F%E2%E9%A7%FBG%F5%F3%13%FD%FD%C7O%8D%9E%3F%8A%CB%9C%1E%BE%C9L%22%C8%06SM%CC%10%D3%94%1A%C4%10%CCy%B3%08%AA%14%82%E8%EE%CC%18G%21%18%CD%20843UVV%EFDP@%E9%0C%23%C6%0F%B2%26%E0%A7%C7%26%04%5D%D3t%BF%A0%AARBLJ%92%8E%EDU%26%27p%F0%24%82%ECM%A9%1C%83%95%A9%15%CE%A5%B7%C2%FA%24%DC%8B%07%5C%AA%13%89%20%92%5D%EB%A1%22%7F%86%92A%EB%7C%FB8w%04%93%B1%9E%B6%3A%DC%DAB%D8%C8%60LP%8D%A4%95%9CH%B2%EF%AC2%02AO/Q%CF%C1%06%D1%C0%F7%1Ba%23x%DA%9F%19%F8%13%C1%82%E9b%A1%16%21h%18%28%27%80%13A%C1%97%D4X%AEO%06q%D1%92%AA%20%0BI%9F%90%0FA%2C%03R%AA%F2%8F%E1%8B%BF%C1%11%3C%3F%3C%0C%DA7z%3Ex%FE%D4%A9%D1%D1%FD%E7%F7ci%009%80lS2i%00d%20%82%8C%1F%FA%93%A9%1Cd%98%D7B0*%BA%20%98%11%05k%99%3B%8E%20Z%DFo%3D%17%B1%82p%00%FE%A7%BE%1EC%10%25%15%08%26M%943%CA%08%A2%97%E8%29%98%0A%B2%029%15A%10%B0%DC%9B%124Q%1D%D5%8A%25%0Bc%B9%C1%2C%83%95%95%A9A%9F%CB%F2%EFX%FFw%BE%F3%8D%09G%82Vf-%88%A0oa%EC%99%E3@%06%F1%B7%9AI%84%06r%7FB%E4%B9Q%23%F1V%9C%C0%D6V%3C%C7e%0C%DA%9E%01%03la2%A1%11%5B%AA%97%ACg_P%25%10%F4E%F8%5C%16%03n%06%7E%25%82+%D9%22%A1%96E%15%AF@%DF%CC%89%60%00%F44%86%AASS%0Dt%7Dx%F4%F8%20%CF%F46%3C%03Rj%605%7CSI%20%98%3E%D5%7Fjt%7F%CC4%CFA@%3E%97%144%22%10%1B%D0%D7%07%9D%1BL%ADb@%05A%06ob%9D%5E%84%A0%994%C3%92%CE%E5QA%1AAq%26%11%8BEPW%C4z%11%05%FAT%9FK%D1%12%0D%0A%02%DA%A0%29E3%A1%07l%A1%E5%C4*%92%18%F4%1B%B6%89%82b%A2%29r%0B%FBMJ.%96%1B%EC%82%C6%B7%3CXT%81%1E%F3VV%15%C5%04%D9%25eB2%06q%21%E6%7C%E6%7E%7F%05%16%C2%ED%BE%0C.%83%04%82%7FH%9E%AF%CF%08%AF%23%AC%B5%1B%1F%F11cB%8C0Q%C2%1E%ADx%7B%3D%EB%19xcs%0823%F6W%18d%9AG%A8%15%7C%0E%23u%AEL%B5%18%94uA%D5%0C%C7%D3%22%02%C1%85%0CH%A9%81%FF%09%BE%F9%03%1CA%01%C8%DB%1F5%93%D6%3B%3A%83%AA%22%E3y%C38M%17%C3%91%9B%FC%FET%B6D*%E5%E2%15P%C2%93H%B8L%E1%A9A%99%CC%25%C9%24%FB%C0U%21%00%11xH2Ds%C6R%C3%99%88%E8%CF%15V%A2%D1%88%08%08B%10%B0%D2%BB%88fD%CEy%0DI%84%C8d%1B%90%E8%87%08%25%92%B9R%E0%A6%F3i%B2%DF%AF%CB%3Egj%17%5E%0F%88%8F%DD_%94*%BA%FF%B1d@%D0x%17%15%9Cx%F8%3B%BF%D9%E4L%12%5CT%B1%9A%B2%BC%08%EE%A4%CE%17%9AX%D4%A3%91%F5x%C2D%3D%A6%DAz%93%7D%D2k%C4%94%A5z%3D%EB9B6%CF%9C%08%CA%F9Cm%20%E2b%A4%CE%25T+%0B%89%2C%F3%A2%D7%B4a%3C%03%12%20%F87%AB%FF%98@%90%B1S%FC%A0%B6%A8%95%D0%84%CC%B5%C3%F9dE%3A%17%0E%A7r%E6%E2Ev.%16%D0%C9%C2%9ES%293i%7E%A0Z%ADA%19S%D5q%16X%9E%3As%19%F3J%9C%E6%CB6nPnAhDK%F2%92%E7%FE%5B%897%83VL3T%C1%91%7D%08%1D%97%1A%83%06BLp%E4o%C9%8E%27%60%86wY%3C%CF%D7%CE%10%85%99F%E4%1F2c%94%99K%F7z%ED%17xs%18%81%A0%8E%5E%DD%E9%1Ej%97be1%1D%E8%F7%29%B12%CF_%C32%20%A5%AA%06*%E1%A7*%B5%B4%1D%2Cd%A9T%A3%0B%04%DE%E4%E2%0D%A0%2CV%8E%3C%3F%8E%09%E5%06%B5i%8E%E3m%96dG%0A%21+%B7%10%96l%09o%B1%B9%04ZS%F3x%FD%14%7DF%28%BDP%86%02%97D@pJ%BA%FD%B4U%D7x%E7%D84%84%1D%01%A9%11%3A_%CA%BB%C8%F9.v%5C%BC%AE%1AH%FA%93%A2%EA%C8%24%E6%E9%F5%DA%AFw0%5D%3C%D4.z%D4%02j+E%25%FF%92K%0Cf8%1A%5Cr%89LS%D2%27%0BV%E21%3A%9F%99%ED%B5%9A%C3ny%B7%E0%D6%CF%5E0%D74U%F9sry%A5%EC%B2%9F%5Bf%12%ACjN%ED%5E%F4%8C%EC%09%0C%F9R%81Y%D9%BE%7C%3C%9F/%87%15B%D8%C7_G%CE%AE%C5k%3AS%93%CB%F5%E6%AFg%EF%60%BA%84P%5B%B0%82%15%AC%60%05+X%C1%0AV%B0%82%15%AC%60%05+X%C1%0AV%B0%82%15%AC%60%9F%BDq%07f%F3%A7%CC%A22A%B9%AD%D03%CB%B2%23%99%86%E4X%3Ee%E6%9B+%F0%D9%98%B5%E8%A4%C1%EB7%C6%DC%9E%99%2C%FE%9E%8AMi%E6%BAm%C5%FC%1C%D8%FC2%C6%FE-%D3%A9%BA2%FDK%F7f%CC%F2%2Cw%CB%60%7D%EA%8A%C1t%7B%7B%7B%BA%7F%85%DAw%FD%27%7E%DDvx%B6%BC%7CvS%3E%AF%A8%E0%E6%92%15%C90%95%2CX%A6n%D2%0CJ%C4%DA%03%8E%AC%1F%B2f%E8%A2%A6%FD%FE%AD%3C%5E%C4%91%10A/W%96%C2%11%21%FF%DBN%C6%5C%18t%CC%A4ql%5CH%E7E%90G%8F%97*nb%F4%9B%9A%E0%DF%26%FA%F5K%CCMs%ED%5B%DE%00%DB%D2%3E%B7%C2u%03%ACj%BD%84%86%7E%08%A6%07%A9g%DC%BC%28%2C%D5%8B%7EA%B7j%9BC%EF%A0%D2%93%FA%D2%B7%CC0%FE%F2%E7%FE%FD%DF%FF%23c%FF%FEL%B9%13na%16%DDTs%B37%F2%D4%C4+%3F%D2ly%7B%7B%F9l%BEk%28*%D8%8B6%DD%104e_%26O%5E%D2%F0%91%29E%3AJJ%24%3E%FBH%1E%19/%95%94%60%AF%1A%25%81%A4o_%C9%9F%B8%FBP%DBH%C2%EF%EA%5D%3B99y.v%D8%B2%D89%F8%9F%B5%8C%AB%8D%9Dr2%88O%D7%B1%E7%12R%BF%22%A4w%E5C%D0_%81%DE%AF%BC%3A5x%B8b%B5%F5%A2%E5%26%D2%DF4%FF%C6%96%CB%97%3F%FE%F8%F2%E5-%5B%E6g%5D6%80%DE%84%15%08+az%15%8F%C9%FBr%13%11%ED%F5%F0%E6R%BD%80%99d%06%10%822%27%E9%BA%24%97%E9K%DE2s%D3%F8%BF%3F%B3g%E3%C1%23_%06%3Bqp%F7%C9%FFx%86%5E%D31%3B%F7%F9%23%DB%1E%DCwd%CB%1C%7DB%83%A9%9B%94%A6%22%B8%27%8B%06%D5%15%9E%CF%7C%F9%88%22%90%86%CDw%04%04%1B%CBgoR%DD%29%24%10T%5D%10%2C%E3%7D%9A%05%9EO%F2i2Q/%25mu19K/%3A%029VWW%82%A3%C0%E5%99%60%8C%2C%91%18i%EDji%EBI%C8n%DE%C9%F4l4%22%D9%DB.%8BDgg%27%DDO%7D%EC%94%93A%3C%0F%95%FD%26c%F27%84%F4%B5ky%10%D4R%00%DEj%F8S%91B%FF%AE%5EE%CD%DCoj%7F%04%F8%B3%ED%F2%1B%EDN%06%A5z%00%D0z%05%9B%E3%A00L%2CT%F0s%F6%F6%02%81%3C%0Fb.p%1Ag%96I%3E%96/%A3%B7%8C%9E%A5%FF%04LC%A9%0F%88%B2M%20%7C%5B%FF%F9k%8F%7E%F3%81%07%1E%FD%DA%3F_%29%DF%F3%1F%E5%C4%81%C9s%5B%B6%DDw%F2%D5g%5E%3D%B9%7B%DF%9A9%F2%D5%EC%A9%F6%7FJ%BD9%3F%FF%C1%FC%FC%D5%D4%3F%B5%7B%CD%7CQLE%24%DBt%D84%F7%C3%96%0C666%BA%BE%1AJ%B4%5El%16%CC%1F3%A5%24%EF%0B%22%06MA%F3%919*J%DA%3A%C3%825%DB%1C%19%7CP%22%9D%CB@%B0%A7%B5%F7%CE%DBZZ%7B%12%19o%23%96%A9%81%9Dl%8FrZ%E6%E5%A2%11%8D%8B%A6%9D%08%B2%07%D0j%E2S%EF9%18%04%04%E3%98%F1%14%82@%E0%9E%5D%E9t%BA%7F%B3%F3%A6%1C%5C%BD%CA%9E%26%B8*%FB/%21%83k%E7-%02A%01o%B1%18%9C%A7cq%12%ADU%A8%AF%9F%9A%3AGo%D9%CE%C9%A1%5B%B3%20U%9D%CE%CAAg%EC%A0rvH%3A/p%28%D4XsE%20%1C%E32%88%10%D4%7E%F2%C4%13O%BC%F3%CE%3B%F0%F7O4%B2%ECM%ED%FF%FE%1FO%ADY%F3%C6V%B0-k%D6%3C%F5%1F%17%C9%802w%FB%E9_%3F%F3%3A%D8%AB%CF%5C%DC%BBq%8E%F05%CD%CF%CF%BF%99%B1%8F%FF%C9%B1%12%113C%A4%11Lb%1D%0B%D4%18L7%9EN%BB%B6%073*%98%3Ff%FA%24%88%B0%80%A0%60%FA4%3Fyk%03%82%25%94%D1%08%E6%26%94%3B%11%1C%89%DF%D5%DB%1Bo%EDN%D8%DE%C6%B7%B0l%21%80%60%84%C9%AEw%04B%23%ED%0E%047%8F%8D%9EB%F6%9E%83A%84%20z%85%BC%F5W%07%8D%20%10%F8%DFw%EF%3Ay%F2dy%7F%3F%FDFK%C61c%B5%92X%08%3F%F7%86%A5%81%5B%18%E6%8A%A5%83%5B%E6%A8%F2a%29%11N%A0u%3CSSTD%B1%E7%DEg%E6%E1%AAN%04y/%04%F52Y%B7g%9E%D9%04%12%8DA%28%0B%F8%7D.k%EF%3CA%96e%CAo%D9%B3%EB%A9%AC%ED%DA%FD%15R%04g%B7%9C%7E%E6%99%A9%99k%D7%AEMM%3D%FB%EC%DE5%84%B0W%5C%9D%AFz%F3Mh%26%BE%F9f%D5%7C%15%D5%26%21%AFf%80%EC%D8%8A%C1%20%AE%D4%9B%1A%1B%D3%DB%F7%CE%BB%C6%F2%8C%0Az%08%96i%F2%9A%20B%9BP%83PL%23H%5B%C52T0%DE%D6%D2%DB%D2%15o%B5U%B0%F1%E1%F2A%07%82%0B%85%9D%08n%1E%B3%F0%B3%EC%D4%185%916%83%A0%9B%0A%FAA%03w%EF%DE%7D%DF%EE%FBN%EE%1AO%D3%C9%85R%ABr%F0UV%0ETUU%ADN%E5%0A%AF%9D%DFb%85%E05%0C%B3%D5%FE@%F7%8B%A7%D0k%A1g%86%C2%91h%84%12B%1B2%C3%00%01%D4%0D%C3%15A%B4VVqE%10%221%97%99%01%8B%26%A1I*%B5%E5w%3Ew%E6%0F%FE%E1%0C%D8%3F%7C%EE%CC%99%CF%91e%85%F2on%DF%B8%B1%BC%1C%C5%C2%F25%1B%B7o+%C7%7B%1D%C2%DC%B6%8B%CF%9Em%EE%86%5B%B26r%F6%D9W%0F%CEa%5E-%05%1Ax%FF%CC%D5%0F%EE%9F%29zs%60%3E%05%FB%CF7oX%A2%08%04%06%F1I%D6Zcz%CF%F8%D5%AA%CA%D4Zg%E4YP%C1%FC%B4H%8A%A6%C1%265-H%8D%B9%94%B49%CDM%05%CDh4%12%0D%3A%10%0C%B4%F6%8C%C0O%8F%A5%82%24%81%19%04%B3g%E3%82%A06%B6%00%A0%83A%12A%9E@p%13%10%88%10%DC%BDg%CF%F3%17O%FE%9A%CE%87%98%CA%25%26%AC%B4%17%C3%E2%08%CEn%B1%1B%82%97%19%E6v%FB%C3%E5y%B2%BC%95L1%8CVj%85%23S%AE%08%A2%0AvGP%90%85%200%C8q%7D%86%03A%C6%E4%7C%3C+%94I%A6%CC%F2%BC%26Q%5B%06%F8%7E%C20/%BE%C80%7F%00%1F%C9%B2%83W%F6%C1%1Dw%DF%DE%D3%DF%3A%BD%F7%3E8%EF%83%8F%E0%B7%CD%EC%E7%B7%3F%FB%CC%EB%C3%DD%20%94%B5%DD%B5%CF%7E%B2%E7%16L%06+P%00%9E%F9%60%E6_%E1%CF%9B%03%03W%07%19u%A0%CF%95@%9Dn%0AB%246s%F7%B7%1F%08l%AF%1A%A8L%CD%A7%1B%1D%C1%18%A9%600%18%F4%8A%99%AA%C4k%B2%01%1C%D2-%EC%12r%CD%A2m%0E%15%D4%A3%7EM%96o%16%22%D4%60@%2C%D1%D6%1A%8F%B7%B5%25P*%10%88%C2Mx%D2.B%05%FBx%27%82%A3%24%81D%24%86%C6Q.%10%C7%D1%DAB%CC%8DZ%82%7B%F6%5C%7C%1E%D9%27%7B%D2%FD%E4fsI%B92%CB%B1%ABVc-%A0L%1CF%08%BEawI%D6P%91x%0A%25.%89D%00%C4%FAs.%08%EAY%04e%CE%81%A0%0C-%0E%B4bK%017%E9-CK%089D%20%C7pe%1A%CA%87P%26%E1%0B%0A%01A%84%1F%B2%17_%7C%91B%B0%FD%CB%00%DE%E8%F0%DE%A9%E1%FD%F7%0D%8F%EE%DE%B3%FDH%3B%E6%9D%3Br%11%21X%BBg%D7%AE%03%B5%AF%3E%FB%C9%AF%F0%D6%60j%7E%00%18%FC%E0%83%0F%3F%F8%E0M@%F0%CD%8A%A6%81%81%01W%04U1%90%A4%2C%80%8D2%8F6%A6%1BS%95%03%A9%F6%3D%7B7%A6g%A9N%C9%12T%90%91%0CP@EK%D2%C3F%CD%25.%86-%5C%B4%B8V%A3Z%0FB%B3G%8B%10Q%AFA%14%13%D0%10lK@%3F%0F%118H%A4%8D%23U%D0%05%C11%D4%06%CC%DB%16%F4a%BD%11%0AA%C4%E0%AB%CF_%7B%EA%A9%93%AF%3E%F7%DC%C5%C7%A9%9EqS%06%C17%B3%04V%E1%DD%91%B9G2%9D%E1%FF%8B%B9%0C%17%C6%0D%C1sS%F5%B6%0D%9DuA%D0X0h%D09%021T%BD%DF%CD%5B%A6Y%0B%24%19%EB4%B8%24%0Bw%B4%CE%11%08%BE%88%2CC%E0%8B/R%08%7E%134%BF%FC%FCT%FF%E9%FE%A9%F3%C3%BB%F7%3C%7E%82@p%DB%B3%CF%3E%FBzm%ED%9E%A7N%F6%87@%05%9F%FF%3C%81%60%0A%B7%D5%A9%01%C6%9D@%A6L%0D8%964%E2q%CF%B795P%95z%FF%B9%E7%5E%DD%7Dz%B8%91%7CO%81%88rqx%AB%A0%D5%23%91U%D59z%2C%E9B%C4%88%19%D1%60%22%183%12%86%19%11%FCD%3F%0Eq%1DE%F9ko%BB%B5%AB%AE%5B%23%D2%7B%82R%97%A1%A1i%A3%8Fw%12%88%A9%60%1F%5C%9B%95%AE%08%9E%3A%95%B7G%EC%C3%09%E4%9C%3D%E2%E7%AF%A1%97%C1%8F%3F%F7%EA%1E%0AA%3Ee%E7%A3Y%20pu%0Ak%FD%CC%D9%E1%F7%E3%0F%FE%0F%E6%E3%0F%3E@%10%D2%08N%F6%87%1B%D0%B8l8%3Ct%60%92%3A%28%60%0CG%F0%BF%D0%08Z%A7%EBw%F1%96%F1%F6%1A%03%29%F3%AB2%EA%3D%3B%10%B4%3A%E7%ABV%C0%27b%CB%CC%AAo%A2%96%C7%7D%B6%EDF%08%AE%22%10%FC%E4o%9F%7D%A6%FB%5Eh%0B%5E%0B%85%9E%7D%FE%7D%12%C1%0A%CD%80%1D%07%D1%80%3B%07%08V%E4%1B%1A%944g%1E8%99X%EF4%98%AAx%FEy@%F0%E2%C9%FBN%977%E2M%C2%A5%A8%20%23%1E%F6%C9%8A%CBC%3EIWcI%D3%90%0C3h%8A%A6h%C4%FC2%85%A0%20%1A%28%7F%ED%9D%B7%F5v%D5%8D%88%D8%90N%02%08%B4%A9Wt%27%81%B8%0A%A2%BCPLd%DE%81%E0%A9%F7%C6%E4%7C%E3%82v%20%CE%C4a.%CF%B8%E0%A6%F4%F8%F8%9E4%F5R%1E%3D%B5%3A%C7%1F%22%10%1F%C1%98CmA%88J%1Fl%FD%F4%03%DB.%3B%10%1C%EE%8F@Kp%EA%FC%DA%CD%0E%04%FB%08%049%0A%C1L%92%06%17oY%99%99D%EBv%CA2*%A8%CB%A6%24Q%08%AE%C8p%B5%0A%20%E4%A8@%BC%DB%EA%7Fe%09%A4T%F0%C4s%9F%D4%D5u%F7%BF%7E%ED%F5%D7%9F%01%11%FC%25%16%88Y%84%20%CA%91%01%E1%8F%D5%9F%00%04%D5%81%3C%BD%E2%98%5B%D2%062S@%E3%F6%E7%10%82%BF%BExr%D7%E9%83iL%09%1D*%C8%91i9l%82%07UF%1Dt%DE%01%92%9E%2CSE%05%C2%BE%22%C2%BF%9A%09%9Df%12%C1%BE%88%D6%19%EF%BD%F5%B6%5B%EFj%AB%EB%C6_I%91%28+c%2C%85%CBh%20%B5%CB%8C%0A%F6%D9%C3%B4%AA%1B%82%C7%C76%5B%FF%8E9FV%24%96%F7RA%E8M%DBOG6%A7/%5E%A4%11dW%A6V%E3%04%12%03%7FM%E5%19%F2P%EB%C8F%90%1A%9C%9E%2C%87Xwr%FC%93%C6%FE%CD/%D1%08%1E%E3%BC%10%3C%86%11h8t%A0L%85%D3%90%AD%B6%A0%0E%FD%16%AA%3Brl%C5%AAU%3FF%F81%CC%8FW%ADZE%21%F8%E9%F6%DD%E3%8D%A3%C3%BB%87%87%CB%CFF%BF%7Cp%E3%23%04%82G.%D62%DD%F7%8E%3F%F3%CC%AB%FF%1D%08%7C%FF%17%B8%AE%A7%DE%AC%D0P%EA%02%1D%A5v9%93Z%5D%C4%AC%1Dp%7F%88%87%27m%C8%AEa%F5%91%08%CE%0E%8F%5B*%B8%E7%D7%27%9F%DAuzo%23%8E%20%AE%82%7D%BC%E3%EC%B5%A6T%AA%B2%22UQ%99J5Q%0DII/S%05UO%AA%86%3F%28%88%82OTi%04%B9%C8%CDu-w%F5%DE%DA%DB%D5%D5QG%AE_%95l%C42Q%D8%0DA%1B@%3F%1C%A1%AB%0A%8E%21x%C6%9C%04%92%08%02%84%0E%04%D9%CD%E94%BAM%D3%17%C7%1DOIX-%C7%20E%20%B3v%FE%B2M%DE%A7wdH%A4%C7%A6%27%87%CB%CF%83%B8%DA%08R%23E%7D%9E%08%F6a%04%3A%11d%92%D0%1AdA%FD%92%3Ah%05%F9%A8%1E%CA%02w/%FE%18%8D%24%FD%F8E%07%82%83%5B%FF%E7%3D%7B%86%D3SCCS%E7%A4%95%07%1F%DF%FB%06%D1%23%DE%B2%BB%B6%AE.%F2%2C2%20%F0%FD%AF%E07U%C5%D5%8A3h%3D%29%BA%EC%DC%8A%D5%D0%2C%E6%F3%C8%20B%90X%C8%8C%10%24%1F%93%F7%8F%EF%81H%7C%F1%E2%C9%DD%17w%ED%FA+%AC%1F%98C0%13%155%D8%271%B8%5E1%00%8D%23h%8C%A2%26R%05%F9xQ%D2M%5E%D4%7DI%5DA%A36%BA%C0%8AT%5B%B0%8F%8Fhu%F1%96%BB%EE%EAji%E9%A8%D3%28%04m%11%3C%B7bp%92wE%D0%96@%94%7D%9EsE%D0%FA%D7I%A0%85%A0%1D%86g%E0%A7%8B%7EF%8C%E0S%D3%E8%82%A7%7F%3D%9Ev%8ES%E9Y%06%9D%EF%8Bh%5Cc%A3%B7%E5g%F6%BF%1FSq%98%9B%2C%DF%7D%DF%9E%93g%C7-%049r%A4h%29*%C8.%E4%CF%26kD%B5%E6%83%C0%06Y%1F%5C0%DD%24%87%A6%8F%ADZ%95%1B%CF%A4%11%14%CA%BF%BC%FD+%27O%EE%DD%F6%CD%BF%F8%B7%07N%3C%BE%F1%085.xd%B8%A6y%FC%93%A9%A1%A9%7E%20%F0%17%1B%F1qA%3F%F4G%10%82%E8%5E%E7*V%A7Xfp%C0%FDY%3DB%10%1FX%97%7D%3Ez%C5%BC%BFq/D%E2%8B%BF%BE%F8g%BB%7F%BDo%18%7B%5C%2C%AAY%04%FB%162%E7%E0%08*%15%A0%80%03%03%83%87%07%07%06@%09+%C8%5Cg%0AZw%9D4%03%AA%A8%EB%10%8AU%9ED%90%E5%F9%A4%D0%DD%1A%8F%B7%00%82%A1n%81%8C%1Fv%A2%ED%C9%A9%C9IY%E3b%D4%3A%F7%0C%82%D0%AC5%18%CE%27%3B%10%1CM%9F%1A%CB7%17+%A7%82%5D3%5D%5D4%82%99K%8C%F6%9D%DE%83%D4%D0Q%9F%82%CD%E0*%2C%B9N%C6%D6%CEo%F9%00%B3%8F%E7%C9%0B%C2i%93%E5%B3%80%E0%F8%C9%C6%89%CDg%26e%7C%B7V%9CQ%B2%04*%0E%049/%04UI%E5y%5D%93u%BF%A0%A8r@%95%93%C4%03%3A%CE%26%D0z%D5%0F0H%E1%5B%FE%C8%89%C7%1F%FF%CAW%F6@3%10%EC%E0%1B%D4%D3%915%7B%7F%F9%FE%FB%CF%1F%7Cx%A2%F1%FD%F7%7F%F98%F9%D8%BB%A9j%3E%85%B8%87%8F%15%F3U%15@%82%F3eV%0B%08%12aXQh%04%D9%D9%E1%8B%10%89%F7%5C%FC%F5%B7%F6%A6g%B1%E6O%0EA+*Z%86%3F%AEk%AA%AC%1A%1C%1C%80F%F9%DA%D4%C0%E0%60U%25%A1%C3%92j%18f%99%F5%07%104MQTh%04%05%F3%E6%3A@%01%20%AC%BB%99%7CC%1E%F2r%9C63%A9*%82%A0%C54%17%04%7D%3E%D9%AF%1A%8C%CF%1F%99%A5%10d%7D%D0%084%F2L%08%CC%21%C8%CE8g%CA%D8%0B%C6%D1%0A%F9%03%E9%E1%F4%01%3A%E1%0CZ%E5%BE%161%08%04%CA%8Ed%20%83%F3kp%02%C9%A8%C4%F2%C2d%F9%F0%F9%F4%C9%F4%B8%85%A0@%26%21%F3%D2%B9ET%B0L%E6y%CD%14e%94%9B%D8%94d@%90%80%FB%D8%E7V%AD%CE%0EhB0%A6%10d%DB%B7%9E%D8%F8%B8m%1B%0F%DE%DEN%9D%EF%DC%96%8D%BF%B8%E5W%BFx%FF%17%F0%DF%E3%B7%CC%91%97%A1%A2%AA%B2%EA%C5%BFg%FF%FE%C7@%60%D1%C0@%85%9Eg%C6%D5%A2I%1B8%5E%5B%DB%B8%7D%7C%FC%B9%8B%BB%C7%F7%A6Ge%3EwI2%CFU%EC%A8%88%9E%FDP%D3%22+*%ABTh%0AB%C5@cP%AD%22%93%E6J%AAh%10%E6%27%11D%7F%C5%82%F1%BA%BA%AE%B6%3A+%917O%7BQr%0D+%DDM%CCG%23%18%CD%FE2%87%08%A4%11d%0F%A0iZ%0B%D2%E1@%D0%EE%0E%BB%22%88%14%10%E5%E9%D1%D3%E3%B3%07%E8%AC2%28%91%82.%1CN%ADZ%95%3A%AC%0A%BA%23%85%090%F8q%A6%1D%F8%F1%FC%20u5%7D%CAd%7Fzx%8F%8D%E0%E6I%22%05%94%89%27%C1%B3u%D8%5C%AA%97%D1%25%9F%26%29R%19%AF%F2RP%90t%89%9C%AC%E5Y%96%E5%D4%F67N%1CD%10n%DC%7B%E2%8Dv%958%23%F0%CE%5D%FE%FCW%7E%F1%CB%F7%7F%F9%AB%AFl%BC%3CGxYN%ABH%5D%ADDS7*R%8F%89%AA%20%E7%CB%AA%B2X%D2%06%8E%97ue6%0D%0C%8E%9F%DE%D8%B8_%D1q%06%17%AE%9A%7D%22%92%AE%AB%12%F6%F2%9FTeUUej%104c%B0%C2%FA%88g%26%92%92V%7F%3A%10P%03%96%96%0AZ%12OIfO%BF%89%98%BD%00%A0%9D%C8%1B%1F%9D%5E%98%9Cs%14%FD%25%93G%04%08%CE%E6%B2%8E%02%81%0E%04YultT%CD%D6%BD%03AO%15%B4%12%DD%E8%E9%F4%EC%26M%E3%DD%10T%0F%A7R%87%83%AA%82%10%246%CD%F1+%1A%E7/%7F%E9%F2%E5%CB%1F%CF7%1E%D6%C8z%04%15%DC%3F%3A%3C%9B%3E%F9%EA%27%8D%13g%F6%9F%D5q%AFsb%29%FE%7E%1Bo/%3A%27I%95u%89%0Fj%92%00%17H%60%97%BCe8%23Ym*%7Fd%EB%91%83%07%1Fy%A3%BCI%95%29%04euv%EE%96%8D%60%5B.%CF%CDR%5Eh%02%AD%AD%B0%E6%0B%DE%9F4%02%8A%DF%91d%87%BC%26%9C%9D%83%C5%25%F5%0B%AAU%C5h%1C%DF%BD%F7+%E9F%09*%D6%99%CC%C7%8E%8A%A8%19%A8%07%F0%E9%F7%EC%60%D5%C0@U%93%D5UiB%1F%07%B1C%E4%92%92%1Fe%CEB%29%C9%AC4%89P9%12%DEi%B6%12%A1%88%91%98%A4jRD%B4%12y%D3%5E%3BM%8A%E6%B8%27%26g%D1L%F3%F6%F9%8C%A5g%1D%08%B2Zr%F4%80_sE0k36%83tw%04%18%D4%04%F5%C0%01%B7%DAB%0C%CA%BA%10%5C%AB%08%7E%5D%A3%08%B4r%85%1D%9EmDS%8C%07%D1%15%21%F9%E54%23z%B6%FF%E9o%7C%E3%A5%CD%9BG%CF%AA%E4%F5rL%AF%D7%97%EEeX%05%A2%B0%9A%D0x9a%5D%3Fn9eyY8%DC%D4%DE%BE%AA%BD%BD%E9%B0%20S%D7%DE%F2ffM%3B%BD%8C%95%D5J%11P%5E.GDX%8E%A1%ED%F8%83%8D%C3%E3%8D%FB%11%CA.j*e%95%5C%224%D0z%AFo%C5a4S%08%1A%9A%83%15%15%83D%3A%7E%CD%91%FF*%21P%C3l%286%18R%24R%16dL%94%C8%DB%E9u%22%84L%9D%A4%8Cz%B3%B0E%98%EF@%F2%80%7D%E0%84%CF%91%87%8A%98%5Ea%C7*%7D%93M%AF%7B%AA%26%DE%CAz%C49%AB%DC%CE3f%8D%22%A0%8At%5C0%9F%90%3Ck%1F%EF%7E%C1%A9%18%28%F9%26%BAg%FD%EE%19%A0%BC%BC%E8%C6%B03%CF%0A%0Ei%5E%AC%2C%96h%CC%99%E9n%11o%B6%B6%99%1B7%D8%91%DA8%BAI%CB%93%D5%09%BB%8F%5C%12%60q%9Ck%C0Cw%10%CA%1D%9B%E9%0A%A1%3F%9A%EB%D63%9AdD%22%8B%BC%CE%96%DA%B8%A0%8A%28%D7Y%10%EE%7B.%DFF%F3%ACO%B2%03%AA%9E%A7R%BD%CB.rX%BEl%DE8w%8CP+S%96%F3%EE8%1F%F7%8Bz%AD%F6%83%CF%F5%C6X%7C%CB%05+X%C1%0AV%B0%82%15%AC%60%05+X%C1%0AV%B0%82%15%AC%60%05+X%C1%0AV%B0%FF%01-%99y%DE%1D%B8%F1MQf%3A%9EQ%E1%E90%7E%7B%5E%CA%02%91p%F8%DC%EC%DC%DC%B9%B9%B9%D9s%E1%F0%B2O%F4Fr+y%97%FD%FD%F4%FE%EE-%92%99O%17%D1o%7C%5B%84%B1%92%F3%3B%EC%D5M.%5E%89%F9L%BC%A4%05%C2%F7vH%D1p%7F%B8%FF%5C%7F8*u%DC%1Bv0x8mY%BFk%F9%FC%B9%95%2C%D3%A4sSS%E7%24%F9%3A%CA%FE%BEyuo1%F2%D4%AA%1B%29%CBDD%D36%3BO%8E%B8%8C%B2j%E6%DD%CC%EE%AFz%02%04%15%FB%E7%0E%85%B5%7FH%043%0F%22w%FE%8D%CB%EA%B4%25zsO3%F3%22%18%B9%B7v%C5%E6%CDG%CF%8D%8E%A7%CF%1D%DD%BCyE%ED%BDTz%BB%03%E9%F4%D3%0F%FF%C5G%0F%3F%9DN%1Fp%94%B6r+%8D%BF%FA%CC%ABign%25dk%A7%26%27%AF%5E%9D%9C%9CrI%BC%E5%95%97%E9%F7%D1%1B%C9dR%8C%B8%BF%CF%8A%D6*%E2%89%EB2%CB%D2%5Ebfe%98%5Bz%D9%98%FDPZ%DF%E0%FA%84%7DA%05%EF%60%D9%BB%BAlP%28%04%ED%18%CA%E5%87l%E7%CE%85_%B9%5E%04%C3%1D+%5Ezi%A2%3F%FC%FA%EB%AF%87%FB%27%5E%3A%B3%A2%83Lo%D7%9F%7E%E9/%BE%F9%E0%BE%7D%FB%1E%FC%EA%D3N%21%9C%BB%FD%F4%E83%E7%A1%EC%F9g%D2tn%25t%20S%93%8D%8D%8DW%01%C2%29%E7%0B%7C%BD%F22%FD%3Ez%23%8A%ADFb%D8%1D%14R%AB%0El%DA%E4%CF%5D%CDH0%23GK*+%D2%DE%10%F5%A2%CE%EA%8E%25%96Md%26%C3%DF%1Dsc%D0RA%C6%9Ez%D1%DA%E6%8A%20%82la%89n%9F%1B%82%F9%BDx%82%04v%A7%07%82%B1%CD@%60%7F%F8%DA%B5k%80%E0%C4K%07b%04%82%07%D2%0F%7F%F5Ad%FBN%9F%7E%E0SZ%07g%B7%AC%7D%E6%99%B3C%E9k%E9%A1%F3%CF%3E%3B%B1%86%8Ak%FC%D4%D5%89%B7%B7%CE%CE%9Ey%7Bbr%8A%CEjc%E5ez%1Dv%7B%ED%F5%D7%1Dy%99%96%E2M%5B%FBM_W%D9%F1%99%F1k%E3S%E3%CB*%1BQ%CC%06%CBD%FBbSUJj%D5%C4%F0%C1%23%C3%B9%14pP%96%00%C5%B3%ACS%E7%AA%BB%E2%A4U/%B1%2C%20%88tP%93%F5%CCK%3Ai%04%B3%04v%D5%B5Z@%B9%AB%60F%07%5D%10%DC%F9%87%5C%F6W%FA%3CTp%A7%CB%12%F3%05%3B%17%0D%F6O%FF%20v%EE%E4%C9%93%E7%8ENO%F7%27%A3%8F%E1%EE%F4K_%DD7q%EE%DC%81m%D1s%FD%DB%B7%5D%21%17.%0As%DB%C6%CF%9F%1D%3A9%94%1E%BA6t%FE%2C%99%5BI%9D%9Azl%B2%FC%ADO%7Fv%E6%CC%CF%3E%7D%AB%7C%F2%B1%A9%29%95%2C%7B%F1%D9W%C7%87%C6O%02%0A%AFRy%99%96%E4%CD%EC%F7%BA%CA%A6gfO%9E%9FI%E7%F1B%5D%B8%94%8D%04%AD%D9%84eh%A1%0BJ%FFD%25C%25%B4%AA%B9%FF%E0%3F%EE%7Cexe%AEl2%16K%E4%CC%AB%AC%8B%CEU%B7%B5%12F%20%286%CDQ%96%5B%97%92%D05%DCd2%16%D9*%88%109%D6V%B7%8ECQ%D5%A1%82%98%CA%F59%21%F3%F6%E2%04z%208%1B%3E%DA%DF%1F%0B%A7w%EDz*%0D%1F%8F%C6%C2%B8%2C%F4%F7%FF%C5%BE%7D%FD%8F%FE%D7M%13%DB%F6%8Dn%DF%BD%AD%9C%08%C5vn%A5k%27%A1%EC%C9k%AF%92%B9%95Th%05%CELN%BC%FD%E9%A7o%BD%F5%E9%A7%3F%9B%80%FF%99%C4%19%B4%CB6%CE%9C%DFu%7Ef%8C*%BB4%AF%FB%7E%97V6%3D%93%DEe%21%E8%E6%1D%87r.e%A5dF%05%1B%1A%2CA%A11%C2%B5j%F3%C1m%3B%D9%DF%0C%EB%B9%B2%D9p%99%D1B%8F%B2%0E%9D%0B%EB4%82m%84%0A%CEm%A1%2C%D7%7C0%15%02A%CD%81%60f%02d%5Bo%EB%BA%5E%20%90%A3%10%CC5%F4%10c4d%3BA%03%B3k%86%DC%10%CC%C5aO%04%E7%26%F6%87%C3%FB%D3p9w%9DL%C3%C7s%E5x%FB%27%7Df%DF%E9%7D%7F%FE%CD%FF%FA%D5%07%20%12o%DF%FD%F9%09B%06%ED%DCJ%D7%EE%FB%D6%B7%F6%9E%BEF%E5Vzlr%F6%CA%D4%24%10h%D9%DBoON%5D%99%9D%7C%8C.%FB%F4%FE%A7w%1D9%7B%90%CE%CB%94%F1%E60r%F3%5E%B3%8E%F9%DAu%95%7De%FF7w%3Dxv_%1E/%88%A0K%D9dB%2C+C%22h%9B%E1P2%0C%94%9E%89%83%AF%7C%B4%AF%7F%25V6%D6PC%09%5D%A9%7BY%87%CE%85EO%04%C5%B9-_%02%FB%10%E4%EFC%F4%E1K%18%82%A2J.MS%00%87%B2%85%99%E0H%05-%02%7B%DB%D8%B6%CE%AE%9D%28%5C%12%08%EE%B4z%BB%BAM%E0%D7%E9%8C%5E%3B3%7Da%DD%D5K%B5%14%BD%10%B4%F2%23%5D%3By%F2%A9%93%27_%7F%95%CA%8F%C4%A4%DF%9A%25%EC%9E%B4%B3%EC%EE%F9Tyj%7E7Uvj%F2%CA%95I@pj%EAS%F8%F3%DA%DB%93%93W%A0A%E8%D8%EF%03%B0%DF%23%CE%FD%DA%5E%00%F4%29%1BP%87%F7j%EA%EA%C7%EF%FE%EA%A9%93%BF%FA%E0%CDT%D52%CB.%BA_@%D0%A5l%20%EA%A9%82%A4V%F5%BC0%7Cp%A2%94%CB%95U%CD%E6.%CA%9A%F3%94%A5%21s%F1%D6%E0%5E%0B%C1%0F%E7A%E4%E6%3F%A6%10%0C%A8%BC%D3%16%1A%84l6%5C%B6%1Dc%BB%3A%DB%ACpI%22h%11%16%B6%25%D0%91%D70%CBWXw%F3b1xQ%04Q%7E%24%C4%E0%C9k%AF%3FC%E5Gb%D2%FF6+kA%A8Ih3%E8%3E%BE%89F%10%CA%EEiOUUU%5D-%DFM%96%7Dl%F2%CC%D6%C9%C93o%03%7E%AF%C1%9F3%93%93o%94%13*%E8%B9%DF%8C%F7%01%F0%1Eq%F5V%89%CA%FF%B6%AA%29%95Z%B5B%29K9%CB%9E%5D%28%7Bv%BB%EB%96%3D%F6%7B%F6%19%D4%16t%96U%22b%99%94_%05%29%AD%EA%09%06%0FU%E3ec%CD%5D%94%D05%E7+K%23%E8%F0v7%E3%DE%B95%97/_%9E%D3%CA%E7%CA%B5%B9%CB%97%DF%7D%17%CB%ED%A2%04%9D%04%FA%16%D2VeU%B0%25%CE%B2w%ED%E8%E6%1C%08Z%1A%18%B6%10t%E4o%B5%BD%16%82a%DD%C5%CBb%91%D8J%F4%98%1F%C1%13%CF%7D%F2%EAxc%E3%F8x%FAlt%DBv%22%3F%12%20%F86%20%88%5E%A0%01%ADr%99w%20%88%CA%CE%03%81%A9%AA%D4%D5v2%B7%92%3A5%3B85%F9%DA%CF%5E%7B%ED%9E+W%AE%00%82SM%B3x%5B%10%95%7D%16Xp%CB%CBd%7B%FF%D6%CB%FBA%95%A1%A0%7C%FB%B2%A2%A4%3Et%96M%8Fg%CA%8E%A7%B79%BD%DE%FB%CDW%96%0F%9B%0D%D1%A5%AA%20%A5U%7C%18%A9%20%E9%EEi%5EZY%E4%8DS%08%96%E2%DE%B95%EF%BE%FB%EE%9C6g%FDy%97@%90%17y%1E%E8%F9/%88%90c%7D_%3F%86%FE%91%17%C6m2*%F8r%1D%B4%3D%0F%ED%E8t%22h%13%18%0EGb%A6%88%12%ADSY%1C4%C50cVby%BF+%82%B90%EC%CC%FD%8AW%FA%91%8B%9F%00%83%28w%8D%F4%8D%ED%CF%91%F9%91%98%F4D%B9%A0%29%80%204%AC5%9E/w%B4%05%3Fy%FF*J%60%B8zu%EA%EA%BF%92e%A1G%3C91u%AE%7F%E2%CA%D6%89%C1sS+%26%C9%1E%B1%B5%DF%B3C%B6MRy%99%2C%EF%B3%D6Q%85%A5g%9Fs%F3%7EPUd%5B%D5%87N%EFs%D6%3D%05v6zb%A3%B3%EC%27%CF6N%CD%CC%CCL5%7E%F2%FCr%CA%86E%E9%DER%AAA%17%C4%BC%DEZ%15%E9%26%11l%EBV%96%5C%96F0%B4%01%F7f%10%B4T%F0%83w%3F%FE%F8%16%EC%98%CBx%8E2%DEX%D8oF%05%E3%F1%96%96%96%AE%1D5%C7%5C%10%D4%C3KB%D0b%90F%10%07%D0%0B%C1%D9-%BB%9F%FF%E4%93%BF%FD%F5%F6G%FFz%DB%5E%3A%3F%12%F4%88%EF%19%D4%04@P%86%3B%8A%9F%B8%9D%EA%11%5Be%AF%A6*W%AD%AEH%A5%E7%A8%B2%D0%1C%7C%ED%CA%E0%D9%29%B0%C9%C17%5E%9B%9Cr%D9%EF%EE%83%C8%F68%F6ky%C7%1B%FB%DFz%F8%E9%B3Qw%EF%BFVY%F9%3B%81@%17%AF%7DOM%9D%93%BE%B1%F1%97.e/%01%80CC33%A3%8F/%A7%2C%A8%60s%9Cj%CF%85r%5Eo%AD%12%9BGH%CA%DAB%FCR%CB%EA%A5%96w%21%23r%5Bs%90%C5%BCs%B7%7C%F8%E1%87s%F3V%97w%EE%83%0F/%E3%99%0BM%27%82%C9%85%FD%DA*x%AC%9BC%1D%91%CE%9A%9B%9D%08%EAa/%04W%E6%10D%0C%D2-E%7Ch%DA%0BAa%EE%C8/%80%05d%CF%3B%F2%23%A1.q%F9J%DDF%90%E3%28%11%CC%94%BD%9A%1A%A8%1C%18H%A5%1De%A1C%7C%E5%0Azg%05z%EB%15%8D%A0%F7%7E-ozx%EF%AB%9F%BC%BA%7D%3C%7D%D0%CD%EB%5D%F6%93%8B%DB%F7m%FB%DA%A3%27%B6%BBz-%02%C1%C2%8F/%A7lD%EC%A8%E9%228j%8B%E7%E2%A5%B7V%C5%A4%0E%AAh%DBRu%8E%0D%1B%21%225wkk%B3%C1ae%E7%BE%F4%21b%10%99vf%EE%C3%CB%1F%CF%E7%B6%5C%26%D3%0F%CA%B0w%10%D9*%D8ju%85%B9%EE%EAC%CBU%C1%95%5E*h%27%FC%CB%BE%D6%C9%0BA%3B%3F%92%95%C2%FB%7Dg%7E%24%E6@%FAJ%B9%1F%CEWc%98%BE%F2%DB%1DOGP%D9%7FM%A1d%C0%8D%8D%8E%B2%E7%26%CF%5C%B1%F9%7B%E3%8D%ADg%26%CF-g%BF%C8%FB%AB%AF%20%EF/%BErp%AF%9B%D7%BB%AC%B7wrf%08%25v%1E%3A%B0%AC%B2%11%B3%B6%BA%AB%95%80%A1%3B%D7k%F0%D6%AAd%AC%87%CC%F0%DE%16%C2%A6%0Ex%96%E5%C2%87%A0%E8%BA%9C%D5%85%D6%07q%05%9D%BB%FC%A1m%1F%20%0A%3F%BC%3C%D7%84%A5%3F%A3%11%E4%CC%DC%7E-%15%ECZ%C7Ym%B6%9E%EA%EE%C5%DA%82%BE%BC%08%FA%9D%EF%84%C2b%B0%F3%9DP%A4Y%F9%91%DE%B7%CD%99%1F%89%EB%87%E6%E0K%E8%A5%11o%97%0F%A7%FB%A9%B5%D0v%D99%88%87W%E7%DA%E9%B2%8C0%D5%F4Z%86%C07%5Ek%9AR%97%B1_%D2%BB%D1%D3%FB%F8ux%1B%27%A7%C2%E7%06%276.%A7%2C%1B%15%3B%5BI9%C2%3A%0D%DEZ%C5%AA%91x+%E5e%96ZV%A4%9F%9CT%9B%B9%CC%13P6%9B%3Ds%21%83%F0%20%B6_%95B%90%C7%DE%E9%81T%B0%AB%B3%13%A5%93%3D6%B2%A3%BA%A6%C7%89%60_%1F0%B8%F0%86%3A%1A2%7B0%C6%22%D0%C5%CB-%84b%DAK%91%90%CB%8F%F4%0BG%7E%24%94%BAFJ%A7%CB%91%A5%D3%E9%A32%9D%C8%27%5B%F6%FDvgY%D4%23%19%7C%ED%8A%A5%81%AF%0DN%19%CB%DB%EFo%D7%FB%F8%C1%9F%FF%FC%E7_%DB%BB%AC%B2%3E%A3zG%DB%3A%5C%8D%EAj%EE%5E%F0zk%95%CF%08%AF%23%AC%B5%1B%8B%D2%8B%E8%9C%95%0AD%D7%C5DX%0C%27D%DDJ%B8%80idS%FB%BB%A4%CD%DD%84%ED7I%25W%CA%0DLgTp%E7N%BC%A5H%21%88%18%F3%873%04%3EDCf%13%C8%DB%04%3E%E4%04%14%7B%BD%A27%82%5E%F9%91XN%D3%95%98%3D_p%14%25%C7%60%97%5E%16L%9E%9A%5Cq%E6%B5%D7%CE@wX_%E6%7E%7F%FF%BC%9C.QbT%7Co%EE%FAzk%95%B3%ECza%A9e5U%94%1A%ACp%E7%F3%09v%D3%CCT%17%AA%93%D5%E8%27%C4s%83%F8%7E%93%26i%09%7C%BF%A0%82%24%81%14%826c%D6%88%CBC%3E%87%CEe%08D%7E%17/G%00%E8%F3%0C%C4%9E%F9%91%AC%84P%7E%94%1D%CB%EF%92%1Fi%B1%DCJ%9CO%8C%9D%83%1EqL%D4%B8%E5%95%FD%7D%F4%A2%BA%20L%C3%D3%0Fe%B4%0A%A5%C9%F1%FB%FD%94V%DDHYNS%02%C1%A0*%88%09%9F%20F%12%A2%A2%1A%AA%CC%7F%16%FB%FD%3D%995%BDh%F6%24%8F%D4o%8B%94%B5%12%F5%F8%055%A0%0A%7E%D9%DD%7D%BDY%9B%FE%DF%F2z%985%0E%86%BD%E3%7B9y%82%3C%CB%820%27%13%96%0A*%A2d%89%60%83%28h%9F%C9%7E%17Y%FF%E1b%9F%8Dw%B9v%DD%99%A4l%7EQ%5E%5CW%7C%0B%B6L%13%9D%B3Yo%D4%AC%A4%9C%01%C3%CABf%80%D2%12y%C8%AC%08%28%A8%86h%A2%1Ce%28q%2C%1E%14n%C4%5B%B0%82%15%AC%60%05+X%C1%0AV%B0%82%15%AC%60%05+X%C1%0AV%B0%82%15%EC%7Fp%FB%9DeO%FA%FF%91%F1%A9%D4g%9D%60%E8F%EC%92%C3%96%BD%09V%D7%96%5D%E63%B3%DF%DDs%C0%CF%DC%3A%C8%05%DF%BF3%D3S%DF%FDn%8A%BF%F1%ED%7CVv%E98e%CBDP%28-.%01%5B%BF%C1qN%8E%17%01%E1%CFfT%A7%00%5D%DF%A3%1B%EF%F437%E2%FDm%1B%5B%D3%D2%DBU%9D%E7%A9%8F%B7%22%E4%F3%A2o%C62%9F7%ED%CF%B3_D%60%1E%06%B9%E4y%D8%C0%F9%A3y%5E%F8%07%B6%7FS%E6%C3%98%EB%B1%ED_%E4%FD%3E%EAy%97%F3%BDt%7C%8C%B0%E5%21%C8n%28%A9%E9%8E%F7%F6%C6%BB%9BK%82%94%CFq5%897U%3Aj%80%CB%B9EWs%3F%82%DF2%82%C7%EC%D5%3E5%C7%3C%ABA%80%DF%AAq%F9%DES%E7j%3B%5B%D6uu%D6%BA%3B/%BD%60%D9%84%FB%E5%C8%E7%BD%94%93%90%A3cy.%24%10X%F9%5D%F8%CF%85A%E3%D2%D8%29%A4B%A3%97%0E0y%EC%D2%D8Q%C7%AE%08w%DE%92%C8%0E%8C%5DrEp%29*x%D4%FD%96%92%8B%8B%7B%8E%8Dt%D4%D4t%1F%3A%D6S%DCL%BD%91%8F1%0CC%D3%D0%24%95%600H%CD%BD%97%98%DC+%84%EDY%1D%98%5Bt.%1D%F5%F1%F9%11L%12%B6%93%82%8CH%8F%C4%EEd%F2%27m%A0%CAZ%D6Qw+%B2n%CF%80%A9%D5%C4%7B%5B%AA%9D%DF%7B%EA%1CS%D3%D5Z%DD%DDV%E3%EE%BC%F4%C2%E8%E8%7B%60%F9%10t%F7%82%9E%D8_q%FBG_pG%10%08%5C%BD%1A%10ta0i%01x%0A6%7B%7Cl4%CFA%C3%9E%F7s%C4%AEh%F7%D1%FC%F5t%14%0E%CA%A56%C62%8A%3E%3A%9A%F90%E6R696%E6%B6e%B6%B8%A6w%A4%D86%B8%0A5%A5%84%D7F%D0g%23%18p%20%E80%1C%C1%0D%EB%17l%DA%B2Rmi*%B8%D33K%DC%B2%BC%D6%19%D6t%C5%5B%D7%A1U%3E%1E%D3%24%F8%9A%B6%AE%BA.%17%04%3DuN%D9%D1R%13%AAi%ABQ%5C%BD7%86%A066%FA%82%3B%82%88%C0%EF%7D7%95ra%D0%3F%F6%1E%C00%8A%18%7C%EF%F8X%9E%88%0A%FA%3B%3A%A61%F9%11%CC%22%EA4v%3F%3A%A8%BC%F5%B8%7F%14%E47_%E3%81%D94%06%F7%85%8B%C2n%28%E9%0D%95%84%E2%B7%B1%7F%D2%DB%03Jx%5B1Q%9D%08%C1%A0E%20%1F%0C%04%DCT%10tp%E5JM%D3%BE%FD%10%B9%7CI%E4%A7C%96%D5%D8%06%84O/%0DA%3A%03%D2%8Dx%91%05%3A%7B%5B%D7U%03%83%3B%F2%A7%0F%E6jZ%5B%3A%AB%DD%10%F4%D4%B9P%5B%5B%0D%5BS%D7Y%EA%EA%CD@6%E6%89%E0X%1E%04%D5K%FD%C7Ap%5CZ%8B@%E0%F7%80%C0%C7%1Esa%10Za*%E7%3B%3Av%1C6%FC%5E%FF%25%1F%BDO%DB%5E%18%9D%E8%BF%A4%3A%11%CC%BA%FB_%18%3DO%17%B5%8C%3F%3F%FA%C2%C2A9AL%C2%7E%C7%8E%8F%25%19W%F3_%3A%3E%0A7%DCf%FA%7B%A1dd%A4%A4%F8%90%FD%3F%87jj%E2%23%C5%F8%0D%60%21%18%94%22%E1pDt%2C%05%06%04%C9%D5%A1%7D%04%82%EB%3Bk%BF%F1%B0%11%CA%81%E8%85%20%19%88%B9%FC%81%D8%3D%CFf%BE%B2%C8%9A%E3%5D%AD%A0Uu%AD%EB%9A%99%3C%C6%86%EAZ%3Ak%AA%DB%9C%08z%EA%9CV%D3%B5%A3%83%E9%A8YW%ED%3A%9Cp%23%08%C2%D5%3C%3E%3Aj7%17I-%BC%C9%26%B0IJ%AEp0%A8_%B2%BB%0A%C6%18%EA%1D%8C%3A%24%27%D3%FE%84%3D%1FG%A48%10%CC%B9%FB/%B9%0C%F9%E8%E8%B6X8%28%07%82*%22p%2C%9F%FA%FA.%F5%8F%C21%F5_ZI9Jk%8E%81%3E%1D%8Aww%02%86/%EF%A8%09%BD%DC%8CwI%24%06%9A%80Q%F4JfNJ%2C%8A%20G%22%D8%BD%F9%0E%E6%1B%1D%1D%9D%60%16%86%EB%17Q%C1%85%05%04%7D%AE%C9%5C%DD%F3%23y%97%05%93kz%5B%EB@%AB%D6%D5%AD%ABv%CF6%0Dj%06%D1%B6%86%ABnu%22%E8%A9s%1B%BA%DB%8Ay%86/%5EW%B3%81v%ED%B7%D5%26%03%19%D8%FE%A5z-.%F6%8F%81b%BC7j%1B%89%20%F4%85%81%C0ASU%D4%B56%839%DF%81K%19%1E%F7%8F%22%18F%E9%D6%20%DA%A7e%EF%BDw%EA%F8%D8%7E%27%82%A3Y%FFqK%26%29%C4%80@%EC%A0h%045%A4rH%7D%8F_r%B9%23%B91T%18%18%1C%BDD%05%F9%E2%EE%1Eh%03%D6%D4%EC%08u%02%82%9D%A1%1D%F1%1E%BC%3A%11%82%92%C4pat%DD%C5c%9C%1B%82%03%03g%B8%CA%01W%04%3F%FA%D3%97%BA-%EB%28%8D%2C%8E%60%9E%0CH7%E2E%10%B5%82%08ZZ%B5n%87%7B%C0d%3A%3A%5B%EAj%7CLu%1D%89%601mT1%B6%B8%CDB%B3tGg1%7D%3D6%8D%D9r%82j%1Du%10%C76%E1%5E%95%F6b%97%DB%E2b%F4%F8%7B%969%11dS%88%C0%C3%22%CA%D9%27%ABE%DF%25%10Lfe/%08%1286%D6O%F7%092%8CY%9B%3E%05%9D%E6%3C%08%22w%BF%23%A0%1E%18%3B%DE%8F%1F%14u%CA6c%D6%96%FB%C7%9CM%C9%F3%F6%29%C1A%8D%92%83%3AlI%BC%D3%22%B0%B3%B3%85aZ@%AFz%BA%D6S%08F%E0%B2%C2%1F%16%FD%E5%86%E0%8B%03%03/%0E%ACF%9F%FE%90D%B0%A7G%DD%DCc%DB%BD%0D%E1%D2%EE%F5K%0E%C4%7C%DE@%CC%D1%F9%912%08%E6%29%0B%CD%97%E2%96%B6%3A%5B%AB%D6u%16%BB%0E%E6%D6%EE%E8%EA%AE%01%81%AC%5E%F7%7D%12%C1%B66%3B%F7%00ZU%BBn%DD%3A%1A%C1%DAP%5B1%D2U%B9%B8%B3%C6%D1_%81%9Ec%F6b%A3%9E%01%D5%13L%8E%BE%D0%8Fy%F1%8B%8D%B8%80%CB5%F6%DE%82%11%08rhLZ%145%7B%94%E1%89%A2%EF%E2%08nF%97%DE%92%D8K%FD%08%C1K%B4%C2%BE%00%DF.l%F7%F8%29%07%82%B9%9D%BE7J%1D440G%89%83%A2%10%84%AE%C8%C2%099%BB%24%A8%3Al%04a3%848%0B%25%C7j%8A%B3%04%DEVW%D7%D9Yw%AC%84B0%CC%00%82h%BC-%EC@p%A5%25%3Bg%06%06%06V%A1%0F%2C%89%E0H%D6z%3A%C2%F7%86K%7B%BCU0o%06%A4%1B%F1%22%85%EB%AA%DB%91%D1%AA%CE%1A%B7q%99%00%EAq%A0%94%19%DF%EF%24%21%5BL%05%9B%BB%EB2%8E%1D%3B%8A%1D%9B%DD%3F%3A%E1%7DIN-x%89Kb%238%86%B1r%9CB%F0%7EQ%94%B3+%13%B5%22%1CA%CB%D8%F3%A0G%C7%D1%16%2C%81%C55%C7%1A@%F6@%D0%1E_%5E%B83%C8%C6%83%D5%D2%C3%0E%8ADp%D1%5Bn4%CF-%27%03%82%60%A1%CE8%C3%DC%DA%DD%0D%0Cv%BB%22%D8%E0%85%E0%8B%95%A0%83.%08f%14%B0%B6%B6%23%12n%08wt%AC%F7y%20%98%3F%03%D2%8Dx%21X%C6%A1%25h%0F9u%02%29%CEn%9CP%13o%AB%B6z%1B%C5%0E%90%3A%A6%9B%7F%80%5C%CA%0F%8A%A7%1D%F4*5%ADu%28-K%5B%5B%7C%DD%8EbG%7Fei%81%09%11H%06%A6l%20%CE%83%20%B36%A5Y%EBxy%DEJ%F1%C6%A5n%A27%0D%0Cf%9FU%90%04%92%08%BA%04b%8C%FCS%0E%8C%2C%19%CC%87%60%A6%E1a%BB%F24%3CN%E5%BCx%3B%B3%04b%10%12%C1%DB%18%06%82%0E0%D8v%01%EF7J%0C%CFG8%E0%0F%C9%A03%10k%E8%BA%AF%1A%A8%FC%89%CD%20%89%A0%3D%188%FD%03%B0z%B0%1F%FC%60%3A%3F%82%3B%3D%031%E1u%CB%B3%99%B7%2CH%5C%1C%CE%A9-%DE%D5%D5%F2%C5%3A@%D01.%23%D7%B4%B5%ED%A8%FE%FE%F7%BF%8F%C4%AC%84rz%EA%5Csg%A7%F5%5C%13%ACxG%8D%B3%B7%0D%CD%F3S%F6%C5%CC%D3%3C%3Fe_%11%AAy%9E%ED%8E%BC%97%07AP%820J-%08%08%D6%C1%0F%17v%DB%F6X%BFE%20%FC%CDQ%DB%C6%10%DB%EF%8A%60f%9F%C7/%1D%A0o%D7%03%97%FA%B3%CA%ED@0%D3%C1%CA%B8%DC%BB_%A7%DC%BD%EB%7BF%2C%04%EB%BA%E1%86F%0C%B6%F4%E0%B7%3BB0i7%B7%18I%EAsA%F0%27%AB*W%7D.%DB%11%20%11%CC%0EI%83%01%81%F5%D3%1E%08%92%19%90%7C%5E%F9%91%16%C9%9ED%96%05%88Z%AD%EC%14%9D%20%81%60%0ER%7C5%ADm%19-%EB%8A%D3%08z%EA%9C%06%1D%E1lUuT%EF%28vb%86%06%29%F2%0F%11%FB%2CB%81Oj%90%C2%E2%C2w%D4%22%14%04%E3%05%F4%B0%8B%B8%DA%80%20z%1C%EF%F35t6%D4%B8%22%C8%F0%97F%AD%B6%E0%E8%25%9E%DA%B6m%D6%C3%93%A3%1A%85%20%8B%BB/%19%3CG%E5x%E0T%AB%9B%04%C7d%1D%94cAd%0EA%B7%F3%CD%21H96%84%8E%A18%DCy%D7%9Dw%A2%C6ww%F7%CB%CDxU%03%82%9A%10A%832%AC%14%11yW%15%C4%06Kp%04%7D%D3%EBq%9Bn%28%DD%E0%F7D0_%06%A4%1B%F1%CA%C5%AD%EB0%AD%02-%23%C7e%B8%9A%EE%B6%9A%EF%5B%3A%87%FC%14%82%9E%3A%A7%C0%97%D9%0B%CC%C3%AF%B9%8C%1C%A2%A1%DA%F7%BC%86jO%9Dr%0E%D5Z%5Chr%12%5D%EE%E3%99Q%60%9C%05%D6B0%16%F3i%0D%D5%F0%C3%85%9DK%FBYVC%0C%02%81%E4%7D%C1f6w%1C%21%96%D4e%12A%16s%8F%5ER%A9u%FFh5%B6%A6%E2%07%E5%82%E0%7B%D7%83%20_%12%8F%23%15%BC%13%021R%C1%968%DE%21%86%AB%19%08%8A%06ZF%1FI%A8%B2%23%10%A3G%22_%EF%83%AEpv%8D9%81%603f%EB%D7%07%04A%0C2%AE%06%18%91%81%98%86%2C%17%83%F3dOB%0E%B7%B2%A5%3B%EA%08%AD%DAQM%8C%CB%B0%CD%3B%DArcz%25%14%82%8B%EB%DCbv%3D%0F%ACl%04u%BF%01%97%FB%F8%25%C1%AFk%3E%82%05.%CC%23%15%8Cil%E0nU%11%B9%B0%8F%1C%E3%B0%9B%C6%7E%28%3DzI%27%97%FE%5Bk%FAu@%1F%5C%22l%99B%10ye%BF%80%DC%E7U%BFL%EE%D7%F6*%E7%D1AY%89%29%7C%8E4%0F%8B%20%F8t%3E%AFQ%7C%AC%3B%83%20%10%18%BF%93%ACh%89Q%D5%A0%AE%1A%0D%0D%89%80L6%F6%16%7BF%EC+%9D%26%0C%AA+%DFT.%EF%0CH7%E2mF%02%A6e%92%81%C8H%D2%08-%D3%A1%09X%CAgs%85%14%97%90%BD%95%00Uv9%AF%07%CD%3E%B6%3F%9E%D5%0Cj%88%CE%22E%1A%1B%1B%B5r_P%5D%06%A4%82%BA%A0B%AF%E2%92%A0%5B%17%1B+%C8%DB%08J%1Ak%CD%3D%D2H%04%AD%ED%A2%AA%00%C1%BA%A4P%DB%B61B%E4%9D%0F%28%82C%05%17%10%1C%3B%EF%84%2C%E3U%A0%95%EA%8E%20%88%E8%02%82%CE%A4%17%E0%7D%3A%AF%B79t%2C%DE%09%8D%C1%BA%EE%EE%EE%96%3B%A9%01.%84%A0%CAkz2%29%FA%90%D6Q%08%DA%E9%8Ar%93ap%04%B1%14%23%96%05E3%DF%146%EF%0CH7%E2%5D%B8.%5E%B9@%3C%BDy%9D%DEC6%DE%93%97lN%7C%DA%FE%FD%9A%86%F2%E4%E0%99F%AC%19T%88%14%BF%B0%7F%EC%92N_k%D6%17%E6%22%D9%27%95%88_%F2e%5B%99%E4.%3EM%0B%5ER%A9-g9%BA4%B6%DFJ%10%A5%91%93%B5%ACP+%EB%FA%A5%B1%A3%82%A0%EBn%81%18%BC%C2%D1%B1K%8A%C3k%ED%F7%D2%0B%D9su%EC%D7%D3%CBp%A5%C5%F1%3B%E3m%DD%DDm-/%C7%8B%A9%21V%89%E9C%E7y%CC%1Ax%A6%E7%E29%E7%0B%F2%5E%F3%05M9%3F%82%5E_%FD%9E%CE%9A.n%CB%E6%CC%B5%12B%92%03%D7%2C9%02Gu%20-%02%81%05%C0%0C%E5%A0%F2%11%0D%7F%BB%9De%A75J%9E%B7S%E1%E0%5B%E6%C2%949%E3%B0%DDm%D3l%02%A9Lx%B0%E5%F3%C9L%C2%24j%0AD%E6%A8%CE%27uYvQ%B9%CC1%CB%07%CE%3B%EE%0B%FB%8Cr%93p%C93Z%C4%0B%C6%05KB%3D-/%BF%DC%D2%13Z%AF%F8HL%3C%A7%AC%8A%9E%B3%A6%D9L-%EBv%AE/%9FGN%A1%DF%9F%ECI%CB%B0%E5%AA%A0%E3%8AY%B9%96%B2*Hn%DB%CA%24%C6%DB%01%C6-%17%98%A0%A2%97%0D%AA%AABd%00%CA%14e%B1%0E%A2K@%CC%BFe%1B3%DE%97/%85%99%97%D7%FB%8C%169_N%13%82%0D%EB%A1%A93%BD%BEAT%05%8D%F0%3A%28%23%D2%94%B2%A8q%A0%06%82%A8%3E%82%AA%E2%97%A9%98%C0Zw%E1%E2%29%B1%FE%BF%99%3D%C9%16%1C%FB%828%AE%D7%A2%CByr%A8%B8%D7L%B6%F6%F2%F8%B2U%9B%EF%C8XOo%DE-g%8F%CB%B3l%1E%AF%F7%19yz%D1m%A5+%81%80*%10y%09%17%BC%3EYP%A0E%A8%082%7D%DFX%ED%0E_%06n%1F%BF%ACLx%05+X%C1%0AV%B0%82%15%AC%60%05+X%C1%0AV%B0%82%15%AC%60%05+X%C1%0A%06%A67%CD%CD%CD%FAox3%CB%CDk%A3Xc%EE%CA2K%FDnL%96b1%F3w%9A%BD%28om8%9F%85%25%97%BF%F5%CF%D8%3A%AA%5D%CC%B8%FE%ED%E9ss%E8m%A2%D7%C9%A0O%B4%D6%10%5B%95%B5%AC%11r%D5%3EdG%ADKnFmY%AB-%0D%85B%A5%B5%BF%95%ACdZ%80%15bAAH%C6%AE%8BA%0De%9D%12%96%EB%CD%D5%06%5D%89%CE%27%EF%BF%CB%0Cf%EEV%DD%D5%B5%8A%B26%C7b%F0%60%22%1A%89D%13%C1%25l%AFi%AE%BD%1D%BD%F2%EA%FA%8EF%81%7B%12%AD%A3%83%9A%93i%1D%14%3Dn%5D%15%CD%F6%B2%26%F9P%0C%BAU%B0DNx%ED%08%F5%C4%5B%EEj%89w%87%7E%0B%F9%DD%B8X2%16S%84%A4%E8O%12%0F%BD%97%A4FeR%99%24%AD%E4xwJ%F2%7Bs%B5%B1i%D3%26%3FA%E1Q%9D%25%12Wp%3A%BE%B6d%D1%A3RJJ%EE-C%F3%1C%C1%A2%D1h%C4%E5%A0%97%FFb%A2%EA%AE%BA%01%CA%D6Q%08%06%A3%22%CF%FE%D1%1F%B1%3E%23%BA8%84s%1F%02%82W%F1%B7y%13WD%F7s%5E%A5%93%89%04%BAR%BEd%D2%AF%D2%08zLrT%ED%C9%5E%D6%1C/%85*%C4%D1%C6H%B8D%F3%A5%DDw%B5%C8L%A0+%1E%EF%EA.%FD%CC%A3%A5i%E81A%89%09%C1%A4%10%CBw2%AE%DFH%26S%C6i%12%87%9E%5C%BB%FC%B2%97w%A16%B4%AF%0E%1F%3C2%BC%19g%22%C6%E88%82w%18L%2C%FF18%BEQJz%E2%25we%FE%07Xc%9DsI8Ep%B9%C2SN%CB9%01%C1%17%29%A3%104E%8E%D3tk%E6%00%97%5Ct%FE%8A%F5%DA%BF%AB%1F%BB%22%C8%1A%AA%C6%E9%02Jt%E2r%AB%E8pw%25T1a%25X%E5%9C3c%A5%CC%7D%EB%5C%E7%AE%A0%C51%99%C7%DC%14%83p5%1AT%B4FMlh%E08%D5%AF%CA%14%82%A5%3Dw%5D%88%0Bw%B7tuirK%B7%FB*%F9%E6%EA%EA%BCYD%BCMNh%C9%A4%1C%F312%85%A0%B7%1A%A1%03%EF3%D5%A4%84%DEb%C7%96%B9%90%E1%E1%CD%D5%C67N%EF%FBh%E7+%13%F8%B2%96%18%C3%0AL%B6%1E%D9%3B%82%1C%81%E0%22G%05%04%EE%E8%8C%97%DC%BC0%7F%22%E0%C0%81%15%C2%11%D9yq%A7N%D1%86%23%18%AF%A3%7F%9FD%D0%0C%FE%E8fEQ%044U%AB%EF%8F%C5%C5%9A%0E%D0%19%B9zu%EE%C3%267_PCw%0E%AB%A1I_%0E%A7%91%D486%A1%04%25%96SM%B7%A5B%D2%02%81%14%82V%9D%5B%00%A2%19t%24%83%08A%0B@%B1%C1%04%04%C1H%04%3B%BA%7Bof.%FC%F0%D6/%B4%C0%DD%CB%B5%B8%A7%8E%83%C6J5s%5D%26%A9%FE%98%8C%EE%A6%A0%A1%12%C7%EC%ADF%8CV%06%7C%89%BA%A6%AD%5C%A9%B3%92%C4%93%D5%E5%E9%CD%D5%C6%CA%97%F6nc%D9W%86%F1%BC2%0D%CC%CB%AC%9E%99%ED%C222%14mX%F2Q%29%25%DDh%F1%D8%02%83%8C%EA+%A3OX%8B%F8%83%0D%CEdJS%A7%C6%C6%CE%9FK%C3%CF%F9t%DAZcJ%23%C8j%0B%C6P%08%06%CB%FEH%0F%04%EEV.%C8%FA%CA%95%0F%F5%B1%E6%22%B1X%9F%FB%F0%E3%CB%1F%CE%B9ePf%3D%7B%18%26%9A%C1%9BP%8D%04%A7%C9%02uw%E1m%14%3F%8D%20A%60P%D4%09%063%08ZJh%21%A8%10%08j%A1%16%86%B9%BB%F7%87%3F%FC%C2%17y%B8%3B%F8x%C8%ADOr%DD%08*e%7CYP%8E%C1i%C7%04%83%90uo5%E2%25%5D%82%D0%C3%21%C64%ADO%92%88%5B%D2%D3%8B%D7%C6K%C3%7B_%F9%E8%20%A1%82%0DWf%AFN%CE%BE%81v%CA1%0A%AA%CA%86%A5%1EU-h%60%0Db%B0%A5%E4f%8E%FB%1B%8E%09h%8EF%00_%06W.%26%3A%DA3%08%C1s%1A%FC%E8%E7%C6%C6%D24%82m%EB%E8%DF_%87%27%06%8Ar%3EAP%EE%BE%5B%B8%A0%EB+%FF%E5%1F9_t%91j%D7css%83%AE%1Db%DDk%C8%24%60%A0ib%09QD%08%CAI%12a%A2%19%C89%D6_%09v%95%ABAEg%D9%A0%E1%B64/O%5BpC%CF%CBL%E0%87%60%B7%5E%08%1C%0A%1C%E3o%EEv%CBAx%BD%08%B21%5D9%CA%27%03%86%12%14Q4%C6%CC%5B%8DDi%E5a%8E%F7%AF%5C%B9%D2%AF%AA%07%CADRA%3D%BDDm%7C%F4%E0%DE%83%0F%F6%E3%22%D9%D4%5E%5E%5E%9E%9E%FD%3F%D1g%15-%9Dd%22K%3C*%BD%A4%A3%BB%1Bd%10%25%17%B2%DA%83%06D-B%BA%A1%F9%A4FXn%27%1BVh%AD%99%3Au%F6%5C%CC%B6%A9%B3%88%C1E%10%EC%C4%10%0C%1A%AC%2C%00%83%F2%CD7%EB%FE%27V%FE%8B%C6%25%17%1B%B1%938%5Dr%91%3B%81%EAkQc%09%A6%0E%0D%18V%85%86%20BP%20o/%ABu%93%5D%D8%E2DP%174%25%A0%FA%EDv%8C%20%2C%03%C1%D28%1F%F8%02B%F0%0B73%BCp%88%97%7B%DCZ%83%D7%8B%A0%98%E4%8E%0A%7EI%12%25%3B%1A%13%17%9B%FB%DF%AF%94o%B9%C2qsO8%D5HJ%EA%D2NN%E0%FA4Y%87+%0D%7DPf%A9%5E%B26v%BE%F4%D2%FFB%94%9D%B5%08l%9C%9B%9B/%174%D6%89%20%7C%23%A0%BFn%0E1%CAH/%7ET%1Dh%D9X%F7%8E%9An%B4%1A6%1C%16%C5%95%0E%04Y%3D%C2%A3%89%B4r%84%8E%25S%A7%CE%AD%8C%A1f%90%1AS%CF%A5%21%16%3B%11%24%021%8E%60%82%E7%D1%9B%D4%E5%00%B4%A7%BE%F1%92%DF%AF%FB%FC%14%F9%98%E96%5C%5C%E6_R%8E%25%99X%07%A5%93%CD%08Y%E2%B5%CC%FB%C1Y%5E%D7%B5%041p%B2%80%20%02%C8%89%A0O%11%B2%CDh%08P%F4%BA%28%C3%B2%CC%3F%F0%01G0%D4%D2%D6%F5%C5/%DCz%EB%AD-%E8%FF%7C%8A%11%0F%91%A7%D4l%8D%93%02%82%C8%1C%7D%92f%7C0%D5%E1%E5cZ%D0%E4%24%C3%E4%25%3B%1Ac%16a%B8%BE%7F%FC%E9%CFoA%FD%B7%B5%0E5%92%0E%AF48%7E%93%CE%A9%9A%CA%F3%1AO%AA%A0%A7%D7%B36%18f%B2%3C%5D%3E%3F%B7%E6%91%CF%AF%99%FF%DC%1D%81%10%C7%8D%F4%92Ge1%C8%DCUR%ACj%1D-q%E2%A8%18%065%06%FF%9F%F6%DE%C6%3F%8E%E3%BC%F3%EC%EEjR%C0%60%D0%DD%20%FAm%1A/%9C%19BM%A2%7B%C8%99%1E%0B%20D%C8%1A%CE%00%E4H%16%7D%90%15_hoD%10%94%2C%F3E%B2%D6%91%28S%A4H1%8E%A4%B3%5EV%22m%11wT%2C%C6%94%EC%28%24%B5w%BB%A4%7D%1B%C6%3A%91v%18Y%91%EE6%17%AFd%27%BB%D9l%B2%C9%ED%DD%BFqOU%CFtWU%F7%0CHJq%FC%D9%CC%C3%17%0C%F0%A0%BAg%BA%BF%FD%D4%DBS%BFjf%B0x%8C%F7%81%AD%E8%3C%82V%C5%11%11%3A%82%84z%8D%AB%8AI%14t%F1%1A%99%CA%F9s%B8*%5E%29%0A6c%04%CB%A2%ACg%B7e%CDZP%0F6%FC%F5%5B%D9a%0Bu%AC%89q%0B%05%AC.%D9%E1%C20%8D%AEMM5%0AC%AA%7D%1C%FE%04L%18T%5DYo-F%10%25d%5Buft%BC%8D%A0%14%22%C8%0F%D0%88%BA%15e%94%DB%B6%C4%23%98%05s%D3%A3%60%7E%EA%81%F9%F9%BB%EF%BE%EB%EEK%E4%CC%8E%C1%23X%80%AErd%89P%D8%DD%1B@%ED%AB%98AM74T%E7%9A%D0p%B3%7F%F9%D5%1D%FF%F0%FF%1A%C6%D5%FF%EF%EAp%02A%C9%3A%B8A%B2%B6A%9D%8A%E4a%F8l%2C%82%DD%BC%5D%AF%06%20%08%21h%23%10%B8%E6%9E%AD%3F%B9%94%E9%97%24%CC%19%83%20%92-%C3%CDLe%2C%B1%0A%17%23%15%C1L%06%10%FC%60%D8%D4E%86%05%A4V%84%7BI%1D%FE%9E%EF%B2%1D%28%88%82V%AB%22%C6Qp%E5%8A%98B%D0%17%D0%F0%CF%DE%FAY%B9%0E5i%E5%03%20%D0%92%7C%21%DD%80@%88%A2%D0U%93l%15%1EN%5Dg%18tT%14%AD%C2%23%08%B2%90%05%06%C2%EB%06%F0%A3+%5B%B6.%ABL%90%0C%11%8C%3A%C3%EC%B6%14%B8%AE5%5B%8A%10%C8%D6%13J%21.%8E%FFT%14%14i%04%E7%17%0E_%3A%0C%FCL%FF%29%7C%27%29%7B%0F%F3%08%CEc%25%90p%95%5DR%BE%B5%ABW%AF%E8%F5%8A%AC%A9%AA%A4%29v%8Ds%C2%CDF%FA%DF%BD%F4%FF@E%FC%DF%FE%1Bf%90Ep%09A%0B%7C%C9Z%C2%BD%069%81%60%DB%BBj%7B%CB%9E%BC%CE%AB%81%11%BCr%E5%EAKw%BC%BF%FEO%D6_%01%CEl%A9J%3Ft%E4%5DI%BFu%2935%9F%B9%CF%EE%9F%29vF%10%DF%29%5D%A7%11%14%15%0F%89%EF%E1W%F7%89%96%C7%8E%88%E3%28x%FE%85%F3%1A%FC9%9F%D2%16%9CK%18%8F%E0%F2%5B%15%057%AE_%B6%AD%CE%08%DA%1Az%7F%EB%5BH%0A%00%C1z%80%D7/Z4%83%AAa%C5Q%10%1BW%B9%D8J%5B%F7%1FW%FC%8A%C1%0D%E4%B6%00%C4%08j%02%DFP%84%8B%ED%DA%0A6%C7M%8A%D5%A8%18%BC8%063%08V%0Fo%B6%04%09c%D4l%5E%92d%BB%CA%B7%05%C3%AAvz%E2%26*%E2%C05*%AA%5D%D1Hm%CC%F7%CF%E0%BAZ%E2%D5%F5%7Fw%F5%EA%D5%7F%B8%E7%1F%80A%FAf%07%1A%5E%BE%09%1F%07/%C1%1F%3E%22%A1%DD%F4%27%8E%BD%DB%7F%DE2J%19%AE%EB%D5%10%DC%D3%A7%AF%5C%D9x%C7%8E%CFo%FD%93%21%CC%19%5E%C5Z%E4%DE%95t%293%BDy%BE%3F%D3_%9C%E8%EF%8C%60%99%18%3D%3B%82%02zQ%AA%CA%84A%DC%23%FE%CEy%F2%E7%3B%C9A%19@%B0%1A%97%ACr%08%96E%E9%C8%F0%5B%3F%BBE%13%C5%C0%FC%00c%D2%A9%22%D6%24%E3%EF%FF%FE%EF%91%84%F0%02L%84GMu%85%DA%C1%24%B0t%A9%FD%07%F3%E0%1AL%9F%A5%22%E3u%D9%C4%14%BC%CE*%CB%B4%9D%00%C1%B8%1Au%B1%06%01%87%A0d%B9%E1%AEa%29zIu%12y%D3%A3%E0%DE%E6%0C%FE%F2%3C%04%B2%89%05%A4%07s%E9%3D%E2%E9%89%9B%E8%8ET%8Cz%A0f5%DB%A8%D8fb%FC%0Cn%B6%88%AE%EEX%BF%FE%F3_%B8%E7%9E%97%80A%FAfk%80%9E8%ACX%80%A0jn%1B%B6%EC%2C%7D%ADb%EF%F6%9F%0B%7Fs%E8%D0%DF%08%3C%82%1D%AF%86%3By%E1%DC%89%D3%E7%86%80%C0%3D%14g%EC%BB%02%02%E1r%CC%60%8D%C0%04%82x%ADu%88%60%F2%F3%22K%A7%F7%0EI%22%88%E9%FB%F6w%C2%BF%0C%82%9B%E7%E6%3C%D5%C9%E1%C5u%F0%DCx%18A%EA%82W%D0%EE%0F%86%ED%ACeh%9Aq%24%0B%F1%C9%D2%3BtG%A0%17%82%AD%25%1Cj%91o%E2%F9K%3B%90%22%02u%12%97%94%80n*V%2C%C3PZf%60%F9%11%0EAAj%7F%24%0D%19%83%F5%04%82%8AK%16c%A6E%C1pUrz%14%B4%F3%D3x%A4%C8%81%EB%7D%7BCWf%0Fw%18%17%BC%29%04%15M%AF%D8%BAQ%A9%5B%95%C4%60-%B9%B5W%FE.%B4%FF%B2%E3%ABW%19%04eQ%BAs8%8B%B2w*%A6zt%C3%9D%F2%86%3A%B5%282%F6n%FF9%19%5C%F9%9B%9Fo%A7%BC%DD%AE%C6%E4%85%EF%9C%3Cy%EE%C4%A6%A1%A1%3D4g%EC%BB%CALc%0132%02%98@%90%18A%10l/%FB%91%2Cvm%3Es%21%01%C1%FF%FA_%A9%BFI%04M%13%CA%90%5B%CF%23%A8%BA%D0%3B8b%E9%7D%BBv%ED%82%FE%B0%A2%A0N%F3%23%9A%95b%D1%EF%9ATSPU%0Dh%A0%29%1A%DDB%AF%E8%F0%0C%84f%E25%CD%0A%83%A0%03%17%B6%9D%F4%A2%F2%D2%02%B8%966%B2-A%10%D3%C5%F3%C4%D4%1D%11-%15%B7%05U%CBl%99%CC%D4%8A%03%CD%99K%D0%26%C7%BAqy%C5%5Cht%98%1D%B9%29%04MUu+%15%ADb%A7%C8%F0%A8%F8.%9D%DF%B8%A6%05%E1KW%3D%AAN%D3%84%FA%D1m%D6%92%AC%DCy%E4%88%AC+%B2%A8%A9%96%98%E2%DD%FE%F3C%F8%07%87X%04%13W%83E%F0%82%D7%3CS%BEJq%B6%85%7BW%99%85%C5PBo%22%E3y%5C%1E%82%EDU%A1%3B%9C%09%3B%C5%DC%F4%83%E5%D1%EB%CD%13%08%FE%C1%1FP%7F%93%08%1A%0A%9C%1A%EE%BC%AA%F2%08%0Ae%09%0B*%F4%29%87%0F%3F%60@%14%D4%ADZ%87%3C%03-%1B%DE%E2%5C%CE%21%AB%E21j%D9%E8%E3%AB%86-%B5z%A5.y%E1%B23V%81a8%AD%F2P%5C5%95%12%7Bp%F8%9B%0B%BB%24%FC%90%23T%F6H%11%A5H3%D7%84%8B%1E%BFE%0Dek%C4%E2*%82AP%AC6%0F%F75%88na1%B7%D0%1CH%DD%A8%A3%B0s%FA%26%10T%03C%0BtK%E3%1E%27%EA%DC%E2%93W%EF%F8%FCz%88%82_%B8J%0F%0F%88%9A%F6%C2%0B%B65l%1D%B5%A1al.%89Z%25%17%23Hy%3B%20%C8_%0D%06%C1%EFx%CDB%E1%F93%14g%1CIb.%B30W%5C%9Ch63%29c%A4%EB%BC%7C%18%0Agx%02%01A%9F2%06A%11%10%DC%BF%9F%FA%0B%08%C6q%BD%D0%04%04%15%8C%20@S%0F%BCE%B8%1Bs%D4%05W%03%AC%1E%B1%F3%12%98%A1l%B3Q0%28%A4%9BfG%04%3A-%02%D5%5C%F4%F15%5DwYS%95%1Aug%104%98%5C%5C0%0C%939%B6%BF%8C%23%A0%8A%E44%D5%0CA%D6p%03%86%12D%B2pU%15%154%12%5B%98%194%82%28WmL7%E0V%14%8Bs3%8D%AA%AB%A7%D1rsi%0Ab%ADn%A8%F0%BEm%AD%F3%8E%84%7B%A0%3B%B2%E6%BF%FC%C5Uv%9C%5E%B2%8C%9Af%EA%BBI%D2%80%84%D5/L*%0A%C6%DE%94%8A%18%A5%5C%8Dx%8C%0E%9A%82%3F%F6%E6%0A%85%99%5D%C5t%CE%88%7E%8B%9Ai%CEM%00e%03%29%12%25%C8%09%19lz%06/%A8%80%14%C6%E8%91A%F4%DA%9B%7F%BB%7F%3F%F5%F7o%FF%F65%25z%DA%09%82%06%AE%88%01%C0A%CD%C3%8F%06%8D%A0%A8%3A%E2C%12%EA%EB%EB%BB%A4%186R+%9D%E6y%B5v%5B%AE%5D%9D%829%11%8259%8ABv%DBh%04%05%BD%94U%DA%F5t%3Dgp%0DN%CD%E9%28%60%21%A7%E4%91%C4%08%EA%29%19%AB.5%EC%8D%EC%5C5%DF%5C8%3C%3D%B3%D0%CCWU%B3%D3%9E%267c%00%8Fk%28%E6%0A%1B%9B%5E%7D%89%27P%10%91%0E%F0%BE%A0U%C8%DB%ADA%85%91%A5%DA%82%B17%EE%8E%88%F19%93W%23%AE%13%26/%5C%B8%80%11%2CL%CF%FD%A8%C5%D9%5E%5E%B3K%B7%155%B30%93Q%12%BA%5B%D8%AD%DB*0%D8%BF%D9sm%DE%CB%CEC1.9%25Y%CB%88%C2danb%CEs%F0%C0u%A0%D5j%25%1EA8%AB%06m%E1W%AC%BE%3E%BD%0F%BA%8C%15%BB%93%DEG.%E5f%97%A2%E6%9E%91%E2eg%12%F5%8A%1A%F5G%B0%C4%E5%0D%DCh%CEj%B4%FE%9C%04u%19g%8Ch%8E%A58%B5-%D5j%BEZ%DDRSs%F6%A7+fbh%D0%16%5Ca1%C1%D5%A1%AB%89%BCg%11%BAsJ%F8%14%E3%16%B2b3%99E%917mP%A6%DEM%A4%CA%9D%9C%9C%04%04%FB%0Bs%85%1F%F5g%9A3%99%A4%B8%0BQ%B1R3%8DL%8A%FCe%A8%0C%07%0C6%3D%97h%13%5E%EF%D5%92t%13ZW%F5%B0cX%AF%AB%AEc*%F1%5D%B0%81A%1C%00%01%C0R%09%AAp20H%21%08%9F7%28%3B%3Az%E5%15%A4%ABe-ku%96%9C%D1%B3%0A%3ESh.%F4%B0%A9%1B*%A1%A4%F1%07r*m%5Ba/%5D%EE%BA%C8th%85%D72%A7X%960%EE%F2%189%08%BCp%5DL%E5%D3%CEX%15%89%18%25%7E%AAHHI%FD%9D%ABWoI%29%28%B65%ACd%DC%FE%E0%EA%BC%B6%F7%C9%F6%D04%9B%1A%27%11D%F1%08%03%5C%8A%84f%17%04%99*nw%F4%17%80%B3%D47%0D%B6.%D3%9F%A6%5B%14*z9%9Eg%A04%15%A8n7Ij%7D%1C%2C%20%C5%87I%BBX%A0%FB%D2d%94%B5%C1%94FV%8D%24%EEWj%B6u%D3%9AG+%91@%7EG7%95%1C%DC%ADOz%E8%1B.%0F%1FQ%FE%C7%91s%22%D7%BE%EB%EDJ%CDk%8B%3FV%F8%F5F%BD%5Dl0%BA%D5%173%FD%1D%7F%CB%EE%BA%88%E1%1F%E1J%91J%DE0%B2%5C%00%A1%3E/%B2%15%EB%06%A8%EFY%CFz%D6%B3%9E%F5%ACg%3D%EBY%CFz%D6%B3%9E%F5%ACg%3D%EBY%CFz%D6%B3%7F%AE%16%D4b%0B%7EU%5E%9BJXIUD%FA%24%E7%BDI%B3%03v%D60%A0REl-%1A-%1F%B4o%FE%14%D7a%C8%92%3E%F9A%7E%7D%AC%C6%EF%9F%E4%D5R%7EI%89%E6%ED%AD%DA%A7%EC%BD%BFm%9C%D7%F6%89b%0A1%C5O%9B%A1%BE%D9%23%C3g%1E%5C%E1%1E%DA%81%1A8I%8El%87%9B%FBDj%FCphf%F4v%8CR%F2%90%1A%7F%9Do%FE%B9%D0%EB%A6%FC%89%A6W%F0%24y%A7%99%D6_%BDy.%B75Z%3D%25%99%BD%26E%1BHJ%B5O%D7%7B%7F%A1%18Z%E1%21%D6%EB%87%EB%10%5B%CB8%D3V%FA%DC%E4%91%C5%D2%9E%BFR%BD%A0%CB%0D%B0%D5%BA%21%DEk%06%89%D8%1B%84%7BE%FD%E7k%A3o%8C%5E%FB%98%BC%8EA%F2%24%BB%95%1B%90C%29%D7%D0C%5C%1E%B4%27%DC%AC%E9%9EWRn%7ES%25I%3F%BAi%ED%DA%B5%9B%8E%FE%9A%04S%CF%0D%A2%B4%8AJ%ADT%AF%AB%E9%08%E6C%EB%00%D9%CD%7B%1F*N%84VL%20h%C6%26t@%F0%26%8E%8C%FC%83_zt7%AA%7B%F5%0E%F7%D0V%03SD%FA%92%24%9B%01%A7P%89%F7%DAD%F2%B5%D1%D1%D1k%A3co%7C%C4%AE%27%F0%A4%8Ah%18%F0%F7%C3%8A%94%8A%A0%C1X%B6%0B%82%28%CC%07D%82%DC%7E%C1%98%EE%89%8E7%D8q%17ED/%7E%D1%13%C9Z%C8%1C%19%7F%FD%DAG%D7%5E%1F%1B1%3B%24%B4Hm8E9%FA%0D%D4%0E%BCR%8A%92%12%CE%E4j%BF%1F%D1B%9D%CA%CA%D11%E2_%87+c%04w%FEp%1B%D8%0F%97%C1%0E%AA%BA%97%B2%CDlt%B3%AB%5D%21%BB%19o%1B%94%C5%24%829%B9m9%B1+%827td%7F%C3%EF%FD%8B%FF%84%17%D9%06%5E%9A8%90%5D%AF%9B%BB%D1%B0%A9%7C%F01%02%08%EBL%CA%3F%5EN%F0%F1%B5k%B7%91%0C%A9%8FF%AF%F1%08%E6%C0DQMG%90Y%16%05%D6%05A%CBo%AD%5B%09%17%B2%F0%AA%0EPT%94%02_%ED%10%C5%D8%D5/%FC*%22i%C3%DA%D1%8F%AE%5D%1B%7D%1D%FE%AD%DD%90v%04Q%AF%85b%BA%22%AAW%A2%E2z%A9%B5%F2%07%05%B5dF4%04f%8F%E4%E5%88%B2Z%D1%C3%B2%A5%A8%AC%5DRIY%BB%E4%84%C7P%E8%0F%E5%29*%A0%F7%C3%1F%12%02%F7%1Ct%0D/%A5%1D%13%DEl%BC%97xg%C8n%D2%FB%21%06e%0E%A7%5D%7D%98%88%82V%3B9%D8%94%3A%23x%A3G%F6%EC%AF%FC%0E%D4%EC%B2%04W%3A%09%A1%E6%DA%D6%8BY%D3P%B2%1F%7C%E6%A3%DB%D0%3EY%A1%D3%EB1%82%1F%E1%E0%F7%EEm%EF%C2%7F%C0b%12%C1%BE%BE%C5t%04mf%A3u%B8%D0BG%0B+i_o%BFH%22%08%0D9T%F2MKL-M%FD%94GP%D4%D7%BE%7E%AD%25%02%7B%ED%F5%B5I%98%F0%B3%A9%951%82%C8%F1kqq%DD%AB%F99%F2c_%F3%02%BE-j%7B%88%2C%81@J%F9%05%5C%06%E5%FC%C1%B8%AC%E2%D7%7CS%C2_+%25%AC%CC%29%1A%E5%DD%B1%D7s%82%0D%07c%D3T/%E5n%93%9BM6%B4%EF%08%D9%CDz%01%94%B9%B0%C9%96@%B0%B5%0A%07/%A2%E8%8C%E0%0D%1F%D93H%EA%BA%08%95%8Dd%97%7C%EER%0EJ%96%82Waf%F5%0F%96n%03%C4%8E%88t%18%04%04%3F%FE%08I%EFB%5B%F0%8DkH%FA%F8%A3wY%04q%A2%F4%A5%C3%E9%08j%94%EE%1B%DE%24Q%EA%B0%7E.%84Hx%F1E%D1%B7%E2%17%8Ca%04%11%DA%0D%3D%B6%8A%9D%D2%24%94%3DQ%B6%AC%17%89%25%10D%9B%C6Z%04%BE%01%9Fal%13_%15%23%D7%D7%24%04%08JJ%19%CEK%23%28%E8%7EI%01%F4%7Dx%EF%3E%D71%B3%BDa@P%B2%02_%812%92R*%EBTY%A5L%DE+%1CR0%3D%AC%7Eh%94%97%28%045%9F1%8D%E9%A9%D1%23%1B%03%89%9B%FD%A9y%C9%0A%A4d%14%8C%16%8F%98lE%FC%89%8E%EC%99%A6%8B%00@%9C%C5%2CY%1E%B78%BFn%E1%E5kx%A3w%88v%D7%5E%3FrDf%15/%D0%C7%1FK%E8%A3k%9F%91n%3B%7BM%92n%FB8%89%E0%E2@%5C%09%96%A8N5%8D%A0%7C%83%08%CA%097%BE2%FBDh%12%06z%A2%3D%07%08%1Ey%F1%C5%0E%08Zk%F1%F3%13%DA%D8%D8%D9%B5%AC%5B2%CAeK%04%04-%5D%F3%0CQb%11%04t%3C%CD%B2%00A%D1*%97%19%19Z%DB%B3%3C%DB2%FD%FA%BD%F7B%BC%E7%CB*%E5%FB%24%D1%F4%02@%10%B7%20L%CB%28K%9D%11%AC1K%CA%98q%8FD%DF%F3S%F3%92%7E%EB%FD%94WR*%80%20N%1F%C7%8B%BE%15%A8%88%FD%8Ai%C5e%A9%F1%96%94%23S%DE%C4%91E%D7sL%15.%B3%8ED%D1r%3D.%B1%B8n%23%08%82Y%7B%18%AA%D9%8Fn%7B%E3c%A4pQ%F0%5D%08%82%1F%E1w%FD%EE%B5w%A5wY%04u%D2%A7%F3%8C%E8%93%B9T%A3%86BP%5E%09AyE%04%F7-a%93%C4%DD%81%EF%F0MB%C4%20%C8%96%15%B3%87%AE%8D%8D%85%00%8E%8F%1F%3A%7B%88V%2C%91%F4%9A%1F%AE%5B%2C%AB%D0F%B9%17%9A%BA%0C%82%08%DD+%82%A3L%B6%8FV%7C%8D%EA%10%E9%DE%11/%5B%03%7C%F7A%19%28+1e%95%B2%08Q%5B%AA%7B%F0uI%B4%7D%CD%F0%11%85%60P%C1%0B%01Kn%B9T%F6%7C%1F%88%A4+%11f%DC%83%5CU%BD%F6%8F%E0%25%BC%D0%5EI7%A1%3B%12u%88s%92%9F%CB%D9%D1%E3%CE%8C%B7%24%8FL%7B%93G%96T%CF5%DD%2C%C4A%5B%14%E0qgo%9F%EDJ%0A%DE%CCz%7C%FC%AF%AF%8D%1E%19%FBh%C9%A1W_h%82%B4%04%F0%BD+%5D%1B%BFM%FA%E8%5D%E9%03%BA%22%F6%5B%E8y%92%9568%83%11%3Cv%F9%F2%A4%25%C3%FF%8A%8C%C4%EE%08%22%84%11%C4/%8E%BC%97%82%60x%AA%25%24%BD%27%97%CA6%1B%08%01At%04%9B%2C%1FI%20x%CB%F8%D9%F11%B0ql%87%DE%18%BF%25F%10%C1%95%11w/a%15%1F%A8%8CEii%1F%8D%11%9Ct%89H%8A%D6%FC%7D%FB%D0%8B/J%F00%C7%22%A0%3A%B4%05%3D%13%CA%2C-%09%1A%E2%CBf%CB%EF%91%B2r%F0%1E8pI%DF%A2%10%ACk%A1%5E%1E%5C2%BF%E2%03%90%1C%82%F1%B8%C7%A6%D5%DB%B7%AF%DE%A4v%F0%86%F6%29y-%03%A2%60k%0D%B1%25%EB%16%8E%82%F1%92%18f%BC%25y%E4%AE%5E%09Y%F0%20%1B%AE%091J%B5M%9F%8B%20v%20%0F%87%08%A2%D1%D1%EF%8E%DD%C6%D4%C3m%99%C7w%C5k%E3%9F%11o%FBcV%9F%AF%EE%F9%15%BC%AE%A9%C3%E0%0CFP%B7J%97mYW%01A%F9z%11%B4%86%05%1F%25%DC%FB%C0H%3Be%B7%60x%26%8F%E0%7B/b%81%E4%E1a%DDN%20%B8a%FC%EC%A1%F1%C8%CE%8E%9B%14%82%95%8A%14J%99%2185Y%26E%15%0FO%0A%DD%20%CB%97%F6%91f%80T%89w%00%B1%3C%C9%F3%88%A4%E3%92%20%27%CA%DAX%F1%F1%5EI%14%2C%81DP%C3%F3%E5%D8%EB%D5%E3aA%BCP%AER%A7kbj%DC%E3%E3%D5%23%B8%11%3B%B2ZN%F3J5%84%17%E0%A2%9A%F0%A9x%05%AC%CA%A1j%AE%9E%3A.%C8%8C%B7%24%8F%DC%D5%8B%27%06%5C/0%5C%C7%ADy%E5RBh%A1n%CA%8A%A1%D8%F6%91%8FFo%FBhl%C9%E4%06e%C8%8A%A5%97E%C7%B7%5E%FE%CD%97Y%04%7D%E8%BA%E3k%1F%0D%CE%E4%13%08%AA%97/%FB%F0T%A9a%14%EC%BC%CA%0C%A2%A7l%E1%FA%17%5EXV*%82K%84%95%DD%82%5C%86f%7EJ%E9%D0%86%05%8F%1FS%3Ctv%9CB%90i%0BB%05a%DE%07%10%EEFe%7F%F0%5Eq7%7C%BC%B8%B8%8CC%AFx%1F%F4D%CAp%E2%7B%C5%F7%1C%9F%D2%7C%C2%08fK%DA%BE%FBDh%0B%06%08/6%A2%CA%12%04%A1%CE%D1%CA%02%FER%A9%E0%8A8%F2%D2%08%12%83%879f%90%1A%F7%18%19%7B%1D%DA%AFo%8C%8E%8D%A4y%A5%96%CEC%02%B2%9B%F3%12%04%B5%5CM7%C8%95%E4%11%A4%C7%5B%92G%EE%EA%C5%08*%AEW3%5C%E8%FCc%E1%04nB%D2%0E%96t%DC%23%FE%CF%1F%7Dwx%F4%23Ief%E9Z%08%8Ab%E5X%E5e%B2%24%9Cn%0B%EA%E1z%D5%16%82%88%1B%9C%D1%A0%9B%1AFA%8C%20%96%E2%ED%3C%91%0C%10%21%B9%85%E0%91%23%09%04%11%A6%E1%5E%91lWA%06%3B%92%A5%C1%8E%C8/%BEh%25%10Dk%DF%18%8B%08%1C%7Bc%84q%8B%B2%E6%EB%C2%7B%E2%BE%0AR%7D%07%AF%EF%8A%8B%C3Qw%DF%278%BE%8BJx%F5%95%EE%07tg%1C%BA@%9E%8C%5C%0F7%5B%3C%D3WE%A6%AC%5E%82o%A5z9W%82%83%BB%BE%89l_%EA%8E%60%7C%BF%E3q%8F%AF%8D%BC%3EFltdU%D2%1BU%A6%3Cdm%EF%08/%21%D8%BD%2CA%D0%AD%E9Vz%14%A4%C7%5B%92%E7%ED%EA%85%7E%B0%9E5%5C%AFl%A8%3E%20X%0Bj%DCd%AD%EA%22%05%AF%DC%7D%F1%BB%D7%AE-%29l%8C%24%08%E2%09%C3I%A3%26%F2%08%0A%ED%7Epkp%E6R%02A%E5%F2%E5%CB%9E%2C%C3%FF%97%25I%E6%F0%E6%20%DA%87%EE%0B%11%84%1A-%0D%C1%7B%F1%22%3C%B8%D7%29%832%D8%BD%14V%D3r%02A%E9%16*%0C%9E%3Dt%0B%07%B0h%97*%12%C4%29ID%01%0EE%0C%82%82%8D%B9%93J%F8%17*%EC%88%22A%10%DA%D7%1A%1E%C8A%92%EB%E78%04%25%D5%CFI%F0%15%D0%95%25%B1%03%82xD%DE%C7%08%3A%B1%06%04%80R%0Dm%F5%E8%1Bc-%1BIz%01%23UUkJ%8DZ%9D%C8x%B7%B7%F6%23%DA%7E%7De%09%82%F1%20%86%9E@p%B1%A5%25%F3a%CAy%BBzE%B2%3C%D9t%E1%B3z%81%E7%D5T%B7%E63%033v%60c%F0%B3%B7%BD%F5%D6%B0%A4%B2K%C1%DB%08%1ANpL%21%0CR%08%8A%22%9C%09%CE%D7%1E%9CYLFA+%9C%EC%21%ABa%25%D9%EC%D8%1A%C4%10%ED%C3%E4E/%12n%AC%E3%EEk%A9%D9%0A%12%E9%AD%84%10%26%10%C4%03%83m%06%CF%26%87%05%89%D6%BB%AAW%F0%F81t%F0%11%8D%20*%11%EE%A4%92%5E%F7M%EE%C4%04A%FC%80%1B%7E%1D%CA%88%10EM%0AA%DF%F1%F0%C3%02%FC%F9%0A%BC%F3%0E%08%B6%1Ea@0%96%D0%A1%FA%AD%ABG%C7%C2%5E%D8%D8%E8%EA%A47%EC%F3%D6%98%BA%94%F1n%3F%24%08%F8%EF%F6%EB+%0B%1F%06%23%E8%18%D0q0T%8C%20%FD%91k%0FE%7D%DE%FBS%CE%DB%D5K4%03%80A%C7%F7U%C7%29y%B5%20%D0X%F5%E2%9C%8A%86MS%19%1E%B6%24%23%B1eB%88%60y%B2%AC%D7x%04%E1%CCz%8D%F4%88%F5%D6%25Ux%04%09%7E%B8A%0EMr%5DQ%3BJ%1D%84%E5B%04%F1%8BdcO%BAO%AA%94l%A9K%E9%F6%FD%E5%21%13%ED%91%F1%B3o%BC1%F6%C6%1Bg%C7GR%B5%87%A0I%18%8A%7EI%86O%23%E8%87%BA%21R%85%13%FE%24%5E%DF%0B%1F%14%09%A2%27%7E%81G%A9%E3%CE%8A%1FN%A7X%10%B5IQ%0B%22@%3CG%5C%AF8%9E%E3%E1%18h.-%99%B8u%A4%C5sR5%BD%3D%C4%B5%BD%8D%E08%85%60%E4%C5%A3%22%A2%025%1E%D5%BAe%BC%DB%89x%16%8D%60%D7%B2%04A%5Dw%2C%13%FE%E4t%9D%BD%0B5%3D%1A%F9K%3Do7/%D1%86%D0%B1%92H.%07%A0%0D%FA%B5%C1%3A7%3C%8D%05%C3M%C5%B2%90%15pUe%0BA%A9f%AA%9A%87%C5%21%18%04-8%93%02%F5%0F%95%B4%C5%20%18%EFy%24%E19%98.%9B%87J%E1%1C1%02%9A%C8%1Cq%12A%11%A2L%27%D1%82Vi/%B5%2Cn%F0%3D%B9%16Z%81c%E3k%9F%EC%90%F2%25F%89%5C%92%15%23%D8%1E%7E%94%E4%94bb%24+%02%D5L%F8%8Bxh%B0%5D%B6%F5%23%D4%16Z%90h%15%12/%A8@%95%E4C%10%C45%8B%ADi%8AKk%F5F%F3%0C%AB%C7%C6%C6%C3%BAt%7C%24%E9%AD%D5T%FCO%D7%B5%B4%B2%E0%81%8A%18%13%18%23%D8%BD%AC%80%D3%B3t%3D%A7%3B%F0%C7%25%08%8A%1D%8E%DC%FD%BC%C9%23%938h%D9%D9P+%C4%0E%82%E4%0E7%81%8Bte%F8%88%A4%F2%5B%B9%06%28%14%A5%C2%D2%12%AD%17TCR%D3u%7C%BE%92%E7%97%F1%B9%CB%3E%9D%7B%29%26Dr%BC%CE%D9V%D1%9E%29%D1%0B%0E%C1%F4F%20%5B%3A%12%F4M%98%A4oxr%D3%93%1B%F4%CE%B9Z%B1%06_%DA+%A1%BB%B5%7F3Y6%5DD%C2sC%95%1EhE%8B%86%E6J%8A%EFS%17%1E*%162%D0%A0%28O%8E%8C%85%04%8E%D1%DD%91%B6WA%B5%20W3%11%DB%E7%A5%BC%C9%28%D8%B5%2C%3C%88%3E%F9%B0%E1%1F%3CS/u8r%F7%F3%26%8F%2C%B4t%8Dt%DB%CE%DA%B6%AE%98%7C%EA%26x%07%1Dd%1DAj%60p7%3F%99%B2J%E7%B5%D6td%D6r%81%87%BB%1A%92%943%B2%1654-jQb%26%D1%0C%0C%12%D3%D3%D7o%A8%AC%7D%C2T%3F1%C9%F5%3F%9D%C5Y%D3%8A%A8I%12%3C%9C%15%FAf%C7%B9E%23%21%83c%23%23i%5E%9D%C4%1C%D3%D5%84to%B2-%D8%B5%2C%20%C8N%1Cj%F4%0C%C0%0D%9C7y%E4%D6-%C0%F2D%BAn%A5%88%F0%E0%B6%A2%16%2C%E9X%3EON%8C%5Cs%89%FBtM%AD%B9%26%8E%82%1E9.%1C%5D%96m*%D2%05%7C%14%BCy%88%D0%3F%8Ep%D0%3F%99a%21%1E%C5%CC%A9%9Eg@5%CC%ED%3FI%D5i%95%D5%DB%C7%A12%DE%3E%22%A7y%A1%BD%25%3B%95%0A%BD%D5%0F%E3M%F6%88%BB%96%25-69%EA%10%E34sQ%B8%99%F3%A6%1C9%FA%D8%28%5D%EF%96%F4W%EA5%AD%EE%E4L%25%B5%20nJ%1AY%9D%07A%AFU*%8Eluh%89%D1%D2i%09%DD%B4%1B%BCe%FF%5D%01%18%9B%A4A%93%90%7F4%E1%1E%E9%0AV%06%C4%F2%EFGGV%AF%1E9*v%F0%8A%10%3E%98%EA%81%F1%3E%C9%8F%0Bv/%DB%DDn%E0%BC%DD%8E%1C%AA%8Dvpu%91%FAj%F5%29%92%7E%09o%CD*%12%95%7C%FC%06tN%F9%EF%13%E9%A6%F5%ACg%3D%EBY%CFz%D6%B3%9E%F5%ACg%3D%EBY%CFz%D6%B3%9E%F5%ACg%3D%FBgm%EB%98%CD%7B%0B%85%AAz%FD%DEn%BAL%82%13%FDx%B6%3F%DC%1F-%D3%BF%EE%BA%DE%12X%A8%FAC-%CC%D6%B5%1BS%CD%B1%D6%0D%E4%F3%F9%81u%D6%0D%95%BA%5E%1B%08%3F%CF%C0%27%3FRdj%AD%A6%A6%EAm%09%B3%19%DE%F6%DE%E0%B1%3F%91%B5%B7%A3%F9G%3B%81Z%E4%86%DA%A5%FC%BA%EB%F5v%D5e%124+L%C8%DD-g%FA%24IzH%7CH%DA%15%ED%E9%22%3Bx%EE%C2/%05%AE-%E1%FDbx%A1%10B%FA%60%10%01%8F4%B9%CB%EA%98%A45%F2%0B%D3S%BB%A6%A6%9B%F9%86%10%F8%7E%8C%AF%93H%F0%E0%B2%EB%82%C1%96u%3B%7C%26%DFl6%E6%8A%CC%165+%1E%B9%FBsQC%A2%5EK%DD%11*%23%9B%A6n%9AT%A1%D4%ADq%3A%9B%EAD%A6%DE%A8Wxru%CB%D8%7D%7C%04%27%A1%60%E6%AD%B0%9FU%27%AB%A2%A9%7C%BE%98%8FmZ%1A%B8%5Eo%80Z9%B3%A8%BDF%9BI%F5%D97L%CC%922K%EDW%D1%9E.%5Ecafffa%DD%BA%A0D%92%9B%3D%0E%F5%00%F3%AC%1E%89%D4%B14%B1K6Z%C2%D0@s%D7%94.%E4%E6%A7%9F%1C%3BtHskq%BA%96%A6%5B6c%0A%97%F94%A8%E3e%0C%92%84%11%04%84%AAN%F2%F0%10%037%BF%036%97%C9P%3B%D1%ACxd%E6%B9H%BE%E7%9A%18/%DDD%8D%7C%EC%C9%E4%AB%AC%B1%BB3%99%8C%A5%5D%0E%27%F5%E5%F5y%9F%1C%B9%D6%B2%11v%CFuO%B29s%BB%28z%D8%1B%19c%D2%16%0BB%BE%C8X%5E%28%5C%AF%17%EB2E%00%86%99%FD%F4%AE%D7h%98%A80%0C/e%A2W1%82x%D3%D3B%B1%18%BCe%E1%99u%C9OCp%89%1C%F4%AF%91%A9%01%81%F1%0E%EAj%3D%C5%98%07x%60a%D7%A5i%E5%81%A9%F9%F9C%1F%8D%8E%1E%12T%1A%C1-%EF%BCSz%27%B2-*%9Bn%AE%0C%9A%BAe%89%88%20h%3B%82%9D%17%12%96%99%C8%CCU%DFy%A7%7F%B1%B8%99%82A%D3%B5R%E9%E2%C5%D6q/%96%02%EE%C8%F1s1%3D%DF%1C%E0%D7%81a%06%91%D8BP/%08%03%F1y3slc%A80%C7%22%D8%F1%9B%98%2C%19%EF%F1g%DB%28%1D%C1.%5E%20p%B4%B5%BC%E1u%96A%F5F%A2%A0%BD%F1%3F%BD%855%7E%DE%02%7B%FF%AD%B7%FE%9Ca%B0%0D%D9%FC%CE%E9.%08%A6%7B%01A%E4%D6%8E%1D%0B%85*L%C0%91A%F0%88%F5%E2%91%23%E8%88%852%08%00%84W2%A2%10%2C%2C%16%0A%3F%CA%9CX%BBvd%7Cl%ECZ%99C%D0P%01A%E9%08%5E%B6%F7%9C%A3%A9%B2fZQ%80M%0D%87%F4%0F%1B%CD%9D%7D%C2%A5%9F%DEz%D7%94A%F2%B1%1A%06%8D%E0%3B%B3%EA%DB%EA%DE%D2%F1R%A9t%BCRz%87%CD%FF%AC%0F%9A%E1B%08%82%A0P%B0%85%02%7D_Zm%DF%89%FE%C5%DBgf6%F3%08%96t%7D%20%DFf%BBQ%E22K%E3%E7%C2%D2%A7%9Al3R%C5+%83%CD%A56%82%05%21%EB%D0%08%16%BB%23%187%12%3B%22%D8%8A%15%1D%10%EC%E8%A5%08%1C%1B%3B%3B%F2%24%ED%D3%B0%BE%E0w%ED%96%96%02X%E7%96%92%BD%F1%17%7F%FEm%08%7E%DF%FE%F6%B7%87%86%86%7E%F9%3B%EF/%D3%0C%B6%20%9B%13%84%FB%F3y%BC%FFo%1A%82%1D%BCX%97I%B6%EC%10%C1I%EF%B2%C5%22h%A1%23%08%EDC%C8%CAD%AF%12%08%0E%3C%B9id%2C%05A%C1%08d@%D00uC%D1TK3%87%11%85%60%5BD%226%1AA+%3F%25%08%0F%EC%FC%E9O%EF%BA%1B%89%C0%E0H%5Ea%10T%97w%FFL%05%00Kg%F6%3C%5Cy%87%8DU%83%A6%8D%A2%B6%E0%AC%0D1%90%AE%12%05-%87%93%F6T%AD%7Fq%B1%7FK%11o%5C%CD%20hg%1B%0FT%5B%7D%AFj%89%3D2%7E.%22%88%A4%A9%26%B3%9F%60%0D%C95W%86%28H%B2%B0%F4%82%E5%14%E20%09Qp%A2U.%8C%03%1C%82%F7%C5p%8A%1D%A3%20%A9*%012%F1F%BC%84%C0%98%C1Q%86A@p%D5%D8%D9%B3c%AB%2C%224%A8t%16%F4@%1B%7F%F1W%CBW%00%C1+W%AE%00%84%CB%3F%FB%AB%A1%F7%29%06%09d%8B%F9%B9%3F%15%3E777%07%9C%25%11%EC%E8%C5%08%3A%C7J%C7p%12%0F%3AV%9F%CCr%08B%3FcIZ%92%863%D1+%06%C1%E2%96-%5B%7C%BF%A2%05%AE%21%E3%8A%98%CD%F71TW%3AbC%0BR%81%1A%CE%CC%0E%A3%082@0%1B%ED%F0l%87%ADP%1A%C1%D9%85%0F%85%DCO%C1n%BD%94%3B%9C%1B%3B4%DA%ECc%10%2C5%96%1Bg%C0%B6%5C%9C%DC%3F%C4E%C1%C1%25%5B%1A%9C%7Cm%D2/%C3%CD%84%00X%A4kbU%D3%2C%2C%16biZ%BEz%E6L537%C7%22%A8%28%0D%03Zj%A1%B1Q%90%3C%17%85%CB%C7%A0%E3%1B8%05%11M%E7%E9%3EI%80%24%D2%EA%00%161%81b%A1@e%04%02%82a%3AT%7B%15%D3%ED%2C%82t%C6T%C7%28%18%E6%12v%8A%82%E9%DE0%06%AE%22%0CnZ%C53%A8%A1%B7%CE%E2eCc%9BZ%89%8A%1D%11%04%02%87%7E%F1%CB_%FEGl%7F%05%F6%1F%7F9%B4%BC%BC%91%81%0C%18%9B%9B%13%C9v%B3%80%19%8F%60g/F%F0%98n%1C%23%D9f%C7TE%968%04%C3%F5-%18%C1%D6%AB%18%C1%3F%1B%1B%1B%F7%CE%9F%9F%9C%84%FF%CE%9Fo5%26%987%AE%B9%92%AC@%23%05%EE%B8%A9%98%8AE%21%28%DB%11%82zk%C4%80Bp%60%1A%E5%EE%C2%08%DE%D5%27%20%E50%D2%17%5Ea%10%3Cs%06%08%FC%D9%16h%B5M%EE%7F%84%8F%82%C8%96%DC%C1I_%0Bt%BC%21/%FE%B4%C5%B8%AC%A4%CA%8A%AA%1A%8A*i%98%E1%B9E%0EA%C3Xg%E4%BD%99f%1E%FE6%AB%1E%7Dd%F2%5C%00%82%D0%3D/%B9%85WP_%93%1E%A0BZ%16_%20%C2%20%26%90iU%C5%08%B6%D6RN%A4%20hu%EE%94@%8BV%C6A%0E%1A%18BRo%BA%8Bw5%26p%D3w%BF%BB%09%FE%FFX%DE%04-%A6x%A5%17%BE%1A%AB%80%C0%5B%E6%16O%84Q%B0%B3%A0%C7%C6%B7%DE%FA%E5%FB%60_%C0%F6%12%B6%BF%1C%1Ab%10%24%8C%CD%FD%C6%87%E2%87%F3%29%08v%F1%12%04%A1%29h%AA%04%C1%CB%0A%87%60%EBz%00%82%EDW1%82%A3%87%C6F%CF%1F%9B%C4%06%87%B8%7C%F92%FC%CF%20X%1F%D4%5C%CB%86%D6%A0%A6%05x%AFf%9BF0%B1%87r%8C%60%89m%22%CF%CA%86%3B%FD%21%87%20%84_%B2%DA%F2%B1%FD%0F%27%A3%20%92%ACVE%5Ch%08U%C1%AE%C6eE3+%E3%BBm%84%08%B2U%22F%D0t%CD%C6%A0_%EC%CFW%B7l%F1%26%E9%FCz%F2%5C%60%04%B3N%60%16%F0s%C1%B4%06%F5z%ADV%CBa%06U%20%90M%8A%06%04%C3u%3B%C5%BDQ%F22%F5%1B%A6%88%858%3AwJr%15Gh%E5S%03d%15Ns%B6%9Bw%F55R%FD%02%83%9B%3EF%9B%B0f%01%8B%E0wG%C6%5E_%9E%3Bs%94%E4kK%5D%10%DC%B8%F1/9%04%E1G4d%C0%D8%C4%87%13b%13%ABF%80q%08%C6%5Eh%87%B3%5E%8C%20%BE4%938%0E%D6%8Fy%88E%D0n%A5%A9%DB%19%25z%15%21%F8%F1%BBr%B6%23%82*%DE%C3%DDTkx%EFrk%D867lpM%9DB%10%B5%BA%C1p%FApq%20%85%A07%CC%98%27%3A%C6%B4%C8%20%B8%C5%02%02%87*%17/V%F6%A7U%C4%26NA%17E@%B0%21%D8%B3Bc%96BP6%0D%88%82.%EA%80%20%3CejU%9Bl%F8%BE7%F9%DAk%AFy%D4%91%F3S3%F3wc%04%15%ADn%16%F0s%91%E7%EF%142k%222%8A%3C%81B%E6%F6%02%81D%1Fp%C2En%96%5D%F1c%CAL%92%F3%DC%B1S%92%AB%08N%B4%F1%B9%E4%A8%2Ce%5D%BD%21%828%02%22%200%81%20%BC%E1%AF%9D%3E%FDZ%19IP%1B%B1%3A%0C%9C%D9%1B%7F%F9%0B%C6%FEr%23%E5-%08%18%BB%5Bg%26%1E%9A%DE%29%0A%9B%13%08%B6%BC%8A%A6%29%9A%A4U5%8DEPj%21h%B5%D6%D7Qo%83%9A%1D%89.Nt%3B%BDw%AF%BD%3E%0A%D5%F0%E4y/%05A%0D%FA%A45%D3%ADA%D5%ADA%9FX%B7%B3P%11G%C3.%18%C1%F0%05%5Eg%9DDp%22n%9EO%0C%7B%92%B2%F70%87%20%0E%81g%1E%D9%0F%F6%F0%99%14%04uK%96%2C1%A5G%ACI%C8U%26%27M%C7%EA%80%60%CEU%83%D9%A0%3C%89%F9K%20%F8%C0%FC%3Ci%0B%1A%80%20%7E.%12%08%E2%C5%1A%0B%05+%AF%F2%08N%14%FA%FA%FA%F0%A0%07%5E%FA%0F%9D%A1%9Ao%94%E3%01%92p%95o%A7N%090%269%AD%25S%A7Wo%DF%3E%F2%24M%19%ED%F5%7D%CD%95%B3%B4%B7%85%E0%D8*%24%AD%22%04%8E%BC%C1%20%08%EDW%D9%92%97P%F2%DE%F3vt%23gG%13%08%3E4%D1%14%3F%BC%7F%F3%F4%F4D%1A%82%E0%D5%24CSmU3%E8Fg%8C%200%E4%27%DF%06%BF%B6%A0%9A%89%DA8%DE%E8%F8%F8%D8%F9+%87%C6%C7%FF%03%7C%B2%AF%1D%E3%10D%BAXS%D5%9A%A8C%23%89%982%2C%B3%08%E2%93a%04%C9%B5%C3%AFc%04%0BS%60%F3%60SS%85aO%B6%AB%0B%0F1%08%86%92%07g%86%1E%7Ex%E8%CC%C5%AE%08%E2q%C1Y%EA%F3J%B2%5C%F3%BC%92%D5%09%C1p%1C%BDB%86%BF%DE%DA%F6V%85Fp%7E%E1%F0%25@%10/%232%0B%F8%B9H%20%88%E0%19-%88%8B%0E%E9%930%08.%02W%DE%87%08%8F%15%E1%85%D5P%11%9B%F1%272I%14%EB%D4%29%A9D%CBd%7Fs%24T%12%A3%87%F7bo%EB-%20%3D%16%3F%89%A3%A0%24%BD%3B%3A%3A%3E%02%07%60%10Tb%5B%01Aa%E3%D0V%CA%86%E8%20%18B6%F1a%B3%B9y%F3%CC%FD3%3B%9B%13%CD%04%82%D8%AB%19%AD%21p%83A%F0%15%B2%ECe%F2rI%F2/%3B%DC%DB0%FA%D9%89%CD%7E%E8*%C6%08%DEv%DB%C7%FA%89%F1%16%82%23%AB%E1%F9%5C%BB%96AP%D44%11%23%D8%5Eu%86%A2%D5%C0t%14l%D5%C8%2C%82%D3%D3%D33%FD%603%9B%01A%3D%98%7B%9Em%0B%5E%DCBY%02A%8BF%10Z%82%B4%B4-%B2%AC%12%10%AF%EB%DD%10t+%BF%20%0F%E6%07%1F%D0%08V%0Fo%B6%84%C2k-%04%F1s%C1O0%A3L%FE%F0B%A6%E0%18%B2%C0%AE%90%CB%90%F6%D1%9C%27%86%B5e%D6%F3k53%BAV%D2%0A%08J%0E%12%89Ve%A4%24%B6%3A%CD%DB%AA%8B53%81%20%D4%C3%A3%A3%1F%BC%3B2%B2v%ED%DA16%0A%D2%26vG%F0/%86%28%FB%8B%14%04%A7%EF%07%02%3F%9C%DF%7C%FFt%1A%82%E0mh%B6%A9%D55%D3%D6%FA%E97A%A4%26%01AI%B4.%D78%04%B7%2C%E8t%A3L%7F%20%93%19%885%0FP%D6%0DN%8C1%08%8E%D0%08%22%D2C%C4%08%0E%0F%5B%D6%11%84%E4%A8B%8C%DB%82%80%60%8BE%06%C1%7Cqf%1A_%95%89%09@P%99%3D%9C%97%B8%D9%11%CA%B6p%08%CA%C8%C6%08%22%942G%0CoK/U*%A56%82s%DC%A0LP%22V%F9j%CBh%04%F76g%E0rN%1E%AB%AB%01%20%08%CFE%93K%D9%00%02w-%CC%0CdjXz%8Aa0%D3%0C%87%23%3C%08%C2%B2dz%95%A0%5E%B7%A3k%85%DA%08%BE%1C%D3@%23%A8%07%92a%C1%B5%5C%95%A2%24%16%7B%DB%3B%F2T%12%08b%02%C7%B6%8F%FE%F1%1F%8F@%90x%A3c%14%14W@%F0%A5%8E%08%8A%18%B2%DB%27%9A%D3p%88%A9%99%CDM%12%05%C5%84%17%BA%05%B66%9B%1F%D0%A0.%8E%CE%24%92%28%28%86%08%EA%97KD%1A%2C%8E%EB%19L%60%28%8E%192H%11%28x%F8B%9E%18%FB%F9%7F8%7Bv%AC%8D%E0%18%8D%A0%AE%5B%02%DE%23E%23%CAr%B8O%12EP@0%AC%7F%E1%D4%ED%1A%99%AD%88w%CD%CCD%08%9A%0B%8D%86N%23%18%BE%19%3D%9A%DAdgG%D4p1%B8%A8%A4%24%E6%E0%B7U%D3%5E%88%10%84%D0%BEEH%1Ey%5B%A5R%19%1A%82%FF6PG%B6%F3%D3%86P8%7D%FA%D8%B1%3A%20%88%9F%0B6W%01e%1A%3B%1B%033%D5%8C%9E%600%B3%F9vb%C0%A0%28%FA%25%0F%87Z%06A%D2%16%A4T/8%04Ke%7C%B1%D2%94%C4bohe%BF%94%A3%11%7Cc%FC%D0%A1%B1%3F%DB%84%E3%DF%E8G%E4%7F%06A%FE%F2t%26P%DA%F8%85%CFS%F6%85%8D%D4L%84%84%21%9B%98%C0%D50%D8%E6%CD%04A%29%E1%D5%24%DB%AE76%E7%EB%B6%1D7%06%A5%B6.%13T%C4%A2w%D9%25%08%C6%C7%CE%0CC@%19%0E%B5%FA%A1KqD%EF%A7%CE%0B%D7%0D%D9%27%3E%FF%EF%1F9%F0%CC%B3%8F%AE%DFze%3B%8B%20%5E1KF%5E%2C%AD%5E%C7%F2%1CuJ%B5%09c%17%23H%7E%C2%228%3F%B5ys%84%E0B%F3%F9l-n%FE%D4%93%F9%2C4l%F6%A0%89%7BV%B29%982%E1%19%22%A8%D5%08%82%B3%B3gr%CC%0E%05%F4%91+C%DF%AEpG%1Eh%CE%5C%02%041%84f%01%3F%17%1C%81%0B%B76%AA3%F9jF%B6%09%83Z%CC%60%1BA%60%D0%F3%F66%3C%C7u%DD%18AI%0D%07%BC%E8J%91FP1%DD%3AN%F4%D8%9E%A2%24%16%7B%EBZ%AD%86%07%20%06i%04%8F%02%83k%0F%8Da%F6%D6n%07%10G%C6F%BEF%DDB%FE%3Av%D4%D5%12D%B4%F1%CF%DF%A7m%23%D5%DAE%00%D9%04%C4%B9f%88%60%18%05Q%C2%8B%A3%60P%CF%EFe%A2%20jGA%0FwG%BCP%20%91%9AW%1A%960%81Gp%14%03%06%97%18%04%B1%C9%27%1E%FD%BF1%82%DF%3Fp%60%FF%23%AFn%DD%FAe*%0A%92%F8%94U%14+%E5%D1%C2%08%92%1A%0F%23%98%D6%16%DC%3C1%B1%19%F3K%10lT%95A%9F%B9%3A%22%AA%B8d%B2%B7n%96lV%E1%02%A2_8%3BW7S%D6f%13%04%07C%04%D53g%02%9DW%E2%89%8E%8C%A3%20%A7%9D%21V%9B%87%A3%3E%EBBs%80IO%D33%0B%1F%86%04Z%A2%A4%D4%B0%08m%2C%1B%93%D9%3CA%9Bg%3A%8E%13%23%88eV-K7%A5x%90%94A%D00%CD%1C%9E%D2IS%12%8B%BD%D1%8E%7D.%85%60%C8%60d@%E0%E9%E4EQ%82%F4%2CG%F6%C3%3F%C9v%88+%B4%7E@%F5s%D3M%DA%26%A6%A4*Jz%F3%9Ad%06uM%B15%CA%AB%A1P%8E%09%3A%0F9%85l%BF%82%18%04E%1D%13%885%A5%81A1%81%20%3A%F1%E8%F7%0E%3C%F3%D43%CFb%04%BF%F7G%FF%FE%D1%18A%B9E%60G%04%5B%E2%A0Q%1B%86A%F0%F6%C8%00%C1%AA%E2%F8%15%A6%BC%24%D7T%1B%8F%22%DB%8A%AF%B0%BB%5C%60U%0C%19%00%1C%94%D3TN4%F8%B8%18A%F8%A2%A5%09D%C5G%AE%2C%1F%AFp%FBg%A0%5C%B51%DD%C0%F3%EC%C5%B9%99F%D5e%84F2%0B%0F%AD%23%04%DA%B6le%CD%1A%7CB%0A%C1%99%D0%1A%E1%A8t%05%C7%3C%9B%1E%C6%C7%17%80E0%BE%D2AE%09%B5%01%D3%94%C4bo%DB%20%C2%D0%EF%EB%E8%EA%B1C1%81%DBO%24%F5%16%0C%CD%29_O%1A%1DV%FF%D2m%9C%90cq%97Vr%12I%A9%C5Y%29%E9%A5z%C4%947%A9%CB%E4%D0%1513*%23%A4%21%B8%BC%7Cp%F9%ED%FF%EB%EB%CF%7C%FFa%60%F0%FB%D1e%0Dd%5B%21%D3%C0Y+%A5I%D6%3DS%C6%1B%5E%88G%C8%1A%C3%9Ej%FB%89X%A5j%95%C01%EA%25%3F%B0%19q%0C2%EEk%0D%DA%F6%A0e%A5%21%88%C78%D5%3A%A9%B0%D2%AFs%FB%C8%95%F7%DF%AFp%B2%1B%C8%CEU%F3%CD%85%C3%D33%0B%CD%7CU5%D9N%EF%F4%87%B3%98@%23l%A2%3A%01%F4y%23g%7BX%D5%23%7Cy%0A%F9%0D.7%0F%B1%08%C6%F7%C5%C6%C4%E2%09%B44%25%B1%D8%DB%9E%D9s+l%03%24f%10%13%98%FC%C4Y%A9%5C.%9B%9Fh%7Bd%11%CD%F2%A9%F9%B3q%F5%12%7B%A9qA%CA%EB%D4%D8%D6@%CDAtw%C4%A0%BAL%8A%91D%D0%3B_%AE%05%15%FF%BCWY%7E%EE%B9o%7C%E3%1B%D1euLxbp%0E%82e%A6%A5%F9v%CD%17%E4gG%14s%D0%F7Yj%21lC%5DY%AAi%AA%CE%89%1Bb%81*%19W%C4r%BA%D6%93%A4k%9E%A7e%3B%CA%C5%B4%8F%0C%F1%84W%8E%15%2C%C5%A9m%A9V%B1%22zM%CD%D9%8C%3B%93%9F%99Z%A8f%94%96%7E%16t%C9S%DA%01%5E%80%F5%FAUO%D5-%DB%E76%10Cl%5B%90%0A%0D%D1%BE%A4%AB%93Jb%94%B7m%01s%5EQ%C6%D1%93%EC+3%BA%FA%84%C2%EF%10%A7%1BV%D9%94%CC%B2y%3Dq%B0%A3%85%D2t%B1%B1a2%F6BkP%83%16%10%13%A7%BB%97%ADr%832%B9%7E%EE%D4%92*%23%A2%CD%98%0B*%27%CE%9F%3F%E1G%9D%06%290B%15*CM_%B2%80%C3%15%91%D4%D3-%99%D3%92%AA0S%C4%AA%02%3D%9BWl%7E%C77%12%EDR0%8BDC%3B%CA%03Ii%DAU%FC%91u%3Dk%25%05%CB%24%DD%C8%E1%96%BF%EA%9A%0A%9F%B1%9A%C9%E7%07%20%06%92%01%3A%09u8%BDO%F6Z%86%7Fu%9E%C0%A8S%D2%8Ael%03%17%D9%A6%AB%06Z%A8%2462%C2m%DC%11z%EB%B8%C7%C7%87%7EQ7%D5%13%ED%C4%FD%AF%D5%20t3%AAFv%CE.%E3D%3D%B3%AC%06v%97k%B2%A2%89%9Cu%F4jx%B0%F8%FA%CB*%5B%B8%A1i%23%F5%E4%98%25%AC%97M%17%16%ADv%F7%F2f%F6I%86k%9A%83%3B%ED%97J%D1h%C3u%3F%A5%9F%8AP%14Q%3De%E5%DB%E2Ok%A5%06X%1D.%90%BC%92N%95Xj%E9%AC%F9%C9M%12%25j%83%EA%C4%F6%E4%E4*%23%99%28%89%ADJ%1E%B6%A5%0B%97%F6dAXWr%80o%A0%9A%0A%F7%81%A0%DF%04u0%0E%04V%0Eo%E0%D6%D3%D6%EA%D9%AF%DCHG%10%85%3D%C3%7F%EA%F7%D2%B3%9E%F5%ACg%3D%EBY%CFz%D6%B3%9E%F5%ACg%3D%EBY%CFz%D6%B3%EE%D6%C8_GbCl6%BB%1E%CD%BE%91%B2%91%DD%C4P%AF%D2-%13%28%D7%DAn%B0%A3F%D0%86s%A7O%9F%DBp%13%EFT%E5O%AB%FEj%CA%7E%8AV%BE%11%F9%27+%99%BA%16%CF%5D%8Bd%7E%E0%85%17ZSwel%DA%27%1F%00%CC/%CE%158%15%AB%0D%E7%18c%EE%9C%ED1%C25%3A%CB%60w-%A9%20ZqX%86%F7%1F%A4%BCw%AD%5C%F6%CB%7E%DA%1C%B7%C2%EF%8B%2C%99%F4%F6i%98%C1jg%8D%A0%CA%E9%8DW%AFn%3C%5D%11n%D8VH%C3%ECJ%D9%27%29%FB%89%CC%C5%B7%ED%BC%7F%BE%F5%C5%3F%EF%26%BC%B4Q%5EK%13%13V%8E%18T%5D%C4n%FE%26-%C9%C1%0Dh%94%A5%5B%A38%3F%3F%97g%7E%B4%E1%DCW%99lB%9AA%DBC8%3F%B2m%8Ea0%0Cv%D7%92%F2%5C%ADF%A6%C8%02%3C%88%5EKL%F6*%BEb%E9z_%9F%E2gy%17%5E%28%C4*vX%ECZ%FD%DC%C5jg%8D%A0%0D%A7%AF%5E%F9%F6%95+WO_G%1C%A4vf%24%E7%15%AC%E8%C4%BCL%93%B0%02e%9A%10%CA%8C%28%ED%E9%82%1B%28%DB%DD%EA%ADi%C6%5C%AA%D7%3D%97%D5%7D%C3G%15%DB%AEW%FC%3EI%A9%9C%B3Y%AF%82%B7G%B4%ED%3E%DD%EA%D3%15%85%F2%02%81K%D4%C4%9E%0EA%06%C5%CB%815D%B2Rq%D6F%D02S%BE%01%8D%B2t+%CC%CCl%9E%292%3F%3A%B7%FC%85%F6%12%E3%F0%EB%B9%D8%E7%E1Ys%8ByH%E8%24%A1%C4%3A%0CF%99%C0s%82%1A%C9%0AWq%B9%3A%BF%E1%A6%5D%B6%16n%C7K%97%17%2C%3FQ%BF%13%CD%AE8%23%10+v%D0%1F%0F%08d5%82%98%94%D4s%1B%AFl%1D%DAze%E3%C6s+%EE%B1%98%5D%5E%5E%DE%26%D2%E7%05%88%A2%D9%7E%84%12%18u%21%14%23%18%EA%DBt@0z%A8%C2%C5%F7%B4%DB%C7%EB%A9%DBs%B4%B3%F8%1B%3A%C3%CCo%DDc%3F%AD%1Dd%9F%CB%AA%BE%EC%23%B2%C5%DD%A0%A2%28e%B3%7C%DE%A6%BD%B8%22jg%E6%97%CBf6fP%13%97%8A%ECD%7E1K%21%28%05%C78%ABS%FB%20v%BF%FB%9D%CC-%CEOL%CC%17%99%D6%D3%B9e6%A1u%99A%10%9E%8D%12%B3N%8AE%F0%9D%D9%D9%D9%BD%7BK-cW%A3yf%A0a%06K.%5CMQ%E5%13%7D%7C%AB91%BF%AB%EF%D2%F3%13M%8B%CF%FB%20%08R9%A9%12%8B%20%D4%C3%B1v%16%04%9D%A9f%8Df%10%82%E0%D0%FA%A1o%03%83W%A1%3EN%84B%AA%E28x%FC%7B%FB%BFw%7C%1B%CFI%7B%D5@*%82%1D%09m1%86ZQP%EC%84%20J9%F4%E4%BAFc%E0%B5%96%C17%B3%03%93TK%C4W%83%F6%EE%A2%24%FB%879%EE%D1J%9Fo%8A%86%21%1A%9A%A8%90%FD%3AU%B9%ECR%DEI%16AU%3E%19y5%11eJ%03%03%03%F5%81%B6e%8C%18%C1%0A%20x%90%B1cu%8BF%F0%9D%A8%18V%3D%7CG%EF%1A%05%A5-%FD%B8%17%92%9F%98%99%98%98%99c%D6n%9D%5Bf%9B%0A%0C%828J+%DD%10l%D37%5BW%07k%5E%9D%89%82%86%8A%19%2C%95ri%08%AA%EA%C2%C4%A5%5D%3B%21%96-L%3C%9F%D8%27%17%CB%C6%89%92%94%C5%F9%88%01%C9%60%A2%EFXp%91%D6%CE%12%B0F%10-%F9%04%08%E2%28x%E5%EA%D6%7B%5E%BA%E7%8E%9F%3C%C9%1DZ%8D%3E%0C%FA%D2%FEg%FF%F0_%7Eo%99E%7F%85%28%D8%91%D0%96%DA%A2%22%28%DD%10%8C%25H%18%04%1B%B3%D5%C9l%DB%26%17%E0%3B*o%C5%B7%5D%DAX9%D0s%DB4%15Y.%04%C9%8A%E6%00SH%B1%82%EC%B9%D8k%F2%08%DA%B1W%13%AD%0C%26%F0%CE%3A%96/%25%08%9A4%82%F5c%8F%D1v%FEr%5D%E7%11l%CBnn%D9%F2N%F7%28%D8%9F%29f%80%C1%FC%ED%80%E0%F4%22s%C3%819%3A%08%26%11%24Q%D0Vk%BE_S%ED4%04%CF%9C%99U%F1N%B8Y%B7%A2%B2%08%D6%81%C1J%CD%0C%114%18%A5X%DF%9A%5B%B8%D4%D7%A7_%3A%BC%AE%D1%D4%F90%88%A5B%00A%EF2%96Y%90y%04%AB%17%19%ED%2C%AC%11%B4%89%0A%83P%11%93%B6%E0%D6/%DCs%C7%E7%D7%DC%F1%136%0EB33J%A3%7D%EE%07%CF%FE%9FO%3D%C2%22%18%02r%18%DF%93%C6%E1%04Fr%B7%28%18%02%A6HB%3A%82%AC%1E%28%87%60u%F2%CE%B6%1D%3C8%B9%AE%3AIeF%F9j%9E%13%B9%2C%C4%5D%B0s%1F%1A%F0%9C%97Ke%DC%E6%A9%90%FE%B0X%8F%11%FC%D0%E7%11Te%0E%C1%FC%F2%D3w%B6Dt%B7%F0%08%BE%10%D9%C9%C7%1E%BBp%99Z%B9%87%EF%7E%84%1FQ%F6%EC%1A%05%E5%CC%DC%5C1%23%0A%8DE%8C%60%81iRtGP%D7ue%FB%F6%ED%A7%CB%7F%F6%81%20%7C%F0g%E52%87%20%5E%CBx%A6%E6%ECV%82R%B9f%DA5%AA%17J%10%0CT50%08%82GG%B6%D3C3%BE5qx%FA%D2%25%C5p%D65%26%E44%04%B1Z%074%3F%26%8FY%3C%82%17%AB%09%ED%ACM%14%DF%D0%1D%01%087%5E%7D%E9%8E/%BC%7F%CF%FA%3F%19bE%92%B5H%A06%9B%7D%FB%E1%FD%CF%1Ex%84%A9%88%5D%84L%D3%CD7%A7w%7En%E7t%83%12%94a%08E%E9Q%B0%F5%D3%0EQPn%07O%DB%D3y%04%07h%02%81%C1%8B%0C%82v%01%BA%5D%8C%C5k%BD%CF%1D%FC0%B2W%D0+u%7C%D8%CA%B9%0E%5E%AC%07i%1De%11%CC%AB%CF%09o7%1An%3E_%BD%F8%0E%87%E0%E5%F3-%0B%9B%08%97U%85G%10%CB%01%00%7F%17%01%C2%AEQpo%FFba%AE%BF%21%E8%85%99%89%CD%9B%0B%8CoE%04K%DB%B7%9FX%25%EAv%16%AF%B3%D8@%C3%D2V%A9%0A%24%BB%84%C7%E9%7C%D7%A1%11T%00%BF@ss%04%C1%91M%D7%00%E5x%ED%96%DF7%B1%F0%00X%CEq%D7uD%D0%3Fv%B9%5CNC%10k%04%DDu%EB%AD%B7N%E1o%B1F%D0%99%983e%F2%1C%19%94i%21%B8u%E8j%07%04%C1%9E%3B%B0%FF%F1%3DLwD%863_%CAO%A3%3Ecz%7Ez%E7%F3y%9D%7D%5B%84P%B3%BDl2%05AER%A4%AC%98%8A%A0%DB%02W%F70%83%5Cw%E4N%06%C1%CAd%89%AE%88%D5%C2%CCf%C6%28%04+%17%BE%03v%E1%C2%85VC%EA%7CE%DC%7D%FE%B1N%5Eb%91%17%23%B8%A5%BA%FC%F4sO%AF%AB%1F%DC%9B%81HF%23X%96%D4c%C3%07I%15%7C%21%B4%CB%AE%C9%20H%F4%28Z%83%B4%17%BBG%C1L%B1X%28%163%D0%1D%99%99%98Y%CC%A7%21%88%DFZ%3A%82%C6%F6%D3%AB%24%C34%14%E8%B1%23%D1%A5%16%EE%B4%DE%C4%C5%BA0%18%BE%8B%17%14J%AF%04%23%085%B1fBE%FC%F2%C8%18%91%CC%8E6%90%C6Qp%9D%93%CB%E5%8C%9C%B3n%C2JE%10%21%FF5_%D3%8E%E1%9Cj%1E%C1%07%E6%E7%EF%BE%FB%AE%BB/%E1o%89F%D0%A1%B6%D3%9E%3C%F9%9D%1F%E3%F5%BAW_Z%7F%C7%3D%9F%DF%91D%90Zj%60%FF%EC%EDo%1A%DCy%91%91%9F%96u%E8%EAL%1B%D6+%AC%ECFHh%5B%0B-%81%60%18%03%21%0A%0A%E9QPh%13%E8%D4%3C%9Dq%8B%AFei%02%0FV%5E%B3%A9%06%9Fg%B7%11%AC%D7%C3%AF31%82%E6%85%E57%5Bv%12%EC%3B%E7%CEo%DBp%DE%8C%BDo2%5EBR%E4%C5%08%BE%F3%CE%1E%E1%F8%83%7B%8E%0B%7BpM%CD%21xy%DB%BD%C3%E7_%8B%EC%B2%EB%24%10%BCx%9D%08b%5D%9C%C5%8CX%5C%9C%D9%3C%C1%06%C1%15%BA%238%0A%AE.%8B%B6a%3A%0EN%DD%96%84Z%96%7D%13prM%28%87o%A2%9CM%20%18%28%81%29%BE%3B%12%CA%C5R%08jF%B3%B1%0E%22%A4m%E4%F2M%85%7F%EFm%04%81_gR%26%F2%8B4%82D%23%E8%F0%FC%FC%FC%F4%9F%C2%B7X%23%E8L%84%E0%E4%C9%0B%3F%3Ey%F2%C7%3F%BE%B0%F1%8E%1Dk%D6%EF%D8%9A%AC%88u%C6%A4%04F%0B%8D%87%E4%3E%DC%CE%EC%13%C5%5D%0D%87w%1BmKE%10%FAP%9Dz%C4Q%0C%CC%E5T%DFC%E9%08%1El%21H%AF%D2%F0%DAQ%B0nYu%1EA%21F%F0%14%06%ED%C2%F9%3B%CFS%B7%F0%C2%1E%C6%7B%F2%E4c%17b/F%F0%F8q%E1%E1%3D%0F%EEy%02%18%04%A3%11%F4%25%17*%E2m%EF%D1%08%BA%1C%82%17/%5E/%82%10%05%0B%C5L%A38%B1y%A6%C8%0Daw%1D%94%C1K%CA%CD%D3o%21%5B%C1AP%CEZ2%8A%95%EF%23%95%AAr%84%E0%11%06A%17%104%DC%9A%21%8E%8C%8C%8E%8F%8F%1F%1A%1F%8D%11tj%7D%13%18A%E3%81u%13%7D%3E%3FS%805%BB%E0v%95r%F8%96%25%10%1C%20%1AA%12%5E%5B%DBl%5E%92%B0F%D0%26%0A%C1%1F%FF%18%FE%FE%F8%C7%E7%7E%F2%F9%AD%5B1%81%EC%C15%C9b%2C%89%60cZ%BE%9Btu%8C%E7%1F@%CF%CF%F2%EE%AEQ%10%F8%83%28hwF%10%08%C4%CB%E0J%25%8Fq%B7%11%3C%98%8A%60+%0A%D6%F1%FB%25%0C%D2c%BB%17%96O%11%C4%96%DF%7E%FB%9B%CB%00%D9%85%F3%E7/%D0%08%B2%5E%CC%20%8D%E0k%F0e%CF%DE%07%8F%0B%7B%1F%24%A4%D0%08%22%F7%B2+m%7B-%1E%16%BC%EC%B0%08F%F8%A5%08Dq%D6_%20%15q%7E%0E%07An%7E%AE%7B%14%04%04K%27%24Y7%B2n%B9%AC%20E%87%1E2%F3%26%E0%DCWh%82%F70%08%AA%8EY%1BT%C4%DB%B6%8F%90e%D4c1%82%D0%C0%99%9E%C8%BB9%27%3F1%AF%A8%3E7w%82%A5B%90%2C%97%14I%92%3D%3CJ%CD%21%885%82%04%E1y%B81%13%0B%08k%04%8D0%08%12%02%BD%13%3F%C1%12%3A%9BNx%CC%14%E0%0A%08%3A%08%E5w%1D%BE%FB%D6%9F%FE%14%B73ek%3E%9F%C0%28%1E%A2J%89%82%98%3F%89%D4%C4%A9%08%86%04%9AA%BD%5E%F2%E8ffw%04%C3%28X%0F%DF0f%B0%19K%BFF%08%9E%3Au%EA%ED%1F%928x2%05%C1%C8%7B%F2%24%8B%E0%F1%E3%C7%F7%EE%85%BA%B8%B0%27%11%05e%E7%B2BU%C3%10%05M7%D9%16%EC%A0Q%C6Y%03j%E2b%FF%96%EA%ED%9B7%DF%5E%E4%7C%DD%BB%238%0A%FA%02%B2%0C%AB%BC%1A*d%07%DA%83%09%95%AA%8DO%3F%FD%7F%3C%F8%BB_%FF%D2%97%9E%FB%C6%A3%C7c%CD%24%82%20%5E%7C%A5%88%E2%BB%DB%F1%D6%0Ec4%82%BA%1F%EC%84%FE%F9%C4%CC._v%03%8E%C1%16%825%3C3%E3%5B%3C%82%B9Y%AC%11%04%91%B491q%7BC%C7%1AA%87%A2%C5%B1%80%E09B%60sK%F9%CA%D5%ABW%CEl%99%A3%18%ACk%9A%C8j%F5K%1A%B5%A9%0Ep%22%A0%FCN%DC%CE%BC%0B%B73%C5%DC%14%8B%A0%B3rE%8C%C21%99%24%82N%9B@%23%17%B8fP%A2%F6%E9%28M%DA4%81%D0%1DQ%D9%288M%1A%82%84@%82%20%D5D%7D%EC%20%86%EC%EDo%9E%8A%EA%DA%3D%8FQ%DE%93%BC%F7d%EC%C5%151p%F7%A3%C2%1E%E1%204%D5%7E%D4%9F%A1%11%3Co9%9793%D5.%B3%23%5D%11%14%A1%26.%16%FA%F3E%08%82%7C%A2Lw%04%B3%D9l%C9%17%24%A4%20%82%A0%CB%21HV%08o%FC%FD%A7%9E9p%E0%C0%C3%8F%3C%F2%EA%AB%C7%E3%D2%80%20%AE%89%83@U%90%F8%C7%23%23%E3L%8F%18%18%2C%D7%02%D325%DF%F5%F5%A0%EEss%B5%12%5E%95%A8%A9%86a%D6%20%F2rw%3B%C0%1AA%92%D4%B8%FD%F6%B9%B9%BCb.4j%87%A2%01hh%0Bb%04%BDf%A1%F0%7C%EELia%A1P%98%A3%AE%5B%7B%C5b%5C%A3%D2%1Bf%10%8C%F2S%87/A%3B%F3%B0%28AHc%C4P%09%A1Q%18%E4%28%C3em%3C%84%0A%F8%D9%A9QP@%9EM%A4%0BT%D3%80%EBR%8A%DF%D6dc%92%26%F0%A0%D7%A0gG%20%0AN%B3%15q%9Ej%1E%3C%B6%E78%C48%02%D9%9B%A7%7E%C8A%06%DES%9C%F7%02%8B%60%26%D3_%28%EC%7Db%16%EFm%C2%22%E8%E9%26%16%1E%25%AF%5B%151%8B%60W%8D2%DE%1A%19%A8%893%03%CC%98f%8B%CE%24%82%94%04%1BR%14e%C3%89%97%25%A4%CBJ%B9lX%0E%B4%08%13*U%80%E0%B3@%60*%82j%10%D4%014QT7m%DF%CE%BF+g%10Ok%3AB%E0%DBu%96%C1%16%82Y%DCIs%ED%04%82%B9%5C%B5y%B8%AF17%B7%B8%B8X%CC-4%CF%1C%3A%14%F5%F2*%93%A4%D7%E7%CD%15%0A3s%BB%E4%B9f%E1G%05%EA%BAq%00%22v%CF%96%10%C1%C3%3B%05%99%B43Ey%DDB%A2-%18G%C1%24%82xL%10%91ZXH%AD%88%15%1C%05%E1%A2%E4%DC%00j%E2x%7Cv%B2%D1%F0h%02%1F%60%26%E8%3C%7B%20D%90%18%AE9%AA%D4%0E%21/%9C%5C%86Z%F6%9B%A7p%9C%3B%F56A%90%1A%1A%7B%E1%E4%9B%9C%F7B%EC%C5%13t%04%C1%E2%C1%C5%C5%10Aj%82%0E%DEk%B7%28%B8%82FY%C22%FD%C5b%7FF4%FAx%87%94%1C%94%89%3F%3CA%B0t%1AZ7K%3A%3C%F4V%DDTt%93%91S%C1ZR%1B%7F%FF%89g%1F%DF%FF%BD%3F%FA%A3WS%10Tk%7E%5D%97vK%81%FC%E4H%F2m%B5%E6%80%5D%DFVU%9Fj%A4%12%D98%A8%89-%D3P%14%5DN%C8%C8%BA%8CF%10%10%18%CF%80H%15%A2%A1%EE%91%5D%B3%A6/%91/%5Et%C34%8E%3E2g%C6%21%D8h%EC%82%C3@3%F3vK%7F%7E.%D1%23F%ED%18%98%12%05Q+%0A%A6%A6%29%B4%DA%82%10%FF%DC%AC%AA%D2%04%E2%09%BA%C6%E4Q0%9C%9F%F7%82%B70%3B@%0DM%8B%9E%9B%9F%A1%B5%B56o%AERz%09G%09%82%CB%FF%EE%9B%D0%E58%F56%AE%87/Pg%3Ez%F2%14%EB%3DIy5q%1F%9F%A6@%ED%DD%E0e%EB%26gu%95%D1%B2%E9%A2Q%96%B4%7E%5C%15%A7l%E0%28%A2dw%24%CEi%21i%0A%F6%93%15%BC%E0%DE2%5D%07%9A1%A8%C2%D4%E4XKj%E3%D3%CF%FE%E0_%FF%EB%FF%F5%7F%FB7%FF%E6%DF%FE%5B%0EA%B7%EEk%8A%8C%F6%ED%A3%25%B7%22%FC%80%B2%ACa%E0%A1%1E%D3%B7%15z%96Nk%DFerS%89%CA%06%BB%9F%5B%A8%114%7Bxv%A1%99%87%7E%CE-%D4a%25%3C%03%60%03%82%F0t%CF%E5%A1%81%03Q0%3A%B9%96j%1CF%F9%85%9D%E2%03%13P%C9%DB%06%3F.%18G%C14%04%B3d%E6%19%3FXz%87%EE%C8%FF%0E%0Cb%85J%97%ED%11O%CE%0E%0C%94%26%DBi%0A%F0%0D%3DA%27y%87%81%C0X.%EC%F6%89%FC%16JP%CA%25%08%82%BD%F9%CD%7FG%C2%DC%9E%0B%07%05%CA%7B%8A%F5%9E%A4%BC%D0%2CyQW%8C%181%888%14%82%E6%20%256s%AA%E2%F9%95J%9DA%B0%8BFY%D2%5CR%13%27s%97%00%C1%E3%CB%8C%1D%A7%10loug%12%06u%5B1-%B5%C6%E9%95%A8Z%27%04%0D%D7%ADh%B8%0F%8D%AD%C2%3D%23%22%21P1%0D3gX%96%ACx%96L%0D%E8%98%06%9F%B2%9Ae%DB%B0X%23%C8%BB%E8%A9%F0%8F%C8E%99%DC%B1%91%B7%B8X%5D%24%7B%15A%FC%F7%98%AB%83%E4%AC%89%03%B4%0B%97%DCb%C7%07%C2%CB%7F%29%BF%D07%07%B5%FC%DC%E1%85%BC%212%93%27%ED%20%28%8A%29%08r%96%86%A0%24a%06M%BB%E41%9B%DBz8p%97%0C%5B%B7%A1iU%22Q%3C%9E%1D%11U%7E%86%B8%10P%A24%0At%89%C3%5E%EF%9BaKp%0F%9D%95%AA%5C8%C5z%DF%A4%BCJJ%CA%AA%1F%5D%E8J9%E4%D2q%DA%02%84%D0%5Cb%F2%80%3Bk%94%A5Y%7F%06j%FA%94%9FK%15%3E%AB%96%DA%0EN%242%D8%96R6C%7D8%1D%A9%3Ew%26%11%E9%1B%1F%FC%C3%27%9E%FD%FE%81/Ck%F0%F8%ABt%14%0C%02%23%DBn%AA%F2%0Aka%10%C4u-%CE%B2%83%86%BCOo%E7%92%94%F0%A4%EAC%CBP%B5P%5DE%06tC%C52%3A%0E%92%A3%D7%E8%F5%06%B5%EBM7o%A56%97%F2%8D%85%E9%F9%E9%85%B9%81%92M%0B%B4t%A5%2C%91%16%ED%A6%FF*%F4I%9C%92%C7N%D0%89%F8%E2%12%85%26%D2%00%C1%DFP%A7%15%11%CE%26%B5%B3d%5E%1B%27%84%D0z5%A2%7E%E1%ED6da%8F%F7%D4%B9%B8%9A%06%EF%29%D6%FB%26%ED5%935B%AC%08%A6%FB%C9%CD6%7DvC%8A%CE%1Ae%29%86%FA%29%BD%7B%FA%20%A1Hg%5B%1B%CB%E2%05%081*%B2%5D%C1%B7B%92%8DRYI%88%F5%88%7B%A2%AE%0CX%FC%94L%BA%26%A5%EE%E6%A5i%E6DBVb%BA%A2%8E%E1%C0%A3%97S8%C1%27%C92rx%AEF%AD%C9%8A%EA%D7T%5C%13%24j%01%11%C9%ED%91%3F%F9%86%B4%91%C8%D62%C6@5%9F%AFV%F7%BA%06%EE%09%5D/eD%A2%A9u%CA%84F%10U%D6%83Z%98/%CB%5D%18%FE%16H%B4%7C%20s%BD%90y%FE%B1%3D%7B%96%0F%1E%24u%18%F4e%F6%3C%E6%BB%3A%E5%BDp%92%B1s%94W%10%E1%A3B%9C%CB%11%C3%C3E6%95S%27Q%5B%FD%B5%2C%29k%D7I%A3%EC%86%AC%BB%B2Vx%16%D9%A9%91%5C%1Fu%C5Fgl%01%F3%F8%DC%8C%0Abx%E9%13%7CJ%BAY%AF%91%27%14%C2%21y4K%AAb%A5%94%BEy%91%AC%B6l%9D%C5%9F%1D%9E%C7%B0%F1d@%DB%27%E5%BAw%3D%25%3E%28%D6%8Eu%3C%E7%93%DD2%E6%A0J%C0%D7c%9Ac%5D%9F%B7%FD%86%E3%0E%16%23%DEF%29%A8%AF%A0y%D7%B3%9E%F5%ACg%3D%EBY%CFz%D6%B3%9E%F5%ACg%3D%EBY%CFz%D6%B3%9E%F5%ACg%3D%C3%26%97%3C%AF%24%FF%9A%96%B5%B7m%DB%C6%2C%A5u%F3%AC%CD%FE%FA%0F%01%D7%93%D3w%D7%23k%94r%A0%E4%2Ca%D7M%3AV%9C%8CN%DA%AAU7%5C%E4%93%97%95%BD%BA%E9h%5E%27%16%BA%82%B2R%D9@%D3%02%EB%E6%CA%86%96%5D%3E%7E%FC%F82%A5%E64%9B%DF5E/%15%9E%D9%FB%07%7B%98%12*%11%E2%C2%D2j%AA%C6+%CBt%D3%EC%F2%94I%FC%C7%F0L%CF%F4M%DF%20%13%A9%9Bn%A8%2C%94%9E%C4%A5%F9%B2%9Eds%A6r%BB%921%E6%0C%92%E3%0F%26%F7%EBLIt%E5%7Ed%E1%84%DAbq%16/%07%16P%F1%C6%83%C3%9A57%5C%E4%93%95%15U%BC%DB%8Db%3Aj%85%A8*%25%7F%A33%28%D7QVSd%CB%D4n%B4%2C%95u%B7%7C%FC%7B%3Fx%FC%07%8F%2C%C7%18%E7%A7wM%ED%9A%9E%89lz%EF%FE%3F%60%CFI%16%CF%E2%FD%FF%B4%20%B9%DC%B0%F3%0Fp%DA%8Bd%D9%8Aa%18%0A%CEe%D9%F8%1C%D8%C6%EB/+%84%F3%C8%06%9E%8F%E4%CAj%C9%28%D8y%DA%5B%D5LE%AD%AB%BA%AD8ZR%BD%05%27%FF%B4%A7%00%F1k%EEm%CD%16%27f%A6%E7%A7g%9A%C5BC%92%8A%85%C4%A5%B7T%1C%15%D4%94%B0%10%D65%F6%9A5%E4y%B7%B6%09%D7_t%E5%B2%24%16%A5%3F%0F%AAf8e%C5%B6%0DU%AB%19N%89%5B%9E%D7%1D%B2%15%CA%BA%F0%20+x%AE%17%E7%98%B8%89%B2%26.%9B5%A1%AC%C9%97%A5%9E%FE%3F%3C%B0%FF%D9%A7%9E%FD%C1%F18%0C%E6%01%C0%29@p%01%A7D-%24%10t%BBJ%13%D2z_%B2%CC%EA%7DM%82%AF%95%11%18%22%B8u%EB%AB%2C%82%DD%CB%82/%2C%8E%11%E4%CB%E2%9C%8C%EF%DAV%A4%95%D39o_3%87%EDA%A4%ABu%9C%E6%C3%EF%9AH%10%8C%A7%A1%13%08%E6%E7%E6%E7%01Al%D3s%C5%E2D%22%0A%AAAV%12E%29%9B%22%D9h%ED%18%82%AB%BC%EA%8B_%1C%02%98%86v%24%A4%C9%A2%A2%29O%CFJe%3B%C7%22A%F0%0D%B7%ECU%5C%C5t5%C3p%EA%DC%3A%F9%EE%90u/%EB%9A%C8r%E1r%E3%04%13x%95%2C%5B%F1*NzYS%88R%93%1F%7Bu%FF%13%C2S%8FS%92%5E%80%E0%F3%80%E0%82%BA%00%06%0Cr%08j%82FB%12%F9_%D1%84%24%82r%ACz%84%D8H%06%08%92%7D%86sF%18%05_M%20%28%CB%BE%0F%FF%F0%97DY%FCC%C5%A8%E1%EDgQ%A2%2C%B2V%8D%9D%3D%3B%B6%CA%22i%BC%8A%D4%11%C1%C1%EC%B0%AD%D4*%95Z%60%9Bu%D3%CE%B2%0Cb%04%8Bsm%02%F1zc%06A%BB8%83%17%15%14%8AM@p%7E%BA9U%E4%B2%A2%02W%BCw%09n%8A%28%3A%C9d%98%A15k%86%96_z%E9%9E%F5%AB%F0+%DE%8B%8BJ%9D%8Av+%DB%3D%16%09%A2%A7%FA%AEY%3F%B1%7Dd%03%5E%B2%ABqyb%5D%21k%95%F5USI+%AB%21%DD%1Dtm%D9%D2%95%25%E0P%E3%CA%BA%5D%CA%9AB%94p%FA%C2%9B%DF%FB%FE%13%10%05%E3%E7*%BF%8B%20%08%04%E2%28%A8%A6%20h%920%83%FFW4%B1%CE%3C%91%B1raR%E9%26%19%05_%7D%F5%9ET%04%A1%20%87%E0d%CB%97%25%C5%01A%BE%2Cz%EB%EC%D97%DE%18%1B%DB%D4J%22%EB%84%A0cZv%D6V%83%C1%C0Q%07_%A8%0C%EA%A6%C3%BE%FF%824%D7%7C%3E%24p%A1%B9%28%B1%08%CE.B30/Hv%1E%20%9C%99%99%E7%11T%1D%11Y%C3/%A2a%F8xn%22%0E%DAk%1E%FE%BD%AF%7C%EB%B7%9F%FE%DD%3B%D6%EC_%C3k%A9v/%DA*%FB%FBieq%2CR%E3X%C4%C5P%DD%F7%80%02G%5D%3D%3A%B6%9A%AC%22e%17%CAw%85%AC%5DV%F3%BDJZYM%AF%EB%28%5B%B7extd%DD%60%94%97%AD%EEe%CD8%7B%EC+%CF%3E%BE%FF%F1%FD%DF%5B%8EK%E7w%B6%10%C4%10b%04O%7E%FB%3B%D4%A5%D0%A0*%C67%99%EC%7B%A9%19%82%CA%21%08%A4%C4+K%B8H%065%E9D%A3%B1n%5D%1C%05Y%04Cx%B3%7B%F6%EC%C9r%F8*%A1%CF%DAv%F0%E0AR%11%F3e%A5U@%E0-s%8B%27%C2%28%28vBP%93%E1%3A%07%AA%0Dm%94%3A%D6x1%99%8C%F1p%A1%D0%C2%F4%14%26%B0oza%F7%BD%BB%19%04%8B%85%C5%7C%91%7C%60%BDZ%9C%9E%9F%B8%9BE%D0%D2Dy8%ABd%8F%90T%D7d7q%E8%F7%BEu%DF%D3%FF%EA%A9%27%BE%FE%E5D%10%84%A2G%AC.E%A1%EC%D3%A4%ECS%89%B2%1A%CE%0C%25%C9%C5%CA%BED%2C%12J%AA%A9%19%8A%B9illt%E4%84%E7W4%8D%0Et%DD%21k%95U%B5%BA%A9y%89%B2%96%A6%D5uI%86%C6%CC%A0%AB%EBv%D6P%99%B2u%B3%D6.%EB%27%CA%0A%E6o%C4%09%8C_%7F%F6%C0%E3%07%8ES%8D%88%15%10%AC%8BHFJM%96%CD%C9I%03%0B%FB0O%1D%D1%EC%9A8%FC%3F%29%A4%273%C1E2%19%E9%13%AE%E98-%04%13Q%10%CA%C2%D3%BC%E7%D4%A9S%7B%12%11%94%F8%AC%3D%07%0F%BA.i%0B%F2%08%7Ewd%EC%F5%E5%B93G%A5p%23%CF%8E%08.%D5%D5J%00%0CB%A7Yu%B5zv%89yv%09%82%12%21Pz%B9o%F7%EE%DD%EC%82%88bq%9D%9B%0F%F10%8A%F33%13%3B%8B%AF%D0%07W%B3%24%B56%FB%A2b%EFC%BB%B3%89X%B6%FC%95%FB%EE%FB%D6%83O%3D%F3%EC3%FF3%3F%BA%82%8B*%5D%8A%0A%1B%3A%95%155%0C-%CE%D6M%89E%82%E5%E1dmc%C3%F6%D1%D1%B1%D1%D5j%1DX%A0Q%E8%0AYX6%A7B%AD%E5%D4%3D%97/%1B%D8%C8%94q%EC%85%AF%01%1C@1%B4%C4y%9D%0Ee%05%93N%0B%FE_%BE%FE%D4%83t%0F%8BBPM%22%E8%A8%22B%AAw%CCB%7EP%C3%7B%B3+%CC%D5%C2%9A%5D%F2%C4%03%BF%F3M%00p%DD%02%1F%05%5D%0AA%B1C%14%F4%5B%08%FA%29Q%D0o%21h%27%A3%A0%80%CC%AF%9D%3E%FDZ%19Ip%23%F9E%8E%0C%82%AAZ%A9%D7%5D%13%AF%D7qM%5B_%D2%24%E6%FD%B7%9A%81%F7%BE%0C%F8%25%D6%E4%14%D7Y%AF%84%19%BEzaz%EA%EE%9D%B7%16%98q%C1@%92m%9C%CE%7D%C4%B2%E1%DEH%5C%93.%3Bt%CF%B7%9E%FE%D6%D3%0F%E2%95%EE_%DE%B1%8A%EDR%04%D2%8B%5D%8A%F2e%E9%9A%D8%CD%21%11%10%B4%86%25Y%85g%CBd5Z+%AA%01m%1F%17%82%E0%FFx%DB%D8%C8%09%9CW%ED%C7%21%B6%3B%28%A4l%AE%BFj8%AE%AAU*%7CY%B2%AEO%D6%E5%25%F8%D4%9Ae%D7M%1A%C1J%9D%9CW%25eK%7C%D9%16%82%BB%C5%87%1E%BA%17%DE%FCC%F0%9A%1E%20%CB%EF%9C%8A%DB%82%B8%3B%C2%20%88%9B%2C%10%F8%01A%84%02%0F%D9%5C%AB%8BhvM%CC%1C%C6%046%1A%89%28%28%E8%13%EB%D6%B9%18AYL%8D%82%D8%3AEA%BC%CE%15%23xgZ%14%14%A4%25%A8%8A%E4%25%94%22oB%5BM%CE%AA/%D4%5DGu%E1_%D6%B6_%94k2%B3%28%0C%97%DEM%2CeYX%F4%9B%A8X%28%92%B64%83%A0%26%0E%03FpP%A4%28%C3%88%8D%C4%AB%D6%AC%F9%E2K%BF%FD%BB%0F%FE%AB%AF%3F%81%EB%9D%1D_%5C%B3f%D5u%16M-%1B%DDO%8DH%20@4%B2%82%2C%F4%8A%07YeE%CF%C4%0B%BE%8EB%10%14%84%B1%B1%D5u%1C%EE%E2%05--@%D5t%C8p%D9z%7F%7F%A6NFl%3D%95-%1B%D8%00%FE%92e%E3%85%5C%8A%EB%EAF%A0iT%D9%2C%1C%18%0E%EB%98%98%22%BE%2CF%90K%81g%10%BCk%AA%DD%23%0E%C7%057%8D%1F%8A%BD%81%29%B5%11%D4%BD%3AV%C5g%B6%C4%26k%F3%A0%BD%076%3B%DB%98%90%D8H%06e%C0Az%C4%18%C1%E7%B6n%3D%BE%91+%8B%E5%98%A0-hs%EAQJ%E8%93%E3%B6%20_%96ZU%D2%0D%C1%B2%29%E1%E5W*%5C%1B%B7b%DA%C3%92%A9%E9%0C%82%98%BD6%7E%C9%28%18%F6%E1%8Az%FB%02r%DA/%22%AEL%15%0B%ED%C3kv%92%08%DE%F1%AD%27%9Ez%EA%89g%0E%3C%FE%F8%C3I%04%5BEmUS%B3%29%08%B6%CA%3E%BAG%DB3%F4%C55T%10%D5%A0%89%3Cl%9BAP%CF%92%5Eq%C0%F4%8Ak%1A%F0%13%8C@%10%14E%08%83%9BH%B8%8BH%C1%90%A9n%3DH%87%0C%CA%0E%F47%E72%E12%BA%8A%C9%96%C5%3BQX%164%AAux%DF%C6%20%D4%C7%FB%A8%21%AE%9A%8B%F7Lh%A9%5B%D4%5Be%A9%E6%5Ew%04%C3%28%18%8E%0B%26%10%D4%94v%14%D4%3D%15%3FCR%89*%DBB0%D4%1D/%95X%04%27%09%82%B3%B3%81%EA%B4%A2%60%02A%E8%C1LN%92%8E%8C%EF%F3eC%1F%F4f%1C%D2%23%E6%CB%B2%1F%A8%23%82%7E%25+%0D%03%82%95J%C9%D7%B2%96%94%AD%24%10%14%C5%A9%90%BE%9D%09%04%0B%3Bo%BD%F5%D6%9D%8B%A4%13%0D%15%08%CA%B1%AAH%81%24ADP%D0%3E0%B4%8F%AFM%EDU%EB%7F%F7%89g%9Ey%F6%C0%81%FD%8F%3FB%C6%F9R%8AZ*PX%B7%F8%8A%B8%5D%F6Q%D2%05%DDC%AF%22vU%3B%AB%B0%BDb%AA.Fd%D4%7E5%04%C1c%C7%84%D1%B1%ED%B7%90p%17CV%07@%83p%84%B7%CCC%06e3%D3%F3%9B3d%A8%1FjVW%A5%CB%06%86Q%0F%CC%F0%A1%07%97%2C%91%FA8*K/%1B%85%B2%0Es%DE%95%11%24C%D3%D1%EC%08%83%20%DC%C1%16%82%93%25%B2%0FL%1A%82%91%A5F%C1u%9D%A3%20td%BC%C9%99%99%C9%19%1EA%25%F4y%0B%EBJ%10%BF%B2+DA%B1%0B%82%3E%1EM%B2l%B3%5Ew%86%112k%EC%8C%80%86%C7%0A%16%A6w%11%02%A7%0E%27%10%FC%DC%CC%CC%AD%CDP%88F%CE%E7%D6%15Y%04Ux%3E%AD%23%C3H%DA%B7%EF%5E%24%26%FB%14%AB%EEy%EA%C0%81%03%8F%EF%DF%BF%7FM%A2%3B%82%8B%EA%10%86%86I%1F39%EB%0Ae%1F%3D%A8%1D%B5%B1%9B%99%08%21%C3%82T%AF%18%25%7B%C5OB%10%14%26%8F%09%9F%19%1F%D9D%C2%1D%0BhgP%84%FE%E6%ADE%CF%C4%0B%DD%EBA%D0_e%CA%06%83%B6%3E%D8%9E%A0PH%7D%CC%8Eh%0E%7D%F6%E9%7B%EE%80%C3%AAQ%7D%7C%FD%08%CEwF%B0%AC%93%0AQE%D2%A4%E7%07u%09%29%1C%82NX%9B.%25%DBd%93%E0%8B%97%FC%A5G%C1%A8%23%93%28%1B%FA%16%D6A%15%1A%CE%8Epe9%EB%84%A0%E9%FB5%CD%B4%D0%12%FC%C1%3AV%9E%EB%D0%DB%94iBA%2C%CE5%C3%2883W%14%0B%3C%82%F3%B7.%E6%C9%FD%D7%8B%8Db%21%CF%1C%DC%1A%14%C9*K%08%90%FB%A0S%9E%1C%94Y%F3%E8%BF%3C%B0%7F%FF%E3%FB%BF%98%1C%94%C1EU%7D_%F8%7C0uZT%F6%20%DD%05e%9C+%F4%8Aq%10%14%26%27%05%08%83%AB%A1%D3%5B%A2%DBN%B9%89%CFMq%F51%E5%D5%FB3%B3%B5%1A%B0%17%04%7B%A1U%C8%96%1D%B4%EDA%AE%3E%A6/%D6%AA5O%7Cc%07%D8%9AlT%1FS%F7%81%13T%E2%10%DC%B5kjj%3E%B6%91C%D4%5E%15%BE%DC%12q%10%A1%F3g%D6e%C7e%FA%DAD%B3+%9E%1D%E1%23%99@fG%1C%13.%22%20%F8%85%AD%5B%878%04%A3%8E%8C%94h%0B%12%DFB%03%9E%29%DC%23%E6%CA%262%5C%3A%E5%B7H%81%EF%97k%15%BC%83G%AD%82W%DF%964%FA%E1%24%13t%C5v%3B%B0%C8O%D0%15%3E%07%8F%E7%AD%CDB%23g+U%2C%8F%C1%8D%03%BB%AE%20%ED%BBO%B8O%5C%92%0471%CF%96%5D%B3%7F%CD%FA/%1Fx%F8%915%F0%82%9FcsU%D2%C7%DC%0DQ%0E%B1uZT%B6%DD%05%DD%87%DD%CC%5C%5C%DC+%86%EA%5C%CF%1Au%E6%DC%B8%3B%0C%ED%3E%F8%98%10%06%BF%E6y5%BAlqj%E7%FC%00i0%B6%EAc%06P%11%7E%D5%06%F6%E0B%F57o%87/%F4%CEe%D9%C1A%05%EAc%E8%C9%9B%A4J6%0C%97z2%B6%C1%F3%F6%F5%07%7F%FB+%3B%1E%8B%C2%2C%F5%91x%04%25%06%C1%29%DC%99%C87%DAV%CD+%B4%BE%20%B5%C6%17%9Ev%D35%7Dv%5C%8D%12%7EMF%B2D%14%E4%11%8C%3B2%C9%28H%7CA%10%22%C8%95m%DF%8A%C1.%8B%F2%C3k%8A%1C%1Fo%3C%11%AB%5Bh.%8F%60%9B%C0%DF%FA%AD%04%82%F7O%ED%DC5%3D5%DF%9C%5B%2C%14%17%9B3E%7E6%2CpEq7Y%E3%EC%0E%26%CE%8D%A7%D6V%EDX%B3cG%87%09%3AW%A7%FA%98%DC%2C%1D%94%D8%C3tAY%AD%D4v%AF%18B%A0%C2G%23%9D%04%C1%D0%20%0C%B2%1A%14%B9%E6%E7%E6g%26%E8u%F64dRuVx%00z%24%FD%82%B0ez%7E%A6%3Fy%3DU%0D%D7%C5%83zX%25S%ED%03%1B%07%C0%1D%9F%7Dp%FD6%C1Z%FF%95%3B%F8%C4%8A%EE%086r%B4%80U%3E%9F%CFR%17Z%F5yc%94%5D%08%82Q%0C%94%1EJ%B4%05%C3%28%18%22%F8%D5%24%82qG%26Q%96%F8%F6Bo%86%20%F8%D5%14%04%91%AB9%E5%9A%D0%DD%A0%C9%85%3F%03%91%1D%F0%E1%C9%AF%D5%12Q%900x%DFo%08B%98%29C%7D%FA%C2T%B1%D8l%D7%10%D33%C5%24%F0%AE%86%9B%A2%92%92%F2%2C%D8%3B%86%20%F4%0D%7D%F1%8B%AB%F08%DF%8ED5%ADv%AE%D3H%D9.%EE%B0WL6%EE%CB%26%10%0C%83%20%B4%82%E1%BF%DF%84%D6%20s%D2%E2%D4%DD%D3%9B%A7%E1%DF%CC%E7%EE%9E%9E%282%3BQ%09r%7F%7F%BF%D0%0F%3D%92-%80%E0%DC%CEbJ%0E%A6F%DA%9Da%95%9C%D4t%1C%FA%D2g7%08%E2%D0%97%FF%C5%3Dw%ACa%07%0FV@%D0%A0%C5%87%1B%F9%3C%DDe%84.%25k%EC%BC6%1B%05%D9%C6%FC%F5D%C1%B8%23%93%28%1B%FE%1C%F7%88%15%5BN%8D%829%B3%5C.%D7Wd%10%AB%A4%D8x%26B%D7eI%0Cj%01%83%60%F2%22%D3%08%16%0D%BDQ%9D%9B%C1%04%CEL%14%1A%29%22%16%96%3A%D8%29Y%8B%FC%2C%BB%26%AC%83%EDd%C2U%A0%00%5C%D9v%1F3%E0%CB%D6%0D%CA%CD%06I%D2+V%14%BB%D5%1Fc*%E2%EC%F60%08z%93%F8%FF%D1%F1%D5tw%3A%D7%BC%15%3E%C8%5D%F0%EFnxq%3B%1B%D5%0D%5C%F7B%8F%E4s%C5%1C%7C.h%15%B2oW%C1%F5p%98%3C%9A%8D_%D2%96%FD%EC%3D%10%ECW%ED%F8%FC%1D%FF%C3z%AE%E1a%F2%DA%184%82%B3%F9%D9%85%1Ce%03y%9Bzc%A2%DF%D2%B5%0A-%08%D8%04%08F%01%FB%5E%16%23%25F0%8Buwp%7Bn%EB%C6%EB/%8B+%1BR%1E%23%C8%95%C5%17%0C%3A%16%AA%A4%AE%CC%A0%D0%92%C5iI%828%15%EA%EA%04%29Z%99%F1%07%14%0B9%0BZZj%B5%80%07%A6%AB%8E%7Ec%12A%A1uN%3B%95%E9%B3%27%93%FFPg%B7%C8%A5%D0%D1%E92%23%23c%9F%89%BF%BBm%7C-%AD%ADY%9C%9A%9A%9E%98%9E%82%20x%EB%14%10%C8%06%C1-3%D3P%F7%DA%09%F6Z%86%3B%C4%F8k%88%9E%9A%92%9E6t%C7%90H%DAk%EB%3F%7B%27%E72%13F%7F%9CY.q%9FAP%A8yD%FD%ADm%9E%CF%21%C8%19%13%C9%B0%3C%14%C9X%0D%11%84@6t%9D%BD%DA%C9%F6%5D%C2%19%AFa%14d%CA%0A%22%3C%F9e%15%C0R%CBA%CD%5C%11%8CnZ%3B%16%19Qm%C9%86%B1%90%E5p%3B%CC%C6%0Do%C7%B0%F5%1B%14%A9j%D9%AF%3Aq%7F%F5%C8%28%D8X%CB%C6%C7%D7%D2%0A%D7%CCnn%1C%81B%FF%C4%AD%F9.mkR%FB%0A%ED%BAX%B3%13%F7ox%FDPX%FBnX%7F%83%EF%5B%94lV%C2%8A%A9%88%B1%B2%9Em%989%07K%22%EA%BC%F8Y7%BD/3%CE%BC%C7%B9%F7%A5%15%13%F7%13e%E3%BC%7F%BE%AC%28%E5%CA@%20V%A2S%CBe%F3%13H%1E%B5%95%95%DA%1D%FFD%FA%1D%91%DC%25%9B%7F%FC%FA%AF%A9%C1%B6j5g%B7%5Co%C9%BE%FE%CC@%177%A9%7D%85.Qph%CDpx%C9%D6%0C%DD%F0%A5%EA.pF%EB%E0%25ou7%BD%AFP%24%2C%29%DAw%7De%E5p%3F%ED%0E%A5%89*%17I%92%E9%29n%F5%ACg%3D%EBY%CFz%D6%B3%9E%F5%ACg%3D%EBY%CFz%D6%B3%9E%F5%ACg%3D%EBY%CFz%D6%B3%9E%F5%ACg%3D%EB%D9%3Fg%FB%FF%01%AA%CE%F1p%87wm%08%00%00%00%00IEND%AEB%60%82'

  escaped_pixel_gif = 'GIF89a%01%00%01%00%91%FF%00%FF%FF%FF%00%00%00%FF%FF%FF%00%00%00%21%FF%0BADOBE%3AIR1.0%02%DE%ED%00%21%F9%04%01%00%00%02%00%2C%00%00%00%00%01%00%01%00%00%02%02T%01%00%3B'
  
# End of Constants.


# Dispatcher (router) function.	

  dispatch = (req, res) ->

    final_dispatch = (req, res) ->
      parsed = urlparse(req.url)
      pathname = parsed.pathname
      pathname = pathname.replace /\/$/, "" if (pathname.split '/') .length > 2

      req.get = if parsed.query? then querystring.parse(parsed.query) else {}
      req.body = _extend {}, req.get
      method = req.method.toLowerCase()
      if dispatch.logging
        dispatch.log "#{dispatch.getRemoteAddress(req)} - [#{new Date().toLocaleString()}] - #{method.toUpperCase()} #{pathname} - HTTP #{req.httpVersion}"

      if dispatch.routes[method]
        selected_method = dispatch.routes[method].concat dispatch.routes['any']
      else
        selected_method = dispatch.routes['any']
      
      for route in selected_method
        m = pathname.match(route.pattern)
        if m isnt null
          if route.params
            req.params = {}
            args = m.slice(1)
            for param, index in route.params
              req.params[param] = args[index]
          return route.handler(req, res)

      if pathname is "/"
        homes = ("#{dispatch.static_route}#{path_tools.sep}#{home_page}" for home_page in dispatch.default_home)
        send_static_home = (candidates) ->
          if candidates.length is 0
            if dispatch.list_dir
              return dispatch.directory dispatch.static_route, '.', res
            else
              return dispatch._404 req, res, pathname
          else
            fs.exists candidates[0], (exists) ->
              if exists
                return dispatch.static "/#{path_tools.basename candidates[0]}", req, res
              else
                send_static_home(candidates.slice(1))

        send_static_home homes

      if pathname isnt "/"
        if dispatch.serve_static
          return dispatch.static pathname, req, res
        else
          return dispatch._404 req, res, pathname


    if dispatch.use_nsr_session
      nsr_sid = dispatch.getCookie(req, 'nsr_sid')['nsr_sid']
      if not nsr_sid
        nsr_sid = uuid()
        dispatch.setCookie(res, {nsr_sid: nsr_sid})

      dispatch.getSession(req)

    final_dispatch req, res

# End of Dispatcher (router) function.	

# Extends default options with client provided ones, and then using that extends dispatcher function itself.

  _extend(default_options, options)
  _extend(dispatch, default_options)

# End of Extends default options with client provided ones, and then using that extends dispatcher function itself.

# Directory listing template	
  dispatch._gallery_template = """
                              <!DOCTYPE  html>
                              <html>
                                <head>
                                  <title>Media Gallery of {{ cwd }}</title>
                                  <link rel="stylesheet" type="text/css" href="{{& css}}" media="screen" />
                                  <link rel="stylesheet" type="text/css" href="{{& theme}}" media="screen" />
                                  <style>
                                    /*
                                    body {
                                      margin-top: 60px;
                                    }
                                    */
                                  </style>
                                </head>
                                <body>
                                    <div class="reveal">
                                      <div class="slides">
                                        {{# files}}
                                        <section data-background="#000000">
                                          <img class="stretch" src="{{& url }}" />
                                        </section>
                                        {{/ files}}
                                      </div>
                                    </div>
                                  <script src="{{& revealjs}}"></script>
                                  <script type="text/javascript">
                                     Reveal.initialize({
                                        controls: true,
                                        progress: true,
                                        keyboard: true,
                                        overview: true,
                                        center: true,
                                        touch: true,
                                        width: 960,
                                        height: 700,
                                        // Factor of the display size that should remain empty around the content
                                        margin: 0.1,
                                        // Bounds for smallest/largest possible scale to apply to content
                                        minScale: 0.2,
                                        maxScale: 1.0
                                     });
                                  </script>
                                </body>
                              </html>
                              """

  dispatch._dirlist_template = """
      <!DOCTYPE  html>
      <html>
        <head>
            <title>Index of {{ cwd }}</title>
            <style type="text/css" media="screen">
              *, *:before, *:after {
                -moz-box-sizing: border-box;
              }
              html {
              padding: 30px 10px;
              font-size: 16px;
              line-height: 1.3;
              color: #737373;
              background: #fafafa;
              -webkit-text-size-adjust: 100%;
              -ms-text-size-adjust: 100%;
              }
              body {
                font-family: "Helvetica Neue",Helvetica,Arial,sans-serif;
              }
              a {
                  color: rgb(66, 139, 202);
                  text-decoration: none;
                }
              a:hover {text-decoration: underline;}
              ul {
                list-style-type: none;
              }
              hr {
                height: 1px;
                color: #eee;
                background-color: #eee;
              }
              a.directory {
                font-weight: bold;
              }
            </style>
        </head>
        <body>
            <h2 style="text-align: center;">Index of <em>{{ cwd }}</em></h2>
            <!--<hr/>-->
            <div style="background: #fff; border: solid 1px; padding: 0.5em; padding-right: 3em; border-radius: 7px; width: 90%; margin: 0 auto;">
              <ul id="dircontents">
                {{^ isRoot }}
                  <li title='Parent directory' style='height: 30px; margin-bottom: 5px; font-weight: bold;'>{{& icondir}}&nbsp;<a class="directory" style='color: #aa0000;' href="{{parent}}">..</a></li>
                {{/ isRoot }}
                {{# cwd_contents }}
                  <li style="height: 30px;">{{& icon}}&nbsp;<a class="{{class}}" href='{{path}}/{{querystring_escaped_file}}'>{{file}}</a><span style="float: right;">{{ size }} bytes</span></li>
                {{/ cwd_contents }}
              </ul>
            </div>
            <!--<hr/>-->
            <p style="text-align: right; width: 90%; margin: 0 auto; margin-top: 0.5em;"><strong>Served by <span style="color: #8f0000;">#{dispatch.served_by}</span> <span style="color: #00008f;">v#{dispatch.version}</span></strong></p>
        </body>
      </html>
      """

# End of Directory listing template	


# Dispatch object methods, not meant to be called/used by the client.	

  _pushRoute = (pattern, callback, method) ->
    params = null
    if typeof pattern is "string"
      parsed = _parsePattern(pattern)
      pattern = new RegExp("^#{parsed.pattern}$")
      params = parsed.params
    dispatch.routes[method].push {pattern: pattern, handler: callback, params: params}
    dispatch.routes[method].sort (it1, it2) -> it2.pattern.toString().length > it1.pattern.toString().length

  _multipartparser = (body, content_type) ->
    resp = "multipart-data": []
    boundary = content_type.split(/;\s+/)[1].split('=')[1].trim()
    parts = body.split("--#{boundary}")
    for part in parts
      if part and part.match(/Content-Disposition:/i)
        #dispatch.log "PART: #{part}"
        obj = {}
        m = part.match(/Content-Disposition:\s+(.+?);/i)
        if m
          obj.contentDisposition = m[1]
        m = part.match(/name="(.+?)"/i)
        if m
          obj.fieldName = m[1]
        m = part.match(/filename="(.+?)"/i)
        if m
          obj.fileName = m[1]
        m = part.match(/Content-Type:\s+(.+?)\s/i)
        if m
          obj.fileType = m[1]
        else
          obj.fileType = 'text/plain'
        m = part.match(/Content-Length:\s+(\d+?)/i)
        if m
          obj.contentLength = m[1]

        m = part.match /\r\n\r\n/
        if m
          obj.fileData = part.slice(m.index + 4, -2)
          obj.fileLen = obj.fileData.length
        
        resp['multipart-data'].push obj
    resp

  _bodyparser = (body) ->
    if body.indexOf('=') isnt -1
      try
        return querystring.parse(body)
      catch e
        dispatch.log e unless not dispatch.logging
    try
      return JSON.parse(body)
    catch e
      dispatch.log e unless not dispatch.logging
    body

  _getSessionHandler = () ->
    try
     handler = eval("(#{dispatch.nsr_session_handler})")
     if handler.constructor.name isnt "Function"
       dispatch.log "Handler is not a function, using default 'memory_store'"
       handler = dispatch.memory_store
    catch e
      dispatch.log "Error: #{e.message}\n Using default 'memory_store'"
      handler = dispatch.memory_store

    handler

# End of Dispatch object methods, not meant to be called/used by the client.	


# Dispatch function properties and methods 	

  dispatch.getRemoteAddress = (req) ->
    ip = req.headers['x-forwarded-for'] or req.connection.remoteAddress or req.socket.remoteAddress or req.connection.socket.remoteAddress

  dispatch.get_route_handler = (path, method) ->
    if not method #search all methods until a match is found or null is returned
      for key, objects of dispatch.routes
        for object in objects
          return {method: key, handler_obj: object} if path.match object.pattern
      return null
    else #search a handler for the given path, but only with the given method
      chosen = dispatch.routes[method]
      return null if not chosen
      for object in chosen
        return {method: method, handler_obj: object} if path.match object.pattern
      return null

  dispatch.get_icon = (x, y) ->
    template = """
               <img src="/pixel.gif" style="width: 30px; height: 30px; vertical-align: middle; background-image: url(/icons.png); background-position: {{x}}px {{y}}px;" />
               """
    offsetY = 8
    offsetX = 5

    context = x: (x * 30 + offsetX) * -1, y: (y * 30 + offsetY) * -1
    return dispatch.render_template(template, context)

  dispatch.stock_icons =
    directory: -> dispatch.get_icon 13, 3
    file: -> dispatch.get_icon 20, 3

  dispatch.getUUID = () -> uuid()

  dispatch.cookie2obj = (cookie_str) ->
    ret = {}
    cookiePairs = (pair.trim() for pair in cookie_str.split(/;/))

    for pair in cookiePairs
      eqIndex = pair.indexOf('=')
      if eqIndex isnt -1
        key = pair.substring(0, eqIndex)
        value = pair.substring(eqIndex + 1)
        ret[key] = value
    ret

  dispatch.obj2cookie  = (obj) ->
    ("#{key}=#{val}" for key, val of obj).join "; "

  dispatch.getCookie = (request, cookie_name) ->
    ret = {}
    return ret unless request.headers.cookie?
    obj = dispatch.cookie2obj(request.headers.cookie)
    return obj unless cookie_name
    if obj[cookie_name]
      ret[cookie_name] = obj[cookie_name]
    ret

  dispatch.setCookie = (response, cookie_obj, max_age) ->
    arr = []
    for key, value of cookie_obj
      arr.push "#{key}=#{value}#{if max_age then ('; max-age=' + max_age) else ''}"
    response.setHeader 'Set-Cookie', arr if arr.length isnt 0
    dispatch.obj2cookie(cookie_obj)

  dispatch.memory_store = (request, opcode = 'get', sessobj = {},  cb = ->) ->
    sid_obj = dispatch.getCookie(request, 'nsr_sid')
    sid = sid_obj.nsr_sid
    return cb(null) if not sid
    switch opcode
      when 'set'
        nsr_sessions[sid] = sessobj
      when 'get'
        if (not sid in nsr_sessions) or (not nsr_sessions[sid])
          nsr_sessions[sid] = sessobj
      else
        return cb(null)
    request.nsr_session = nsr_sessions[sid]
    cb(request.nsr_session)


  dispatch.text_store = (request, opcode = 'get', sessobj = {},  cb = ->) ->
    sid_obj = dispatch.getCookie(request, 'nsr_sid')
    sid = sid_obj.nsr_sid
    return cb(null) if not sid
    #dispatch.log "text_store session handling not implemented yet." if dispatch.logging
    filename = "#{process.cwd()}#{path_tools.sep}sessions#{path_tools.sep}SID_#{sid}"

    set_func = ->
      fs.exists path_tools.dirname(filename), (exists) ->
        if not exists
          fs.mkdir path_tools.dirname(filename), (err) ->
            if err
              return cb(err)
            else
              set_func()
        else
          file = fs.createWriteStream filename, encoding: 'utf8'
          file.write JSON.stringify(sessobj)
          file.end()
          request.nsr_session = sessobj
          return cb(sessobj)

    get_func = ->
      fs.exists filename, (exists) ->
        if not exists
          return set_func()
        else
          fs.readFile filename, encoding: 'utf8', (err, data) ->
            if err
              return cb(err)
            else
              sessobj = JSON.parse(data)
              request.nsr_session = sessobj
              return cb(sessobj)

    switch opcode
      when 'set'
        return set_func()
      when 'get'
        return get_func()
      else
        return cb(null)

  dispatch.getSession = (request, cb = ((id) -> id)) ->
    return null unless dispatch.use_nsr_session
    _getSessionHandler() request, 'get', {}, cb

  dispatch.setSession = (request, sessionObj, cb = ((id) -> id)) ->
    return null unless dispatch.use_nsr_session
    _getSessionHandler() request, 'set', sessionObj, cb

  dispatch.updateSession = (request, sessionObj, cb = ((id) -> id)) ->
    return null unless dispatch.use_nsr_session
    #new_sessionObj = _extend(dispatch.getSession(request), sessionObj)
    #dispatch.setSession(request, new_sessionObj)
    dispatch.getSession request, (curr_session_obj) ->
      if curr_session_obj
         dispatch.setSession request, _extend(curr_session_obj, sessionObj), cb

  dispatch.setSessionHandler = (func_name) ->
    num = parseInt(func_name)
    if not isNaN(num)
      if num < dispatch.avail_nsr_session_handlers.length
        dispatch.nsr_session_handler = dispatch.avail_nsr_session_handlers[num]
    else
      dispatch.nsr_session_handler = func_name
    dispatch.nsr_session_handler

  dispatch.addSessionHandler = (func_name, func) ->
    try
      if func.constructor.name isnt "Function"
        dispatch.log "'#{func_name}' doesn't evaluate to a function."
        return null
      else
        dispatch[func_name] = func
        dispatch.avail_nsr_session_handlers.push "dispatch.#{func_name}"
        return func
    catch e
      dispatch.log "ERROR trying to add session handler '#{func_name}': #{e.message}"
      return null

#

  dispatch.routes =
    get:  []
    post: []
    put:  []
    patch: []
    delete:  []
    any: []

  dispatch.static = (pathname, req, res) ->
    full_path = "#{dispatch.static_route}#{unescape(pathname)}"
    fs.exists full_path, (exists) ->
      if exists
        if ((pathname.indexOf("#{dispatch.cgi_dir}/") isnt - 1) or (pathname.match /\.php$/)) and (pathname.substr(-1) isnt "/") and (dispatch.serve_cgi is true)
          try
            return dispatch.cgi(pathname, req, res)
          catch e
            dispatch.log e.toString() unless not dispatch.logging
            return dispatch._500 null, res, pathname
        else
          fs.stat full_path, (err, stats) ->
            if err
              dispatch.log err.toString() unless not dispatch.logging
              return dispatch._500 null, res, pathname
            if stats
              if stats.isDirectory()
                return dispatch.directory(full_path, pathname, res) unless not dispatch.list_dir
                return dispatch._405(null, res, pathname, "Directory listing not allowed")
              if stats.isFile()
                fd = fs.createReadStream full_path
                res.writeHead 200, {'Content-Type': mime_types[path_tools.extname(full_path)] or 'text/plain'}
                fd.pipe res
      else
        if unescape(pathname).match(/favicon\.ico$/)
          res.writeHead 200, {'Content-Type': mime_types[path_tools.extname('favicon.ico')] or 'application/x-icon'}
          return res.end new Buffer(unescape(escaped_favicon), 'binary')
        if unescape(pathname).match(/icons\.png$/)
          res.writeHead 200, {'Content-Type': mime_types[path_tools.extname('icons.png')] or 'image/png'}
          return res.end new Buffer(unescape(escaped_icons_png), 'binary')
        if unescape(pathname).match(/pixel\.gif$/)
          res.writeHead 200, {'Content-Type': mime_types[path_tools.extname('pixel.gif')] or 'image/gif'}
          return res.end new Buffer(unescape(escaped_pixel_gif), 'binary')
        else
          dispatch._404 null, res, pathname

# CGI support (improved on 2012-09-07, further fixes on 2013-08-03)
    
  dispatch.getEnv = (pathname, req, res) ->
    env = {}
    
    #env['REQUEST_HEADERS'] = JSON.stringify(req.headers)
    #env['REQUEST_CONNECTION'] = req.connection.toString()
    
    for key, val of req.headers
      env["HTTP_#{key.toUpperCase().replace('-', '_')}"] = req.headers[key]
    query_pairs = ("#{key}=#{val}" for key, val of req.get)
    if query_pairs.length isnt 0
      env["QUERY_STRING"] = "#{query_pairs.join('&')}"
    else
      env['QUERY_STRING'] = ''
    env['REMOTE_ADDRESS'] = dispatch.getRemoteAddress(req)
    env['REQUEST_URI'] = pathname
    env['GATEWAY_INTERFACE'] = "CGI/1.1"
    env['SERVER_NAME'] = req.headers.host.split(':')[0]
    env['SERVER_ADDRESS'] = env['SERVER_NAME']
    env['SERVER_SOFTWARE'] = "#{dispatch.software_name}/#{dispatch.version}"
    env['SERVER_PROTOCOL'] = "HTTP/#{req.httpVersion}"
    env['SERVER_PORT'] = req.headers.host.split(':')[1] or 80
    env['REQUEST_METHOD'] = req.method
    env['SCRIPT_NAME'] = pathname
    env['SCRIPT_FILENAME'] = "#{dispatch.static_route}#{unescape(pathname)}"
    
    if dispatch.serve_php
      env['REDIRECT_STATUS'] = '200'
      
    env

  dispatch.cgi = (pathname, req, res) ->
    urlobj = urlparse req.url
    #dispatch.log JSON.stringify urlobj unless not dispatch.logging
    
    respbuffer = ''
    full_path = "#{dispatch.static_route}#{unescape(pathname)}"

    env = dispatch.getEnv pathname, req, res
    
    isPHP =  !!pathname.match(/\.php$/)
    
    prepareChild = (req_body) ->
      if req_body and isPHP
        if not env['QUERY_STRING']
          env['QUERY_STRING'] = ''
        env['QUERY_STRING'] += req_body
        
      if isPHP
        if not dispatch.serve_php
          dispatch._405(null, res, pathname, "PHP scripts not allowed")
          return null
        else
          dispatch.log "Spawning #{dispatch.php_cgi} #{full_path}" unless not dispatch.logging
          child = spawn(dispatch.php_cgi, [full_path], env: env)
      else 
        dispatch.log "Spawning #{full_path}" unless not dispatch.logging
        child = spawn(full_path, [], env: env)

      child.stderr.pipe process.stderr

      child.stdout.on 'data', (data) ->
        arrdata = data.toString().split('\n')
        for elem in arrdata
          if (elem.substr(0,8).toLowerCase() isnt "content-")
            respbuffer += elem
          else
            pair = elem.split(/:\s+/)
            try
              res.setHeader(pair[0], pair[1])
            catch e
              dispatch.log "Error setting response header: #{e.message}" unless not dispatch.logging
      
      child.stdout.on 'end', (moredata) ->
        try
          respbuffer += moredata unless not moredata
          res.end respbuffer
        catch e
          dispatch.log "Error terminating response: #{e.message}" unless not dispatch.logging
    
      return child
      
    body = []
    if req.method.toLowerCase() is "post"
      req.on 'data', (chunk) ->
        body.push(chunk)
      req.on 'end', ->
        body = body.join ''
        req.post = _bodyparser body
        req.body = _extend req.body, req.post
        try
          data = querystring.stringify(req.body)
          #dispatch.log "Data to be posted: #{data}" unless not dispatch.logging
          child = prepareChild(data)
          return if not child 
          d = domain.create()
          d.add child.stdin
          d.on 'error', (err) -> dispatch.log "Child process input error (captured by domain): #{err.message}" unless not dispatch.logging
          d.run(-> child.stdin.write("#{data}\n"); child.stdin.end()) 
        catch e
          dispatch.log "Child process input error: #{e.message}" unless not dispatch.logging
    else
      try
        data = querystring.stringify(req.body)
        dispatch.log "Data to be sent: #{data}" unless not dispatch.logging 
        #child.stdin.write("#{json}\n", "utf8", ((err) -> console.log "ERROR in STDIN" if err)) unless child.stdin._writeableState.ended
        child = prepareChild()
        return if not child
        d = domain.create()
        d.add child.stdin
        d.on 'error', (err) -> dispatch.log "Child process input error (captured by domain): #{err.message}" unless not dispatch.logging
        d.run(-> child.stdin.write("#{data}\n"); child.stdin.end()) 
      catch e
        dispatch.log "Child process input error: #{e.message}" unless not dispatch.logging
    
    0

# End of CGI support

#SCGI Support

  dispatch.sendSCGIRequest = (request, sock) ->
    if request.method.toLowerCase() is 'post'
      encPost = querystring.stringify(request.post)
    else
      encPost = ""
    req = ""
    req += "CONTENT_LENGTH\0#{encPost.length}\0"
    req += "REQUEST_METHOD\0#{request.method}\0"
    req += "REQUEST_URI\0#{request.url}\0"
    req += "QUERY_STRING\0#{querystring.stringify request.get}\0"
    req += "CONTENT_TYPE\0#{request.headers['content-type'] or 'text/plain'}\0"
    req += "DOCUMENT_URI\0#{request.url}\0"
    req += "DOCUMENT_ROOT\0#{'/'}\0"
    req += "SCGI\u0000\u0031\u0000"
    req += "SERVER_PROTOCOL\0HTTP/1.1\0"
    #req += "HTTPS\0#{'$https if_not_empty'}\0"
    req += "REMOTE_ADDR\0#{dispatch.getRemoteAddress(request)}\0"
    req += "REMOTE_PORT\0#{request.connection.remotePort}\0"
    req += "SERVER_PORT\0#{request.headers['host'].match(/:(\d+)$/)[1] or '80'}\0"
    req += "SERVER_NAME\0#{request.headers['host'].replace /:\d+/, ''}\0"
    for key, val of request.headers
      req += "HTTP_#{key.toUpperCase().replace('-', '_')}\0#{request.headers[key]}\0"
    
    req = "#{req.length}:#{req},#{encPost}"
    dispatch.log "Sending '#{req}' of length #{req.length} to SCGI" if dispatch.logging
    sock.write(req)
    #sock.end()
  
  dispatch.scgi_pass = (conn, request, response) ->
    if not isNaN(parseInt(conn))
      conn_options = port: parseInt(conn)
    else
      conn_options = path: conn
    
    getData = ->
      retval = ""
      client = net.connect(
        #path: '/tmp/hello_scgi_py.sk'
        #port: 26000
        conn_options
        ->
          dispatch.sendSCGIRequest(request, client)
      )
      client.on 'data', (data) ->
        retval += data
      client.on 'end', (data) ->
        retval += data if data
        dispatch.log "Ending SCGI transaction"
        retval = retval.replace /\r/g, ''
        lines = retval.split '\n'
        statusDone = false
        contentTypeDone = false
        headerSet = false
        status = 0
        contentType = ''
        for line, index in lines
          dispatch.log "LINE ##{index + 1}: #{line}"
          if not headerSet
            writeThis = true
            if not statusDone
              m = line.match /Status: (\d+)/i
              if m
                writeThis = false
                statusDone = true
                status = m[1]
                dispatch.log "Detected status: #{status}"
                if contentTypeDone
                  dispatch.log "Response: Status #{status}  - Content-Type: #{contentType}"
                  response.writeHead status, 'Content-Type': contentType or 'text/plain'
                  headerSet = true
            if not contentTypeDone
              m = line.match /Content\-Type\:\s+(.+\/.+)/i
              if m
                writeThis = false
                contentTypeDone = true
                contentType = m[1]
                dispatch.log "Detected Content-Type: #{contentType}"
                if statusDone
                  dispatch.log "Response: Status #{status}  - Content-Type: #{contentType}"
                  response.writeHead status, 'Content-Type': contentType or 'text/plain'
                  headerSet = true
             if writeThis
               response.write line
          else
            response.write line
         
        response.end()
 
    d = domain.create()
    d.on 'error', (e) ->
      response.writeHead 502, 'Bad gateway', 'Content-Type': 'text/plain'
      response.end "502 - Bad gateway\n\n\n#{e.message}"
    d.run getData

# End of SCGI support

  dispatch.proxy_pass = (url, response) ->
    try
      http.get url, (res) ->
        res.pipe response
    catch e
      dispatch._500 null, response, url, e.message

  dispatch.dir_list = (fpath, path) ->
    "Returns a promise that will resolve to a couple of arrays: filenames and stats"
    d = dispatch.utils.defer()
    p = d.promise()

    fs.readdir fpath, (err, files) ->
      if err
        d.reject err
      else
        d.resolve files

    p.then(
      (files) ->
        stats = []
        d = dispatch.utils.defer()
        len = files.length - 1

        files.forEach (file, index) ->
          fs.stat "#{fpath}#{path_tools.sep}#{file}", (err, stat) ->
            if err
              d.reject err
            else
              stats[index] = stat
              if index is len
                d.resolve [files, stats]
        d.promise()
      (err) ->
        dispatch.log "Error reading directory: #{err.message}" if dispatch.logging
    )


  dispatch.gallery = (fpath, path, res, list_func = dispatch.dir_list) ->
    context =
      revealjs: '//cdnjs.cloudflare.com/ajax/libs/reveal.js/2.6/js/reveal.min.js'
      css: '//cdnjs.cloudflare.com/ajax/libs/reveal.js/2.6/css/reveal.min.css'
      theme: '//cdnjs.cloudflare.com/ajax/libs/reveal.js/2.6/css/theme/default.css'
      cwd: if path is "." then "/" else path
    list_func(fpath, path)
    .then(
      (arrs) ->
        [files] = arrs
        context.files = (url: "#{path}/#{querystring.escape(file)}" for file in files when mime_types[path_tools.extname file.toLowerCase()]?.match /image/)
        if context.files.length isnt 0
          res.writeHead 200, {'Content-type': 'text/html'}
          res.end dispatch.render_template dispatch._gallery_template, context
        else
          # call directory method.
          dispatch.static_directory fpath, path, res, list_func
      (err) ->
        return dispatch._404(null, res, path)
    )

  dispatch.directory = (fpath, path, res, list_func = dispatch.dir_list) ->
    # hack in case someone tinkers with gallery, directory the way guaycuru does.  
    dispatch.static_directory fpath, path, res, list_func

  dispatch.static_directory = (fpath, path, res, list_func = dispatch.dir_list) ->
    context = cwd: if path is "." then "/" else path
    context.isRoot = path is "."
    context.parent = path_tools.dirname(path)
    context.icondir = dispatch.stock_icons.directory()

    list_func(fpath, path)
    .then(
      (arrs) ->
        [files, stats] = arrs
        #context.cwd_contents = ({path: path, querystring_escaped_file: querystring.escape(file), file: file} for file in files)
        context.cwd_contents = []
        for file, index in files
          obj = path: path
          obj.querystring_escaped_file = querystring.escape(file)
          obj.file = file
          obj.size = thousandSep(stats[index].size)
          obj.class = if stats[index].isDirectory() then "directory" else "file"
          obj.order = if stats[index].isDirectory() then 1 else 0
          obj.icon = if stats[index].isDirectory() then dispatch.stock_icons.directory() else dispatch.stock_icons.file()
          context.cwd_contents.push obj
        context.cwd_contents.sort (o1, o2) -> o2.order - o1.order
        resp = dispatch.render_template dispatch._dirlist_template, context
        res.writeHead 200, {'Content-type': 'text/html'}
        res.end resp
        resp
      (err) ->
        return dispatch._404(null, res, path)
    )


  dispatch.get = (pattern, callback) ->
    _pushRoute pattern, callback, 'get'

  dispatch.post = (pattern, callback) ->
    _pushRoute pattern, _make_request_wrapper(callback), 'post'

  dispatch.put = (pattern, callback) ->
    _pushRoute pattern, _make_request_wrapper(callback), 'put'
  
  dispatch.patch = (pattern, callback) ->
    _pushRoute pattern, _make_request_wrapper(callback), 'patch'

  dispatch.delete = (pattern, callback) ->
    _pushRoute pattern, callback, 'delete'

  dispatch.del = (pattern, callback) ->
    _pushRoute pattern, callback, 'delete'

  dispatch.any = (pattern, callback) ->
    _pushRoute pattern, _make_request_wrapper(callback), 'any'



  dispatch._404 = (req, res, path) ->
    res.writeHead(404, {'Content-Type': 'text/html'})
    default_resp = """
                   <h2>404 - Resource #{path} not found at this server</h2>
                   <hr/><h3>Served by #{dispatch.served_by} v#{dispatch.version}</h3>
                   <p style="text-align: center;"><button onclick='history.back();'>Back</button></p>
                   """
    fs.exists "#{dispatch.static_route}#{path_tools.sep}404.html", (exists) ->
      if not exists
        res.end default_resp
      else
        fs.readFile "#{dispatch.static_route}#{path_tools.sep}404.html", encoding: 'utf8', (err, data) ->
          if err
            res.end default_resp
          else
            res.end data

  dispatch._405 = (req, res, path, message) ->
    res.writeHead(405, {'Content-Type': 'text/html'})
    res.end("""
#               <h2>405 - Resource #{path}: #{message}</h2>
                <hr/><h3>Served by #{dispatch.served_by} v#{dispatch.version}</h3>
                <p style="text-align: center;"><button onclick='history.back();'>Back</button></p>
            """)

  dispatch._500 = (req, res, path, message) ->
    res.writeHead(500, {'Content-Type': 'text/html'})
    res.end("""
                <h2>500 - Internal server error at #{path}: #{message}</h2>
                <hr/><h3>Served by #{dispatch.served_by} v#{dispatch.version}</h3>
                <p style="text-align: center;"><button onclick='history.back();'>Back</button></p>
            """)

  dispatch.render_template = (template_string, context, keep_tokens = false) ->
    "Naive regex based implementation of mustache.js spec"

    html_encode = (stri) ->
      String(stri).replace /[<>&'\/"]/g, (m) -> {"<": "&lt;", ">": "&gt;","&": "&amp;","'": "&#39;","/": "&#x2F;","\"": "&quot;"}[m]

    section_pattern = /\{\{(\#|\^)\s*([\w\W]+?)\}\}\n?([\w\W]*?)\n?\{\{\/\s*\2\}\}/
    section_pattern_global = /\{\{(\#|\^)\s*([\w\W]+?)\}\}\n?([\w\W]*?)\n?\{\{\/\s*\2\}\}/g
    variable_pattern = /\{{2}([\w\W]+?)\}{2}/    
    variable_pattern_global = /\{{2}([\w\W]+?)\}{2}/g

    section_tokens = []
    text_tokens = []
    
    stripped_string = template_string
    sections = template_string.match section_pattern_global
    if sections
      for section in sections
        stripped_string = stripped_string.replace(section, '\n@@@\n')
        section_tokens.push(section.match(section_pattern))
      text_tokens = stripped_string.split /\n@@@\n/g
    else
      text_tokens.push template_string
      
    for text_token, index in text_tokens
      variable_tokens = text_token.match variable_pattern_global
      token_obj = {}
      if variable_tokens
        for token in variable_tokens
          k = token.replace(/\{/g, '').replace(/\}/g, '').trim()
          dont_encode = false
          if k[0] is "&"
            k = k.substring(1).trim()
            dont_encode = true
          token_obj[k] = {token: token, dont_encode: dont_encode}

      new_str = text_token
      for key, value of token_obj
        try
          replacement = eval "context.#{key}"
        catch e
          replacement = null
        if replacement
          new_str = new_str.replace new RegExp(token_obj[key].token, 'g'), if token_obj[key].dont_encode then replacement else html_encode(replacement)
      #Erase unmatched mustaches
      new_str = new_str.replace variable_pattern_global, '' unless keep_tokens
      text_tokens[index] = new_str

    ret_arr = []
    for text, index in text_tokens
      ret_arr.push text
      if section_tokens[index]
        property = context[section_tokens[index][2].trim()]
        full_text = section_tokens[index][0]
        section_type = if section_tokens[index][1] is "#" then true else false
        section_inner_text = section_tokens[index][3]

        if section_type is false
          ret_arr.push(dispatch.render_template(section_inner_text, context)) if not property
        else
          if property?.length? and (property?.constructor.name isnt "String")
            for item in property
              ret_arr.push(dispatch.render_template(section_inner_text, item))
          else
            ret_arr.push(dispatch.render_template(section_inner_text, context)) if property
    
    ret_arr.join '\n'

  dispatch.render_template_file = (file_name, context, cb = ((exists, text)-> [exists, text]), keep_tokens = false) ->
    fs.exists file_name, (exists) ->
      if not exists
        return cb(exists, "Template file does not exist.")
      tf = fs.createReadStream file_name, 'utf8'
      ss = new dispatch.utils.StringStream
      ss.on 'finish', ->
        ss.transform (text) ->
          dispatch.render_template(text, context, keep_tokens)
        cb exists, '' + ss
      tf.pipe ss

  dispatch.utils = {}
  dispatch.utils.uuid = uuid
  dispatch.utils.isEmptyObj = _isEmpty
  dispatch.utils.extendObj = _extend
  dispatch.utils.parsePattern = _parsePattern
  dispatch.utils.mime_types = mime_types
  dispatch.utils.favicon = escaped_favicon
  dispatch.utils.cookie2obj = dispatch.cookie2obj
  dispatch.utils.obj2cookie = dispatch.obj2cookie
  dispatch.utils.getCookie = dispatch.getCookie
  dispatch.utils.setCookie = dispatch.setCookie
  dispatch.utils.getEnv = dispatch.getEnv
  dispatch.utils.async = require './async'
  dispatch.utils.defer = require('./promises').defer
  dispatch.utils.StringStream = require './stringstream'
  dispatch.utils.get_icon = dispatch.get_icon
  dispatch.utils.stock_icons = dispatch.stock_icons


  # End of Dispatch function properties and methods


  # Returns dispatch (router function)

  dispatch


# Exports "router factory function"
module.exports = Router
